/**************************************************
 * RCSId: $Id$
 *
 * Old HRI reader
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log$
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 R. Alblas. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/

#include <stdio.h>
#include <math.h>
#include <malloc.h>
#include <time.h>
#include <string.h>
#include "wsat.h"
#include "hri1.h"

/* ==== Header functions ==== */


/*************************************
 * Extract header subframe info  
 *************************************/
void hri_hdrsf2hri_pci(HRI_SUBFRHDR *hri_sf,        /* frame header */
                       HRI_PICINFO  *hri_pci,       /* hri specific picinfo */
                       PICINFO      *pci)           /* picinfo */
{
  hri_satind2satname(hri_sf->sat_ind,hri_pci->satname);
  pci->picture_time.tm_year=hri_sf->year_im_acq-1900;
  pci->picture_time.tm_yday=hri_sf->day_im_acq-1;
  pci->picture_time.tm_hour=hri_sf->image_time/100;
  pci->picture_time.tm_min=hri_sf->image_time-pci->picture_time.tm_hour*100;
  pci->picture_time.tm_sec=0;
  yday2mday_mon(&pci->picture_time);
  mktime_ntz(&pci->picture_time);
}

/*************************************
 * Extract info from frame header 
 *************************************/
int hri_sf2hri_pci(HRI_SUBFRHDR *hri_sf,        /* frame header */
                   HRI_PICINFO  *hri_pci,       /* hri specific picinfo */
                   PICINFO      *pci)           /* picinfo */
{
  int i;
/* Determine some subframe characteristics */
  hri_pci->subfrm_p_line=0;
  for (i=1; i<NR_HRI_CHAN; i++)                 /* VIS1/VIS2/IR/WV */
  {
    if (pci->channels_recorded[i])
      hri_pci->subfrm_p_line++;                   /* subframes per line */

    if (hri_pci->channel_info[i].dbl_res)
      hri_pci->subfrm_p_line++;                   /* double resolution */
  }

  hri_pci->frm_p_subfrm=hri_sf->frames_per_subframe;

  hri_pci->first_subfr=hri_sf->cur_subfr;
  hri_pci->first_line_nr=hri_sf->line_nr;
  if (!hri_pci->frm_p_subfrm) hri_pci->frm_p_subfrm=8;
  if (hri_pci->frm_p_subfrm==4) hri_pci->start_pic=36;
  if (hri_pci->frm_p_subfrm==8) hri_pci->start_pic=68;

/* Determine picture sizes */
  for (i=1; i<NR_HRI_CHAN; i++)              /* VIS1/VIS2/IR/WV */
  {
    hri_pci->channel_info[i].width=0;
    hri_pci->channel_info[i].height=0;
    if (pci->channels_recorded[i])
    {
      hri_pci->channel_info[i].height=
         (hri_sf->nr_subfr-hri_pci->first_subfr)/hri_pci->subfrm_p_line;
      if ((i==1) && (hri_pci->channel_info[i+1].dbl_res)) 
        hri_pci->channel_info[i].height*=2;

      if (hri_pci->frm_p_subfrm==4)
        hri_pci->channel_info[i].width=1250;
      if (hri_pci->frm_p_subfrm==8)
        hri_pci->channel_info[i].width=2500;
      if (hri_pci->channel_info[i].dbl_res)
        hri_pci->channel_info[i].width*=2;
    }
  }


  switch(hri_sf->form_ind)
  {
    case 0x00: hri_pci->format_ind='A';
    case 0xff: hri_pci->format_ind='B';
    case 0x0f: hri_pci->format_ind='X';
    default  : hri_pci->format_ind=0;
  }


/* Determine scan direction */
  switch(hri_sf->scan_dir)
  {
    case 0x00: pci->scan_hor=E_W; pci->scan_ver=S_N; break;
    case 0xf0: pci->scan_hor=E_W; pci->scan_ver=N_S; break;
    case 0x0f: pci->scan_hor=W_E; pci->scan_ver=S_N; break;
    case 0xff: pci->scan_hor=W_E; pci->scan_ver=N_S; break;
  }

  return 0;
}


/*************************************
 * Read a frame from file
 * Return value:
 *   0: read failure (EOF?)
 *   MASK_HRI1_READ_OK_PRE-bit set ==> read frame OK
 *   MASK_HRI1_SYNC_OK_PRE-bit set ==> sync current frame OK
 *   MASK_HRI1_SYNC_OK_POST    set ==> sync next frame still OK
 *   MASK_HRI1_REC_OK_PRE      set ==> No decoder error current frame
 *   MASK_HRI1_REC_OK_POST     set ==> No decoder error next frame
 *   100% OK ==> return = 0xff
 *************************************/
int read_hri1_frame(guchar *fr,FILE *fp)
{
  HRI_SUBFRHDR *hri_sf=(HRI_SUBFRHDR *)fr;
  guchar fh[7];
  int ret=MASK_HRI1_READ_OK_PRE;
  long file_pos;

/* Read bytes between 2 frames. */
  if (!(fread(fh,7,1,fp))) return 0;         /* spacing between 2 frames */

/* Determine pre-status flags. */
  if (fh[4]=='E') ret|=MASK_HRI1_REC_OK_PRE; /* No during record */ 

/* Read 1 frame */
  if (!(fread(fr,LEN_HRIFRAME,1,fp))) return 0;

/* Check sync. If one of the 3 syncwords is OK, assume that's OK. */
  if (fr[0]==SYNC1) ret|=MASK_HRI1_SYNC_OK_PRE;
  if (fr[1]==SYNC2) ret|=MASK_HRI1_SYNC_OK_PRE;
  if (fr[2]==SYNC3) ret|=MASK_HRI1_SYNC_OK_PRE;

/* Error correction channel indication. */
  if (!(fr[3]&0xf))
  {
    int i,m,nr1;
    for (i=0; i<4; i++)
    {
/* Determine # ones 4 LSB's */
      nr1=0;
      for (m=0x1; m<=0x8; m<<=1) 
        if (hri_sf->spectr_ind[i]&m) nr1++;

/* # ones > 2 ==> 0xf; else: 0x0 */
      if (nr1>2)
        hri_sf->spectr_ind[i]|=0x0f;
      else
        hri_sf->spectr_ind[i]&=0xf0;

/* Determine # ones 4 MSB's */
      nr1=0;
      for (m=0x10; m<=0x80; m<<=1) 
        if (hri_sf->spectr_ind[i]&m) nr1++;

/* # ones > 2 ==> 0xf; else: 0x0 */
      if (nr1>2)
        hri_sf->spectr_ind[i]|=0xf0;
      else
        hri_sf->spectr_ind[i]&=0x0f;
    }
  }


  file_pos=ftell(fp);

/* Read bytes between 2 frames. */
  if (!(fread(fh,7,1,fp))) return ret;

/* Determine post-status flags. */
  if (fh[4]=='E') ret|=MASK_HRI1_REC_OK_POST;

/* Check sync. If one of the 3 syncwords is OK, assume that's OK. */
  if (fread(fh,3,1,fp))                               /* vlgnde header */
  {
    if (fh[0]==SYNC1) ret|=MASK_HRI1_SYNC_OK_POST;
    if (fh[1]==SYNC2) ret|=MASK_HRI1_SYNC_OK_POST;
    if (fh[2]==SYNC3) ret|=MASK_HRI1_SYNC_OK_POST;
  }


/* Set file pointer at end previous frame */
  fseek(fp,file_pos,SEEK_SET);

  return ret;
}

/*************************************
 * Search start of (next) picture and extract info.
 * Header subframes contain: SYNC[3], ID[1], LABEL[24], IDENT[32]
 * Data subframes contain  : SYNC[3], ID[1], LABEL[24]
 *************************************/
#define LEN_COMPARE 60
int find_picture(PICINFO *pci,guchar *fr,FILE *fp)
{
  HRI_PICINFO *hri_pci=&pci->hri_pci;

  HRI_SUBFRHDR *hri_sf=(HRI_SUBFRHDR *)fr;
  guchar fr_prev[LEN_COMPARE];
  HRI_SUBFRHDR *hri_sf_prev=(HRI_SUBFRHDR *)fr_prev;

  enum { PRECATCH_HEADERSUBFRM,CATCH_HEADERSUBFRM,
         PRECATCH_DATASUBFRM,CATCH_DATASUBFRM,
         CATCH_READY } pos_frame=PRECATCH_HEADERSUBFRM;
  int rslt;
  guchar chan[NR_HRI_CHAN];
  int i;

  int hdrsubfr_nr=0;
  int datasubfr_nr=0;

/* Clear hri_pci */
  memset(chan,0,NR_HRI_CHAN*sizeof(guchar));
  *hri_pci->satname=0;

/* Search reliable frame info by trying to find 
   6 frames in a row with identical label
   wihout detected errors.
   Also, determine which channels are present. 
   (For this at least 4 frames are needed.)
*/
  while ((rslt=read_hri1_frame(fr,fp)))
  {
    if (rslt!=0xff) continue;               /* Detected error in frame */

/* Translate 2-byte and 4-byte words */
/*(NOTE: How portable is 'guchar to struct'??? */
    hri_sf->frames_per_subframe=GUINT16_FROM_BE(hri_sf->frames_per_subframe);
    hri_sf->nr_subfr           =GUINT16_FROM_BE(hri_sf->nr_subfr);
    hri_sf->cur_subfr          =GUINT16_FROM_BE(hri_sf->cur_subfr);
    hri_sf->line_nr            =GUINT16_FROM_BE(hri_sf->line_nr);
    hri_sf->image_nr           =GUINT32_FROM_BE(hri_sf->image_nr);

    hri_sf->sat_ind            =GUINT16_FROM_BE(hri_sf->sat_ind);
    hri_sf->year_im_acq        =GUINT16_FROM_BE(hri_sf->year_im_acq);
    hri_sf->day_im_acq         =GUINT16_FROM_BE(hri_sf->day_im_acq);
    hri_sf->image_time         =GUINT16_FROM_BE(hri_sf->image_time);

/* Skip if this is not first frame of subframe. */
    if ((hri_sf->id!=0x30) && (hri_sf->id!=0x70)) 
      continue;

/* Check frames_per_subframe */
    if ((hri_sf->frames_per_subframe!=4) && 
        (hri_sf->frames_per_subframe!=8)) 
      continue;

/* State machine to determine location in data */
/*
   cur_subfr=111111101234567...... 
                    ^
                    |__Start picture
*/
    switch(pos_frame)
    {
      case PRECATCH_HEADERSUBFRM:
        if (hri_sf->cur_subfr==1) hdrsubfr_nr++; else hdrsubfr_nr=0;
/* Compare bytes 0-7 of IDENT with previous subframe */
        if (memcmp(fr+36,fr_prev+36,8)) hdrsubfr_nr=0;

        if (hdrsubfr_nr>4)      /* Reliable header subframe */
          pos_frame=CATCH_HEADERSUBFRM;

/* Next is needed if header subframe is missing (start recording too late) */
        if (hri_sf->cur_subfr==hri_sf_prev->cur_subfr+1)
          datasubfr_nr++;
        else
          datasubfr_nr=0;

        if (datasubfr_nr>4)      /* Reliable data subframe */
          pos_frame=CATCH_DATASUBFRM;
      break;
      case CATCH_HEADERSUBFRM:
        pos_frame=PRECATCH_DATASUBFRM;
      break;
      case PRECATCH_DATASUBFRM:
        if (hri_sf->cur_subfr==hri_sf_prev->cur_subfr+1)
          datasubfr_nr++;
        else
          datasubfr_nr=0;
/* Compare except bytes 4,5=#subframe; 6,7=image linenr */
        if (memcmp(fr+4,fr_prev+4,4)) datasubfr_nr=0;

        if (memcmp(fr+4+8,fr_prev+4+8,5)) datasubfr_nr=0;

        if (datasubfr_nr>4)      /* Reliable data subframe */
          pos_frame=CATCH_DATASUBFRM;
      break;
      case CATCH_DATASUBFRM:
        if (hri_sf->cur_subfr==hri_sf_prev->cur_subfr+1)
        {
          datasubfr_nr++;
        }
        else
        {
          datasubfr_nr=0;
          pos_frame=PRECATCH_DATASUBFRM;
        }
        if (datasubfr_nr>10)      /* Ready */
          pos_frame=CATCH_READY;
      break;
      default:
      break;
    }

/* actions */
    switch(pos_frame)
    {
      case CATCH_HEADERSUBFRM:
        hri_hdrsf2hri_pci(hri_sf,hri_pci,pci);
      break;
      case PRECATCH_DATASUBFRM:
        for (i=1; i<NR_HRI_CHAN; i++)
        {
          pci->channels_recorded[i]=FALSE;
          hri_pci->channel_info[i].dbl_res=FALSE;
        }
      break;

      case CATCH_DATASUBFRM:
        for (i=1; i<NR_HRI_CHAN; i++)
        {
          switch(hri_sf->spectr_ind[i-1])
          {
            case 0xff: pci->channels_recorded[i]=TRUE;        break;
            case 0xf0: pci->channels_recorded[i]=TRUE;
                       hri_pci->channel_info[i].dbl_res=TRUE; break;
            case 0x0f: pci->channels_recorded[i]=TRUE; 
                       hri_pci->channel_info[i].dbl_res=TRUE; break;
            case 0x00:                                        break;
            default  : pos_frame=PRECATCH_DATASUBFRM;         break;
          }
        }
      break;
      case CATCH_READY:
        hri_sf2hri_pci(hri_sf,hri_pci,pci);
      break;
      default:
      break;
    }
    for (i=0; i<LEN_COMPARE; i++) fr_prev[i]=fr[i];
    if (pos_frame==CATCH_READY) break;
  }

  return 1;
}


/*************************************
 * Read hri header and init PICINFO struct
 * return: 'r' if read error
 *         'p' if no picture found
 *************************************/
int read_hri1_hdr(PICINFO *pci)
{
  FILE *fp=pci->fp;
  HRI1_FILEHDR hrihdr;
  HRI_PICINFO *hri_pci=&pci->hri_pci;

  guchar frame[LEN_HRIFRAME];
  int i;

  rewind(fp);

/* Reset channel flags */
  for (i=0; i<NR_CHANNELS_MAX; i++) pci->channels_recorded[i]=FALSE;

/* Read file header */
  if (!(fread(&hrihdr,sizeof(hrihdr),1,fp))) return 'r'; /* read header */

/* Search picture, and get characteristics of it */
  if (!(find_picture(pci,frame,fp))) return 'p';

/* Copy size info into picinfo structure */
  pci->width=0;
  pci->height=0;
  pci->depth=8;

  pci->ox=0;
  pci->oy=0;

/* Copy color map into picinfo structure */
  pci->clrmap.size=256;
  for (i=0; i<pci->clrmap.size; i++)
  {
    pci->clrmap.red[i]=i;
    pci->clrmap.green[i]=i;
    pci->clrmap.blue[i]=i;
  }


/* Start of picture here */
  pci->pic_strt=ftell(fp);

/* Determine frame info */
  pci->fri.words_per_frame=hri_pci->frm_p_subfrm*364;
  pci->fri.words_per_line=0;
  for (i=1; i<NR_HRI_CHAN; i++)
  {
    if ((pci->channels_recorded[i]))
      pci->fri.words_per_line+=pci->fri.words_per_frame;
    if ((pci->hri_pci.channel_info[i].dbl_res))
      pci->fri.words_per_line+=pci->fri.words_per_frame;
  }
  pci->fri.header_len=0;
  pci->fri.leader_len=0;

  pci->fri.bytes_per_line=pci->fri.words_per_line/364*(364+7);
  pci->fri.inter_chan=1;

  strcpy(pci->sat_name,hri_pci->satname);
  return 0;
}

/*************************************
 * Calculate some hri-based frame numbers
 *************************************/
FRAMEINFO calc_hriframeinfo()
{
  FRAMEINFO fri;
  fri.header_len=0;
  fri.leader_len=0;
  fri.inter_chan=1;
  fri.words_per_frame=LEN_HRIFRAME;
  fri.bytes_per_line=LEN_HRIFRAME;

  return fri;
}

/* ==== Line extraction functions ==== */

/*************************************
 * Read  hri1 line.
 * Return: Quality level (see read_hri1_frame)
 *************************************/
int read_hri1_line(PICINFO *pci,VIEWINFO *vwi,int y,guint16 *line,guchar *grid)
{
  int rslt;
  do
  {
    int odd_y=0;
    if ((vwi->channel_selected[1]) && (pci->hri_pci.channel_info[1].dbl_res))
    {
/* channel 1 = odd line, channel 2 even. So, if y=odd take channel 1 etc. */
      odd_y=y%2;
      y=y/2;
    }
    else
      if (vwi->channel_selected[0]) odd_y=1;  /* for frame info only */

    fseek(pci->fp,pci->pic_strt + y*pci->fri.bytes_per_line,SEEK_SET);
    rslt=get_hri1_nextline(pci,vwi,odd_y,line,grid);

    y=y+(vwi->scan_ver==S_N? (-1) : (1));   /* incr or decr line nr. dep. on scan dir. */
    if (y<0) break;                         /* stop if BOF */

    if ((!vwi->skip_faulty_lines) || (rslt&MASK_HRI1_SYNC_OK_POST)) break;

  } while (rslt);                           /* stop if EOF */
  return rslt;
}

/*************************************
 * Get next hri1 line.
 * Return: Quality level (see read_hri1_frame)
 *************************************/
int get_hri1_nextline(PICINFO *pci,VIEWINFO *vwi,int odd_y,
                      guint16 *line,guchar *grid)
{
  FILE *fp=pci->fp;
  guchar frame[LEN_HRIFRAME];                         /* Store frame */
  HRI_SUBFRHDR *frm_hdr=(HRI_SUBFRHDR *)frame;
  int i;
  int fr_p_subfr;
  int subfr_p_line;
  int nr_dpsfr;
  int chan_nr;
  int subfr,fr;
  int enddata;
  int ox;
  int rslt,rslt2;
  int pixels_p_fr;
  int pixels_p_first_fr;
  int first_pixel;
  int nr_pixels;

/* Get just 1 frame. This is for showing header info only. */
  if (vwi->channel_selected[0])
  {
    read_hri1_frame(frame,fp);
    for (i=0; i<LEN_HRIFRAME; i++)
    {
      line[i]=frame[i];
    }
    return 1;
  }

/* Determine channel number */
  for (chan_nr=1; chan_nr<=4; chan_nr++)
    if (vwi->channel_selected[chan_nr]) break;
  if (chan_nr>4) return 0; 

/* Init */
  fr_p_subfr=pci->hri_pci.frm_p_subfrm;
  nr_dpsfr=(fr_p_subfr==8? 2500 : 1250);
  first_pixel=pci->hri_pci.start_pic;
  enddata=(fr_p_subfr==8? 44 : 202);
  subfr_p_line=(pci->hri_pci.channel_info[chan_nr].dbl_res? 2 : 1);
  pixels_p_fr=LEN_HRIFRAME-4;
  pixels_p_first_fr=LEN_HRIFRAME-pci->hri_pci.start_pic;

/* Search start sub-frame containing channel to display */
  while ((rslt=read_hri1_frame(frame,fp)))
  {
/* Go on if it is not the first frame of a subframe */
    if (frm_hdr->id&0x0f) continue;

/* Now we have the first frame of a subframe.
   Check if it is the right subframe (right channel, right even/odd line) 
   This subframe has the right code in "spectr_ind[channel_selected-1]":
   SINGLE_SUBFRAME or FIRST_SUBFRAMEin case of 2 subframes/line
*/
    switch(chan_nr)
    {
/* Selected IR or WV channel */
      case 3: case 4:
        if (frm_hdr->spectr_ind[chan_nr-1]==NO_SUBFRAME)
          continue;                                     /* Wrong channel */
        if (frm_hdr->spectr_ind[chan_nr-1]==SECOND_SUBFRAME)
          continue;                                     /* Wrong sub-frame */
      break;

/* Selected VIS channel; chan_nr is now 1 or 2 */
      default:
        if ((frm_hdr->spectr_ind[0]==NO_SUBFRAME) &&
            (frm_hdr->spectr_ind[1]==NO_SUBFRAME))
          continue;                                     /* Wrong channel */

        if ((frm_hdr->spectr_ind[0]==SECOND_SUBFRAME) ||
            (frm_hdr->spectr_ind[1]==SECOND_SUBFRAME))
          continue;                                     /* Wrong sub-frame */

/* Make sure that a odd y will catch a odd line. 
   To enforce that lines are always shown in the correct order.
   Because odd_y=y%2 and then y=y/2, for one y:
     first line has odd_y=0 
     second line has odd_y=1
   So, first spectr_ind[0] then [1]
   If just 1 VIS-channel is selected: Make sure that odd_y has correct value: 0 if spectr_ind[0]
*/
        if ((frm_hdr->spectr_ind[0]!=NO_SUBFRAME) && (odd_y)) continue;
        if ((frm_hdr->spectr_ind[1]!=NO_SUBFRAME) && (!odd_y)) continue;
      break;
    }

/* Start right subframe found */
    break;
  }

  if (!rslt) return 0;  /* rslt==0 -> read error */

/* Now, collect all pieces of the desired line */

  for (subfr=0; subfr<subfr_p_line; subfr++)
  {
    for (fr=0; fr<fr_p_subfr; fr++)
    {
      if ((subfr)||(fr))                 /* First frame already read so skip */
      {
        if (!(rslt2=read_hri1_frame(frame,fp))) return 0;  /* read frame */
/* Add quality of next frames */
        rslt&=rslt2;
      }

/* Use loop var. instead of transmitted frame number info.
   This gives better results.
*/
      if (fr==0)
      {
        nr_pixels=pixels_p_first_fr;
        first_pixel=pci->hri_pci.start_pic;
        ox=subfr*nr_dpsfr;
      }
      else
      {
        nr_pixels=pixels_p_fr;
        first_pixel=4;
        ox=pixels_p_first_fr + (fr-1)*pixels_p_fr + subfr*nr_dpsfr;
      }

/* copy data to line */
      for (i=0; i<nr_pixels; i++)
      {
        line[ox+i]=frame[i+first_pixel];
      }

/* Copy gridinfo */
      if ((fr==fr_p_subfr-1) && (grid))
      {
        for (i=enddata; i<360; i++)
        {
          grid[i-enddata+subfr*(360-enddata)]=frame[i+first_pixel];
        }
      }
    }
  }
  return rslt;
}

