/**************************************************
 * RCSId: $Id$
 *
 * HRI processing functions
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log$
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 R. Alblas. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/

#include "wsat.h"

/*************************************
 * Determine picture size from selected channel
 *************************************/
void hri_channelselect(PICINFO *pci,VIEWINFO *vwi)
{
  int i;
  for (i=1; i<NR_HRI_CHAN; i++)
  {
    if (vwi->channel_selected[i])
    {
      pci->width=pci->hri_pci.channel_info[i].width;
      pci->height=pci->hri_pci.channel_info[i].height;
      break;
    }
  }
}

/*************************************
 * Translate satellite code into string.
 *************************************/
void hri_satind2satname(guint16 sat_ind,char *satnaam)
{
  switch(sat_ind)
  {
    case 0x0000: strcpy(satnaam,"GOES-E"); break;
    case 0x0001: strcpy(satnaam,"GOES-W"); break;
    case 0x0002: strcpy(satnaam,"GMS"); break;
    case 0x0003: strcpy(satnaam,"GOMS"); break;
    case 0xd4e3: strcpy(satnaam,"METEOSAT 1"); break;
    case 0xffff: strcpy(satnaam,"Unknown"); break;
    default:     if ((sat_ind & 0xfff0) == 0xd4f0)
                   sprintf(satnaam,"METEOSAT %d",sat_ind&0x000f);
                 else
                   if (!*satnaam) sprintf(satnaam,"Code %x?",sat_ind);
    break;
  }
}

/*************************************
 * Add grid info to line.
 *************************************/
void add_hrigrid(PICINFO *pci,gboolean *line_grid,guchar *grid)
{
  guchar gr;
  int i;
  if (!line_grid) return;

  for (i=0; i<pci->width; i++)
  {
    guint grbyte=i/8;
    guint grbit=i%8;
    gr=grid[grbyte];
    line_grid[i]=FALSE;
    if (gr&(0x80>>grbit)) line_grid[i]=TRUE; /* Set gridinfo */
  }
}
