#include "wsat.h"
#include "ahrpt.h"
// AHRPT-frame:
//  0...  3: SYNC
//  4... 21: info
// 22... 29: SYNC2
// 30...253: 30bytes[4*C1,4*C2,...4*C6],...
// 254, 255: PEC
//-----------------
//  0...  3: SYNC
//  4... 21: info
// 22...253: 30bytes[4*C1,4*C2,...4*C6],...
// 254, 255: PEC
//-----------------
//...
// (50*)
//
// bytes 1 chan: 1540*10/8=1925
//                           *6
//                        -----
//                        11550 bytes
// PEC's  : 50* 2           100
// headers: 50*22          1100
// lsync                      8
// over                      42
//                        -----
// total 1 line: 256*50=  12800 bytes
// 
static FRAMEINFO calc_ahrptframeinfo(FILE_TYPE file_type,gboolean *channels,int width)
{
  FRAMEINFO fri;
  int i;

  fri.header_len=0;
  fri.leader_len=0;
  fri.words_per_frame=0;
/* Contribution of actual picture data */
  fri.inter_chan=0;
  for (i=1; i<NR_CHANNELS_MAX; i++)
    if (channels[i]) fri.inter_chan++;

  fri.words_per_frame=width*fri.inter_chan;

/* Contribution of header and leader. 
   Leader contains extra bytes to make words_per_frame*10/8 integer.
*/
  if (channels[0])
  {
    fri.words_per_frame+=fri.header_len;
    fri.words_per_frame+=fri.leader_len;
  }
  fri.words_per_line=fri.words_per_frame;

/* Calculate file bytes per line */
  fri.bytes_per_line=12800;   // fri.words_per_line*10/8;

/* Add contribution of extra 'frame separation' bytes */
  fri.bytes_per_line+=0;

  return fri;
}

#define FRMLEN 256
#define TFRMLEN (FRMLEN*50)
#define SYNC2a 0x0218a7a3
#define SYNC2b 0x92dd9abf

int cmp_sync(unsigned char *b,unsigned long l1,unsigned long l2,int nrb)
{
  int i;
  unsigned char sb;
  int err=0;
  for (i=0; i<nrb; i++)
  {
    if (i<4)
    {
      sb=(l1>>((8*(3-i))&0xff));
//if (nrb>4) printf("A %d  %x  %x   l1=%x\n",i,b[i],sb,l1);
      if (b[i]!=sb) err++;
    }
    else
    {
      sb=(l2>>((8*((3-i+4))&0xff)));
//if (nrb>4)printf("B %d  %x  %x\n",i,b[i],sb);
      if (b[i]!=sb) err++;
    }
  }
  
  return err;
}

int search_startofline(FILE *fp)
{
  int ret=0;
  char l[FRMLEN];
  if (!fp) return 0;
  rewind(fp);
  while ((fread(l,FRMLEN,1,fp))) 
  {
    if ((cmp_sync(l,AHRPT_SYNC1,0,4)) > 2); // err 
    if ((cmp_sync(l+22,SYNC2a,SYNC2b,8)<4))
    {
      fseek(fp,-1*FRMLEN,SEEK_CUR);
      ret=1;
      break;
    }
  }
  return ret;
}

/*************************************
 * Read ahrpt header and init PICINFO struct
 *************************************/
int read_ahrpt1_hdr(PICINFO *pci)
{
  FILE *fp=pci->fp;
  int i;
  unsigned long size;
  pci->width=1540;
  pci->height=0;
  pci->depth=10;

/* Copy color map into picinfo structure */
  pci->clrmap.size=1024;

  for (i=0; i<pci->clrmap.size; i++)
  {
    pci->clrmap.red[i]=i;
    pci->clrmap.green[i]=i;
    pci->clrmap.blue[i]=i;
  }

/* Determine channels present */
  for (i=0; i<NR_CHANNELS_MAX; i++) pci->channels_recorded[i]=FALSE;

  pci->lines_per_sec=6;
  switch(pci->file_type)
  {
    case AHRPT1:                /* AHRPT */
      for (i=0; i<=6; i++)
        pci->channels_recorded[i]=TRUE;
      pci->width_original=1540;
//      pci->max_sens_angle=DEF_NOAA_ALPHAMAX;
      strcpy(pci->sat_name,"Meteor");
      pci->sat_type=AHRPT1;
    break;
    default:
    break;
  }
  
  // size
  fseek(fp,0,SEEK_END);
  size=ftell(fp);

/* Start of picture here */
  rewind(fp);
  if (search_startofline(fp))
    pci->pic_strt=ftell(fp);
  pci->fri=calc_ahrptframeinfo(pci->file_type,pci->channels_recorded,pci->width);

  if (!pci->height) pci->height=size/12800;
  pci->scan_hor=W_E;
  pci->scan_ver=N_S;

  pci->ox=0;
  pci->oy=0;

  return 0;
}

#define EXTRA 0
int read_ahrpt1_line(PICINFO *pci,VIEWINFO *vwi,int y,guint16 *line)
{
  FILE *fp=pci->fp;
  int i,j;
  unsigned char tmp[FRMLEN+EXTRA];
  int fpos=0;
  int lpos=0;
  int pix=0;
  int ch=0;
  int skip=0;
  int c=0,x=0,q=0,pos;
//printf("y=%d\n",y);
  fseek(fp,pci->pic_strt + y*pci->fri.bytes_per_line,SEEK_SET);
  while (fread(tmp,FRMLEN+EXTRA,1,fp))
  {
    fseek(fp,-1*EXTRA,SEEK_CUR);
//printf("%x  %x  %x  %x\n",tmp[0],tmp[1],tmp[2],tmp[3]);
//printf("SYNC1=%x\n",AHRPT_SYNC1);
//    if ((cmp_sync(tmp,AHRPT_SYNC1,0,4)) > 2) puts("SYNC ERR!"); 
    fpos=22;
//printf("pos: %d\n",ftell(fp));
    if ((cmp_sync(tmp+fpos,SYNC2a,SYNC2b,8)<4))
    {
//puts("HSYNC");
      x=0;
      c=0;
      q=0;
      fpos+=8;
      skip=0;
      fpos=52;
    }
    for (; fpos<254; fpos++)
    {
      fpos+=skip; skip=0;
      if (fpos>=254) break;
      pos=(x+q)*6+c;

//printf("q=%d  x=%d  chan=%d  pos=%d  fpos=%d\n",q,x,c,pos,fpos);

      switch(q)
      {
        case 0:
          line[pos]=((tmp[fpos]&0xff)<<2)+((tmp[fpos+1]&0xc0)>>6);
        break;
        case 1:
          line[pos]=((tmp[fpos]&0x3f)<<4)+((tmp[fpos+1]&0xf0)>>4);
        break;
        case 2:
          line[pos]=((tmp[fpos]&0x0f)<<6)+((tmp[fpos+1]&0xfc)>>2);
        break;
        case 3:
          line[pos]=((tmp[fpos]&0x03)<<8)+((tmp[fpos+1]&0xff)>>0);
        break;
      }

//if (c==2) line[pos]=x;  else line[pos]=0x1ff;
      q=((q+1)%4);
      if (!q) { skip=1; c=((c+1)%6);}
      if ((!q) && (!c)) x+=4;
      if (x>=1540) break;
    }
    if (x>=1540) break;
  }

  #ifdef XXX
fpos=22;
  for (i=0; i<pci->width/4; i++)
  {
    for (j=0; j<6; j++)
    {
      if (fpos<249)
      {
        fread(tmp,5,1,fp);
        fpos+=5;
      }
      else
      {
        fread(tmp,253-fpos,1,fp);
        fseek(fp,2+22,SEEK_CUR);
        fread(tmp,5-(249-fpos),1,fp);
        fpos=5-(249-fpos);
      }
/*
      line[lpos+0*6+j]=((tmp[0]&0xff)<<2)+((tmp[1]&0xc0)>>6);
      line[lpos+1*6+j]=((tmp[1]&0x3f)<<4)+((tmp[2]&0xf0)>>4);
      line[lpos+2*6+j]=((tmp[2]&0x0f)<<6)+((tmp[3]&0xfc)>>2);
      line[lpos+3*6+j]=((tmp[3]&0x03)<<8)+((tmp[4]&0xff)>>0);
*/
      line[lpos+0+j]=((tmp[0]&0xff)<<2)+((tmp[1]&0xc0)>>6);
      line[lpos+6+j]=((tmp[1]&0x3f)<<4)+((tmp[2]&0xf0)>>4);
      line[lpos+12+j]=((tmp[2]&0x0f)<<6)+((tmp[3]&0xfc)>>2);
      line[lpos+18+j]=((tmp[3]&0x03)<<8)+((tmp[4]&0xff)>>0);
printf("i=%d  chan=%d  %d  %d\n",i,j,lpos,fpos);
    }
    lpos+=24;

  }
#endif
  return 1;
}
