/**************************************************
 * RCSId: $Id$
 *
 * Old HRPT writer
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log$
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 R. Alblas. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/

#include <math.h>
#include <malloc.h>
#include <string.h>

#include "wsat.h"
#include "hrpt.h"
#include "hrpt1.h"

/*************************************
 * Write hrpt header from struct
 *************************************/
int write_hrpt1hdr_file(HRPTHDR *hrpthdr,FILE *fp)
{
  if (fp==NULL) return 1;

/* Write hrpt header in 4 steps, to make it portable between DOS and Linux */

/* Write first part: sign ...  tm_comp_sat_vlg */
  if (!(fwrite(hrpthdr,131,1,fp))) return 0;

/* Write second part: offset_lon ... bo */
  if (!(fwrite((char *)&hrpthdr->offset_lon,42,1,fp))) return 0;

/* Write third part: kr[0] ... channel[9]*/
  if (!(fwrite((char *)&hrpthdr->kr[0],105,1,fp))) return 0;

/* Write fourth part: widthmax ... end */
  if (!(fwrite((char *)&hrpthdr->widthmax,15722,1,fp))) return 0;

  return 0;
}

/*************************************
 * Write chrpt header from struct
 *************************************/
int write_chrpt1hdr_file(CHRPTHDR *chrpthdr,FILE *fp)
{
  if (fp==NULL) return 1;

/* Write chrpt header in 4 steps, to make it portable between DOS and Linux */

/* Write first part: sign ...  tm_comp_sat_vlg */
  if (!(fwrite(chrpthdr,131,1,fp))) return 0;

/* Write second part: offset_lon ... bo */
  if (!(fwrite((char *)&chrpthdr->offset_lon,50,1,fp))) return 0;

/* Write third part: kr[0] ... channel[11]*/
  if (!(fwrite((char *)&chrpthdr->kr[0],131,1,fp))) return 0;

/* Write fourth part: widthmax ... end */
  if (!(fwrite((char *)&chrpthdr->widthmax,15688,1,fp))) return 0;

  return 0;
}


/*************************************
 * Write (c)hrpt header
 *************************************/
int write_hrpt1_hdr(PICINFO *pci)
{
  HRPTHDR *hrpthdr;
  CHRPTHDR *chrpthdr;
  int i,k;
  int chan_max=0;
  if (!pci->fp) return 1;

/* allocate mem for hrpthdr struct */
  if (!(hrpthdr=(HRPTHDR *)calloc(sizeof(HRPTHDR),1))) return 2;
  chrpthdr=(CHRPTHDR *)hrpthdr;


/* Define some characteristics for HRPT/CHRPT/Seawifs format */
  switch (pci->file_type)
  {
    case SEAWIF1:
      strncpy(hrpthdr->sign,SEAW1SIGN,LENSIGN_H);
      chan_max=8;
      if (!pci->width) pci->width=DEF_SEAST_XRES;

      hrpthdr->width=GUINT16_TO_LE(pci->width);
      hrpthdr->widthmax=GUINT16_TO_LE(DEF_SEAST_XRES);
      hrpthdr->alphamax=DEF_SEAST_ALPHAMAX;

      for (k=0; k<chan_max; k++) hrpthdr->dmin[k]=GUINT16_TO_LE(0);
      for (k=0; k<chan_max; k++) hrpthdr->dmax[k]=GUINT16_TO_LE(1023);
    break;
    case CHRPT1:
      strncpy(chrpthdr->sign,CHRPT1SIGN,LENSIGN_H);
      chan_max=10;

      if (!pci->width) pci->width=DEF_FY_XRES;

      chrpthdr->width=GUINT16_TO_LE(pci->width);
      chrpthdr->widthmax=GUINT16_TO_LE(DEF_FY_XRES);
      chrpthdr->alphamax=DEF_FY_ALPHAMAX;

      for (k=0; k<chan_max; k++) chrpthdr->dmin[k]=GUINT16_TO_LE(0);
      for (k=0; k<chan_max; k++) chrpthdr->dmax[k]=GUINT16_TO_LE(1023);
    break;
    case HRPT1:
      strncpy(hrpthdr->sign,HRPT1SIGN,LENSIGN_H);
      chan_max=5;

      if (!pci->width) pci->width=DEF_NOAA_XRES;

      hrpthdr->width=GUINT16_TO_LE(pci->width);
      hrpthdr->widthmax=GUINT16_TO_LE(DEF_NOAA_XRES);
      hrpthdr->alphamax=DEF_NOAA_ALPHAMAX;

      for (k=0; k<chan_max; k++) hrpthdr->dmin[k]=GUINT16_TO_LE(0);
      for (k=0; k<chan_max; k++) hrpthdr->dmax[k]=GUINT16_TO_LE(1023);
    break;
    default:
    break;
  }

  pci->fri=calc_hrptframeinfo(pci->file_type,pci->channels_recorded,pci->width);
  strncpy(hrpthdr->sat_name,pci->sat_name,LENSATNAME_HDR1);
  hrpthdr->satdir=(pci->scan_ver==N_S? 0 : 1);
  hrpthdr->x_offset=pci->ox;
  hrpthdr->y_offset=pci->oy;

/* Load channel numbers recorded */
  if (pci->file_type==CHRPT1)
  {
    for (i=0; i<=chan_max; i++)
      chrpthdr->channel[i]=pci->channels_recorded[i];
  }
  else
  {
    for (i=0; i<=chan_max; i++)
    {
      hrpthdr->channel[i]=pci->channels_recorded[i];
    }
  }

/* Copy Keplerian elements */
  hrpthdr->kepler.epoch_year=GINT32_TO_LE(pci->kepler.epoch_year);
  hrpthdr->kepler.epoch_day=float32_to_le(pci->kepler.epoch_day);
  hrpthdr->kepler.decay_rate=float32_to_le(pci->kepler.decay_rate);
  hrpthdr->kepler.inclination=R2D(float32_to_le(pci->kepler.inclination));
  hrpthdr->kepler.raan=R2D(float32_to_le(pci->kepler.raan));
  hrpthdr->kepler.eccentricity=float32_to_le(pci->kepler.eccentricity);
  hrpthdr->kepler.perigee=R2D(float32_to_le(pci->kepler.perigee));
  hrpthdr->kepler.anomaly=R2D(float32_to_le(pci->kepler.anomaly));
  hrpthdr->kepler.motion=float32_to_le(pci->kepler.motion);
  hrpthdr->kepler.epoch_rev=GINT32_TO_LE(pci->kepler.epoch_rev);

/* Copy selected time. This is needed in case time is changed 
   and then saved without clicking Computertime/sattime buttons. 
*/
  if (pci->orbit.use_sattime)
  {
    pci->orbit.start_tm_sat=pci->orbit.start_tm;
    pci->orbit.start_ms_sat=pci->orbit.start_ms;
  }
  else
  {
    pci->orbit.start_tm_comp=pci->orbit.start_tm;
  }

/* starttime from satellite */
  hrpthdr->tm_satstrt.tm_year=GUINT16_TO_LE(pci->orbit.start_tm_sat.tm_year);
  hrpthdr->tm_satstrt.tm_yday=GUINT16_TO_LE(pci->orbit.start_tm_sat.tm_yday);
  hrpthdr->tm_satstrt.tm_hour=GUINT16_TO_LE(pci->orbit.start_tm_sat.tm_hour);
  hrpthdr->tm_satstrt.tm_min=GUINT16_TO_LE(pci->orbit.start_tm_sat.tm_min);
  hrpthdr->tm_satstrt.tm_sec=GUINT16_TO_LE(pci->orbit.start_tm_sat.tm_sec);
  hrpthdr->tm_satstrt.tm_mon=GUINT16_TO_LE(pci->orbit.start_tm_sat.tm_mon);
  hrpthdr->tm_satstrt.tm_mday=GUINT16_TO_LE(pci->orbit.start_tm_sat.tm_mday);
  hrpthdr->tm_satstrt.tm_isdst=GUINT16_TO_LE(pci->orbit.start_tm_sat.tm_isdst);

/* starttime from computer */
  hrpthdr->tm_comp.tm_year=GUINT16_TO_LE(pci->orbit.start_tm_comp.tm_year);
  hrpthdr->tm_comp.tm_yday=GUINT16_TO_LE(pci->orbit.start_tm_comp.tm_yday);
  hrpthdr->tm_comp.tm_hour=GUINT16_TO_LE(pci->orbit.start_tm_comp.tm_hour);
  hrpthdr->tm_comp.tm_min=GUINT16_TO_LE(pci->orbit.start_tm_comp.tm_min);
  hrpthdr->tm_comp.tm_sec=GUINT16_TO_LE(pci->orbit.start_tm_comp.tm_sec);
  hrpthdr->tm_comp.tm_mon=GUINT16_TO_LE(pci->orbit.start_tm_comp.tm_mon);
  hrpthdr->tm_comp.tm_mday=GUINT16_TO_LE(pci->orbit.start_tm_comp.tm_mday);
  hrpthdr->tm_comp.tm_isdst=GUINT16_TO_LE(pci->orbit.start_tm_comp.tm_isdst);

  hrpthdr->offset_gmt=pci->orbit.offset_gmt;
  hrpthdr->offset_sec=pci->orbit.offset_sec;
  hrpthdr->offset_lon=pci->orbit.offset_lon;
  hrpthdr->tm_comp_sat_vlg=(pci->orbit.use_sattime? 's' : 'c');

  hrpthdr->height=GUINT16_TO_LE(pci->height);

  strcpy(hrpthdr->hdr_version,HRPTHDRVERSION);
  hrpthdr->hdrlen=GUINT16_TO_LE(16000); /* sizeof(*hrpthdr); */
  strncpy(hrpthdr->endhdr,ENDHDR,8);

  if (pci->file_type==CHRPT1)
    write_chrpt1hdr_file(chrpthdr,pci->fp);
  else
    write_hrpt1hdr_file(hrpthdr,pci->fp);

  free(hrpthdr);
  return 0;
}


/* Write a line in hrpt1-format.
   Each line starts with a 7-byte header 'FRAMExy"; 
   xy=amount of bytes of actual line.
   fri.bytes_per_line includes the 7 intermediate bytes;
   so value of 'xy' is fri.bytes_per_line-NR_INTER_BYTES (=7).
*/
int write_hrpt1_line(PICINFO *pci,guint16 *line,int line_stat)
{
  guchar tmp[6];
  FILE *fp=pci->fp;
  short bytes_per_line=pci->fri.bytes_per_line-NR_INTER_BYTES;
  guint16 *pline;
  int nr_qwords=pci->fri.words_per_line/4;
  int i,j;
  if (!fp) return 0;

  if (line_stat & MASK_HRPT1_SYNC_OK_PRE)
    strcpy(tmp,"FRAME");
  else
    strcpy(tmp,"FRAMe");

  fwrite(tmp,5,1,fp);
  fwrite(&bytes_per_line,1,2,fp);

  pline=line;
  for (i=0; i<nr_qwords; i++)
  {

    tmp[4]=0;
    for (j=0; j<4; j++)
    {
      tmp[j]=(*pline)>>2;

      tmp[4]=tmp[4] | ((*pline) & 0x03) << ((3-j)*2);
      pline++;
    }

    fwrite(tmp,5,1,fp);
  }
  return 1;
}
