/**************************************************
 * RCSId: $Id$
 *
 * HRPT record functions.
 * Based on FPGA-decoder, connected to parallel port.
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log$
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 R. Alblas. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/
#include <unistd.h>
#include <malloc.h>
#include <string.h>
#include <stdlib.h>
#include "pp_dec_interf.h"
#include "wsat.h"
#include "menu_wsat.h"
#include "hrpt.h"
#include "hrpt_funcs.h"

#if _HRPT1_
#  include "hrpt1.h"
#endif

extern PREFS prefs;


/* ====== Define button text for HRPT Record window ====== */
#define LAB_CHX     "   "
#define LAB_CH1     "   1  "
#define LAB_CH2     "   2  "
#define LAB_CH3     "   3  "
#define LAB_CH4     "   4  "
#define LAB_CH5     "   5  "
#define LAB_CHA     "  All "

/* ====== Define button text for CHRPT Record window ====== */
#define LAB_CH12    " 1+2"
#define LAB_CH34    " 3+4"
#define LAB_CH56    " 5+6"
#define LAB_CH78    " 7+8"
#define LAB_CH910   "9+10"

/* ====== Define button text for view channel options menu ====== */
#define LAB_VCHR "!Show channel"
#define LAB_CHX_V   "Chan"
#define LAB_CH1_V   "Chan 1"
#define LAB_CH2_V   "Chan 2"
#define LAB_CH3_V   "Chan 3"
#define LAB_CH4_V   "Chan 4"
#define LAB_CH5_V   "Chan 5"
#define LAB_CH6_V   "Chan 6"
#define LAB_CH7_V   "Chan 7"
#define LAB_CH8_V   "Chan 8"
#define LAB_CH9_V   "Chan 9"
#define LAB_CH10_V  "Chan 10"

#define LAB_SATDIR "Sat direction"
#define LAB_DIR_NS "To south"
#define LAB_DIR_SN "To north"
#define LAB_DIR_NSN "South bound/North bound"

#define LAB_MAXY "^Lines/screen"

#define LAB_HRPT    "HRPT"
#define LAB_CHRPT   "CHRPT"

#define LAB_DEC     "Decoder"
#define LAB_GEN     "Generator"

#define LAB_Start   "Start/Stop"
#define LAB_Stop    "Stop"
#define LAB_Save    "Save"
#define LAB_KEP     "Kepler-load"


#define LAB_ADVANCED  "Advanced"
#define LAB_ADV_SET   "!advanced"
#define LAB_Exit      " Exit "
#define Lab_Hrptwnd   "Record HRPT"
#define LAB_Recording "Recording"
#define LAB_RECLINES  "Line       "
#define LAB_Satname   "Sat name "

static GtkWidget *hide_but;
/*************************************
 * Get a (C)HRPT-frame from decoder.
 * Return:
 *   0 -> Signal lost.
 *   1 -> OK; decoder in-sync
 *   2 -> OK, but decoder out-sync
 *************************************/
int get_hrpt_frame(guint16    *frame,
                   RECORDINFO *rdi)
{
  switch(rdi->port_type)
  {
#ifdef __ADD_USB__ 
    case usb     : return usb_get_hrpt_frame(frame,rdi);
#endif
#ifdef __ADD_ETH__ 
    case ethernet: return eth_get_hrpt_frame(frame,rdi);
#endif
#ifdef __ADD_PAR__ 
    default:       return getpp_hrpt_frame(frame,rdi);
#endif
  }
}

void select_channel(gboolean   *chan,
                    RECORDINFO *rdi,
                    gboolean    reprogramming_fpga)
{
  switch(rdi->port_type)
  {
    case usb     :                                                 break;
    case ethernet:                                                 break;
#ifdef __ADD_PAR__ 
    default      : select_channel_pp(chan,rdi,reprogramming_fpga); break;
#endif
  }
}

// Open connection. return: 1 if OK
// 
static int Open_HWConnect(RECORDINFO *rdi)
{
  int err=0;
  switch(rdi->port_type)
  {
#ifdef __ADD_USB__ 
    case usb     : err=usb_init_connect(rdi);    break;
#endif
#ifdef __ADD_ETH__ 
    case ethernet: err=eth_init_connect(rdi);         break;
#endif
#ifdef __ADD_PAR__ 
    default      : err=ioperm(rdi->parport_base,3,1); break;
#endif
  }
  if (err) return 0;
  return 1;
}

static void Close_HWConnect(RECORDINFO *rdi)
{
  switch(rdi->port_type)
  {
#ifdef __ADD_USB__ 
    case usb     : usb_close_connect(rdi); break;
#endif
#ifdef __ADD_ETH__ 
    case ethernet:                         break;
#endif
    default      :                         break;
  }
}

int get_hwselected(int con,RECORDINFO *rdi)
{
  switch(rdi->port_type)
  {
#ifdef __ADD_USB__ 
    case usb     : return 0;
#endif
#ifdef __ADD_ETH__ 
    case ethernet: return 0;
#endif
#ifdef __ADD_PAR__ 
    default      : return get_ppreg(con+2) & PIN_17;
#endif
  }
}

int wait_for_insync(RECORDINFO *rdi,int threshold)
{
  switch(rdi->port_type)
  {
#ifdef __ADD_USB__ 
    case usb     : return usb_wait_for_insync(rdi,threshold);
#endif
#ifdef __ADD_ETH__ 
    case ethernet: return eth_wait_for_insync(rdi,threshold);
#endif
#ifdef __ADD_PAR__ 
    default      : return pp_wait_for_insync(rdi,threshold);
#endif
  }
}

int wait_for_outsync(RECORDINFO *rdi,int threshold)
{
  switch(rdi->port_type)
  {
#ifdef __ADD_USB__ 
    case usb     : return usb_wait_for_outsync(rdi,threshold);
#endif
#ifdef __ADD_ETH__ 
    case ethernet: return eth_wait_for_outsync(rdi,threshold);
#endif
#ifdef __ADD_PAR__ 
    default      : return pp_wait_for_outsync(rdi,threshold);
#endif
  }
}

/*************************************
 * Determine satellite name.
 * frame    contains hrpt-frame
 * sat_name contains the recognized name 
 * If frame==NULL then reset, i.e. recognize starts again.
 *************************************/
#define RECOGN_LEVEL 5
gboolean get_hrptsat_name(guint16 *frame,
                          char *sat_name, FILE_TYPE file_type,SAT_TYPE sat_type)
{
  static int recognized;
  static int id;
  int cur_id;
  if (frame==NULL)
  {
    id=0;
    recognized=0;                              /* Start recognizing. */
    return FALSE;
  }

  if (recognized > RECOGN_LEVEL) return FALSE; /* Already recognized. */

/* Def. word 7:
  static int mfrm;
     HRPT : bit 2,3=minor frame count
            bit 4-7 =ID
     CHRPT: bit 1-4=ID
*/   
  if (file_type==CHRPT1)
  {
    cur_id=(frame[6] >> 6) & 0x00f;            /* Get Fenyung-id from frame */
  }
  else if (file_type==HRPT1)
  {
    cur_id=(frame[6] >> 3) & 0x00f;            /* Get NOAA-id from frame */
  }
  else
  {
/*
autodetect HRPT/CHRPT
    if (frame[6]&0x180 == mfrm)
*/
  }

/* Search for number of identical id's in conseq. frames */
  if (id!=cur_id)                              /* Reset if change. */
  {
    recognized=0;
  }
  id=cur_id;
  recognized++;

/* If recognized then decode sat-id into sat-name */
  if (recognized>=RECOGN_LEVEL)
  {
    strcpy(sat_name,xrptid2name(frame[6],sat_type));
    return TRUE;
  }

/* Not recognized yet */
  return FALSE;
}

/*************************************
 * Record one HRPT picture.
 *************************************/
void Recordpp_one_Hrpt(GtkWidget  *hrpt_window,
                       GtkWidget  *main_window,
                       RECORDINFO *rdi,
                       PICINFO    *pci,
                       VIEWINFO   *vwi)
{
  GtkWidget *drawing_area=Find_Widget(main_window,"GTK_DRAWING_AREA");
  RGBI *rgbi=Get_RGBI(drawing_area);
  PICINFO pci_w;
  int xdrawmax=0;
  int ydrawmax=0;
  int ydraw=0,y=0;
  int nr_frames;
  guint16 *frame;
  int out_sync_counter=0;
  gboolean run_record=TRUE;
  int status,prev_status=-1;
  int i;
  int ypos;

  guint *xmap=NULL;
  guint *ymap=NULL;
  guint *zmap=NULL;

  if (drawing_area)
  {
    xdrawmax=drawing_area->allocation.width;
    ydrawmax=drawing_area->allocation.height;
  }
  
/* If a file was open for read, close it */
  close_picfile(pci);


/* open file if the record has to be saved */
  if (rdi->save_flag)
  {
    if (!(open_new_recfile(rdi->file_prefix,rdi->file_suffix,rdi)))
    {
      Create_Message("Error","Can't open file %s!",rdi->file_name);
      return;
    }
    Set_Entry(hrpt_window,LAB_Recording,"%s",rdi->file_name);
    if (rdi->batch_mode) log_message("Start recording in %s.",rdi->file_name);
    
    memset(&pci_w,0,sizeof(pci_w));
   
    pci_w.width=0;
    pci_w.height=0;
    pci_w.ox=0;
    pci_w.oy=0;
    pci_w.scan_ver=rdi->scan_ver;
    for (i=0; i<NR_CHANNELS_MAX; i++) 
      pci_w.channels_recorded[i]=rdi->channels_recording[i];
    pci_w.file_type=rdi->file_type;
    pci_w.fp=rdi->fp;
    strcpy(pci_w.sat_name,rdi->sat_name);

/* Get computer time, to be stored in file header */
    {
      time_t t;
      struct tm *tm;
      t=time(NULL);
      if (!(tm=gmtime(&t))) tm=localtime(&t);
      pci_w.orbit.start_tm_comp=*tm;
      pci_w.orbit.start_tm=*tm;
    }
    pci_w.orbit.use_sattime=FALSE;
#if _HRPT1_
    write_hrpt1_hdr(&pci_w);
#endif
  }
  else
    Set_Entry(hrpt_window,LAB_Recording,"(no save)");

/* allocate mem to hold 1 frame */
  frame=(guint16 *)malloc(rdi->fri.words_per_frame*sizeof(guint16));

/* Define some view characteristics for the channel to show real-time */
  vwi->zval=zoom_full();
  vwi->scan_hor=W_E;
  vwi->scan_ver=N_S;
  vwi->lum_min=0;
  vwi->lum_max=1023;

/* Determine mapping of this channel.
   ymap from map_pic is translation pic_y -> screen_y, but here vice-versa 
   is needed. So, ymap can only be used to handle sat-direction.
   Actual sizing is done lateron.
*/
  pci->height=ydrawmax;             /* set 1:1 mapping */
  vwi->scan_ver=rdi->scan_ver;
  vwi->scan_hor=rdi->scan_hor;

  map_pic(main_window,pci,vwi,FALSE,&xmap,&ymap,&zmap);

/* Reset sat-name recognition */
  get_hrptsat_name(NULL,NULL,0,0);

/* Start the record loop. Each loop is 1 frame. */
  nr_frames=0;

  while ((run_record) && (rdi->recording))
  {
/* Catch gtk-events (e.g. stop-signal, which makes rdi->recording=FALSE.) */
    if (rdi->batch_mode)
    {
/*
for test
      if ((rdi->sel_hw) && (nr_frames>100)) break;
*/
    }
    else
    {
      while (g_main_iteration(FALSE));
    }

/* Get a (C)HRPT-frame. Abort recording if failed. */
    status=get_hrpt_frame(frame,rdi);
    if ((prev_status>=0) && (status!=prev_status))
    {
      if ((status) && (!prev_status))
        Set_Entry(hrpt_window,LAB_Recording,"Restored.");
      else if (!status)
        Set_Entry(hrpt_window,LAB_Recording,"No transm!");
    }
    prev_status=status;

/* Handle in-sync info decoder */
    if (status==2)                            /* out-sync */
      out_sync_counter++;
    else                                      /* in-sync */
      out_sync_counter=0;
    if (out_sync_counter > rdi->hrpt_stop_after_out_sync)
      break;                                  /* abort by decoder */

/* Determine sat-name if not done yet. 
   This function returns immediately if sat-name was already recognized
   during previous received lines.
*/
    if (get_hrptsat_name(frame,rdi->sat_name,
                               rdi->file_type,
                               rdi->sat_type))
    {
      Set_Entry(hrpt_window,LAB_Satname,rdi->sat_name);
    }

    if (main_window)
    {
/* Do actual mapping */
      if (!rdi->height) rdi->height=drawing_area->allocation.height;
      ypos=y*(ydrawmax-1)/rdi->height;

/* Restart at top if canvas is full */
      if (ypos>=ydrawmax) y=0;
      ydraw=*(ymap+ypos);

/* solve under/overflow (just in case; should never happen) */
      if (ydraw<0) ydraw=0;
      if (ydraw>=ydrawmax) ydraw=ydrawmax-1;

/* Draw the received line into the canvas. */
      draw_a_line_bw(frame,xdrawmax,ydraw,xmap,zmap,1023,rgbi,NULL);
      Refresh_Rect(drawing_area,0,ydraw,
                   drawing_area->allocation.width,
                   1);

      y++;
      if (!(y%10)) Set_Entry(hrpt_window,LAB_RECLINES,"%d",y);
    }
    else if (rdi->batch_mode)
    {   
      y++;
      if (!(y%10)) log_message("Line %d...",y);
    }

/* Save frame */
    if (rdi->save_flag)
#if _HRPT1_
      write_hrpt1_line(&pci_w,frame,(status==1? MASK_HRPT1_REC_OK_POST : 0));
#endif

    nr_frames++;
  }

/* End of recording. */
  if (rdi->save_flag)
  {
    close_recfile(rdi);
    if (rdi->auto_get_kepler)
    {
      FILE *fp=fopen(rdi->kepler_file,"r");
      if (fp)
      {
        if (get_satkeps(fp,rdi->sat_name,&pci_w.kepler))
          Set_Entry(hrpt_window,
                          LAB_Satname,"Kep. ld: %s",rdi->sat_name);
        else
          Set_Entry(hrpt_window,
                          LAB_Satname,"Kep. '%s' not found!",rdi->sat_name);
      }
      else
        if (!(*rdi->kepler_file))
          Create_Message("Error","Kep-file not defined\n(See Preferences.)");
        else
          Create_Message("Error","Kep-file not found:\n%s",rdi->kepler_file);
    }

/* Rewrite header to update number of frames recorded. */
    rdi->fp=fopen(rdi->file_name,"rb+");

    pci_w.height=nr_frames;
    pci_w.fp=rdi->fp;           /* Copy file pointer for write_hrpt1_hdr */
    strcpy(pci_w.sat_name,rdi->sat_name); /* file name now known */
#if _HRPT1_
    write_hrpt1_hdr(&pci_w);
#endif

    close_recfile(rdi);
  }
  free(frame);
  Set_Entry(hrpt_window,LAB_Recording,"Stop");
  if (rdi->batch_mode) log_message("Stop recording after %d frames.",nr_frames);

  free(xmap);
  free(ymap);
  free(zmap);

  return;
}


/*************************************
 * Record (C)HRPT.
 *************************************/
void Recordpp_Hrpt(GtkWidget *hrpt_window,
                   GtkWidget *main_window,
                   RECORDINFO *rdi,
                   PICINFO *pci,
                   VIEWINFO *vwi)
{
  int addr_portd=rdi->parport_base;
  int i;
/* Check if it is allowed to access the selected port */
  if (!Open_HWConnect(rdi))
  {                              // err returned!
    switch(rdi->port_type)
    {
      case usb:
        if (main_window)
          Create_Message("Error","Can't open USB port %s",
            (rdi->usb_use_id?rdi->usbport_id : ""));
        else
          fprintf(stderr,"Can't open USB port.");
      break;
      case ethernet:
        if (main_window)
          Create_Message("Error","Can't open Ethermet port.");
        else
          fprintf(stderr,"Can't open Ethermet port.");
      break;
      default:
        if (main_window)
          Create_Message("Error",PAR_ERR_MESSAGE,addr_portd);
        else
          fprintf(stderr,PAR_ERR_MESSAGE,addr_portd);
      break;
    }
    return;
  }

/* Copy recording channel info into picinfo struct */
  for (i=0; i<NR_CHANNELS_MAX; i++)
  {
    pci->channels_recorded[i]=rdi->channels_recording[i];
    vwi->channel_selected[i]=FALSE;
  }

/* Determine which channel to show real-time, during record */
  rdi->sh_recchan=MIN(NR_CHANNELS_MAX-1,MAX(0,rdi->sh_recchan));

/* First try to get choice in 'show-during-record' menu. */
  if (pci->channels_recorded[rdi->sh_recchan])
  {
    vwi->channel_selected[rdi->sh_recchan]=TRUE;
  }
  else
  {
/* Choice will not be recorded -> choose other channel */
    for (i=1; i<NR_CHANNELS_MAX; i++)
    {
      if (pci->channels_recorded[i])
      {
        rdi->sh_recchan=i;
        vwi->channel_selected[rdi->sh_recchan]=TRUE;
        Set_Button(hrpt_window,LAB_VCHR,rdi->sh_recchan-1);
        break;
      }
    }
  }

/* Check if at least 1 channel is selected for recording. */
  for (i=1; i<NR_CHANNELS_MAX; i++)
  {
    if (pci->channels_recorded[i]) break;
  }

/* If no channels selected: Abort. */
  if (i>=NR_CHANNELS_MAX)
  {
    if (main_window)
      Create_Message("Error","No channels selected!");
    else
      fprintf(stderr,"Error: No channels selected.\n");
    Close_HWConnect(rdi);
    return;
  }

/* Determine some pic-parameters */
  pci->clrmap.size=1024;
  pci->width=2048;
  pci->height=500;
  pci->file_type=rdi->file_type;
  pci->sat_type=rdi->sat_type;

/* calculate some frame charactersitics */
  pci->fri=calc_hrptframeinfo(pci->file_type,pci->channels_recorded,pci->width);
  rdi->fri=pci->fri;

  if (rdi->file_type==CHRPT1) 
    strcpy(rdi->file_prefix,"chrpt");
  else
    strcpy(rdi->file_prefix,rdi->file_hrptbase);
  strcpy(rdi->file_suffix,rdi->file_hrptext);

/* Start the record loop; Each loop is 1 picture. */
  while (rdi->recording)
  {
/* Wait until the decoder is in-sync for some time */
    Set_Entry(hrpt_window,LAB_Recording,"Wait");
    if (rdi->batch_mode) log_message("Wait for in-sync.");
    if (!(wait_for_insync(rdi,rdi->hrpt_start_after_in_sync))) break;

    Recordpp_one_Hrpt(hrpt_window,main_window,rdi,pci,vwi);

/* If just 1 record then close the program (record finished) */
    if (rdi->one_record)
    {
      rdi->recording=FALSE;
      Close_HWConnect(rdi);
      if (main_window) gtk_main_quit(); else return;
    }
  }
  Set_Entry(hrpt_window,LAB_Recording,"Off");
  Close_HWConnect(rdi);
  
/* Open file if recorded */
  if ((!rdi->batch_mode) && (rdi->save_flag))
  {
    strcpy(pci->file_name,rdi->file_name);
    open_file(pci,vwi,main_window);
  }
  if (rdi->batch_mode) log_message("End recording.");
}


/*************************************
 * Get button signals for Record functions
 *************************************/
void Record_Hrpt_Commands(GtkWidget *widget, gpointer data)
{
  GtkWidget *main_window=Find_Window(Find_Parent_Window(widget),Lab_Main);
  GtkWidget *hrpt_window=Find_Window(main_window,Lab_Hrptwnd);
  PICINFO *pci=Get_Picinfo_db(widget);
  VIEWINFO *vwi=Get_Viewinfo_db(widget);
  RECORDINFO *rdi=Get_Recordinfo_db(widget);

  int addr_portd;
  char *name;
  gboolean channelselect_changed=FALSE;
  static gboolean reprogramming_fpga=TRUE;
  gboolean chan[6];
  int i;

  if (!rdi) return;  /* create_options_menu activates this func before everything is settled! */

  name=(char *)data;
  addr_portd=rdi->parport_base;

/* Get status of channel buttons */
  chan[0]=TRUE;
  chan[1]=Get_Button(hrpt_window,LAB_CH1);
  chan[2]=Get_Button(hrpt_window,LAB_CH2);
  chan[3]=Get_Button(hrpt_window,LAB_CH3);
  chan[4]=Get_Button(hrpt_window,LAB_CH4);
  chan[5]=Get_Button(hrpt_window,LAB_CH5);

  if (!strncmp(name,LAB_CHX,strlen(LAB_CHX)))
  {
    channelselect_changed=TRUE;
  }

  if (!strcmp(name,LAB_CHA))
  {
    int nr_sel=0;
/* If all channels selected then deselect all, else select all. */
    gboolean select_val=FALSE;
    for (i=1; i<=5; i++) if (chan[i]) nr_sel++;
    if (nr_sel<5) select_val=TRUE;

/* Select or deselect all channels. */
    for (i=1; i<=5; i++) chan[i]= select_val;
    Set_Button(hrpt_window,LAB_CH1,select_val);
    Set_Button(hrpt_window,LAB_CH2,select_val);
    Set_Button(hrpt_window,LAB_CH3,select_val);
    Set_Button(hrpt_window,LAB_CH4,select_val);
    Set_Button(hrpt_window,LAB_CH5,select_val);

    channelselect_changed=TRUE;
  }

/* Show-during-record parameters */
  {
    char *ext;
    ext=soption_menu_get_history(Find_Widget(widget,LAB_VCHR));
    if ((ext) && (!strcmp(name,ext)))
    {
      rdi->sh_recchan=atoi(name+strlen(LAB_CHX_V));
    }
  }

  if (!strcmp(name,LAB_DIR_NS))
  {
    rdi->scan_ver=N_S;
    rdi->scan_hor=W_E;
  }
  if (!strcmp(name,LAB_DIR_SN))
  {
    rdi->scan_ver=S_N;
    rdi->scan_hor=E_W;
  }

  if (!strcmp(name,LAB_DIR_NSN))
  {
    if (Update_Togglelabel(widget))
    {
      rdi->scan_ver=S_N;
      rdi->scan_hor=E_W;
    }
    else
    {
      rdi->scan_ver=N_S;
      rdi->scan_hor=W_E;
    }
  }

  if (!strcmp(name,LAB_MAXY))
  {
    rdi->height=GTK_ADJUSTMENT(widget)->value;
  }

/* HRPT/CHRPT choice */
  if (!strcmp(name,LAB_HRPT))
  {
    GtkWidget *w;
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
    {
      rdi->file_type=HRPT1;
      rdi->sat_type=HRPT;
    }
    else
    {
      rdi->file_type=CHRPT1;
      rdi->sat_type=CHRPT;
    }

    if (rdi->hwsel_type)
    {
      reprogramming_fpga=TRUE;
      channelselect_changed=TRUE;
    }
    if (rdi->file_type == CHRPT1)
    {
      w=Find_Widget(hrpt_window,LAB_CH1);
      gtk_label_set_text(GTK_LABEL(GTK_BIN(w)->child),LAB_CH12);
      w=Find_Widget(hrpt_window,LAB_CH2);
      gtk_label_set_text(GTK_LABEL(GTK_BIN(w)->child),LAB_CH34);
      w=Find_Widget(hrpt_window,LAB_CH3);
      gtk_label_set_text(GTK_LABEL(GTK_BIN(w)->child),LAB_CH56);
      w=Find_Widget(hrpt_window,LAB_CH4);
      gtk_label_set_text(GTK_LABEL(GTK_BIN(w)->child),LAB_CH78);
      w=Find_Widget(hrpt_window,LAB_CH5);
      gtk_label_set_text(GTK_LABEL(GTK_BIN(w)->child),LAB_CH910);
    }
    else
    {
      w=Find_Widget(hrpt_window,LAB_CH1);
      gtk_label_set_text(GTK_LABEL(GTK_BIN(w)->child),LAB_CH1);
      w=Find_Widget(hrpt_window,LAB_CH2);
      gtk_label_set_text(GTK_LABEL(GTK_BIN(w)->child),LAB_CH2);
      w=Find_Widget(hrpt_window,LAB_CH3);
      gtk_label_set_text(GTK_LABEL(GTK_BIN(w)->child),LAB_CH3);
      w=Find_Widget(hrpt_window,LAB_CH4);
      gtk_label_set_text(GTK_LABEL(GTK_BIN(w)->child),LAB_CH4);
      w=Find_Widget(hrpt_window,LAB_CH5);
      gtk_label_set_text(GTK_LABEL(GTK_BIN(w)->child),LAB_CH5);

/*
Make in options menu not-active channels insensitive.


This works, for whole menu:
Sense_Button(hrpt_window,LAB_VCHR,FALSE);

This should work for 1 item of options menu
Sense_Button(hrpt_window,LAB_CH5_V,FALSE);
Or:
w=Find_Widget(hrpt_window,LAB_CH5_V);
gtk_widget_set_sensitive(w,FALSE);    

*/


    }
  }

/* Copy state of buttons to channels_recording. 
   This depends on channels selected and choice HRPT/CHRPT. 
*/
  for (i=1; i<NR_CHANNELS_MAX; i++)
    rdi->channels_recording[i]=FALSE;
  for (i=1; i<=5; i++)
  {
    if (rdi->file_type == CHRPT1)
    {
      rdi->channels_recording[i*2-1]=chan[i];
      rdi->channels_recording[i*2]=chan[i];
    }
    else
      rdi->channels_recording[i]=chan[i];
  }

/* Decoder/generator choice */
  if (!strcmp(name,LAB_DEC))
  {
    rdi->gen1_dec0=(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))? FALSE : TRUE);
    reprogramming_fpga=TRUE;
    channelselect_changed=TRUE;
//usb_write(rdi);
  }

/* Recording to save or not to save... */
  if (!strcmp(name,LAB_Save))
  {
    rdi->save_flag=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
  }
  if (!strcmp(name,LAB_KEP))
  {
    rdi->auto_get_kepler=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
  }

  if (!strcmp(name,LAB_ADVANCED))
  {
    if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
    {
      hide_but=Find_Widget(hrpt_window,LAB_ADV_SET);
      if (hide_but) gtk_widget_hide(hide_but); 
    }
    else
    {
      if (hide_but) gtk_widget_show(hide_but);
      hide_but=NULL;
    }
  }

/* Start record */
  if ((!strcmp(name,LAB_Start)) && (Update_Togglelabel(widget)))
  {
    if (Update_Togglelabel(widget))

    rdi->recording=TRUE;

    Sense_Button(hrpt_window,LAB_Exit,FALSE);
    Sense_Button(hrpt_window,LAB_Save,FALSE);
    Sense_Button(hrpt_window,LAB_HRPT,FALSE);
    Sense_Button(hrpt_window,LAB_CHRPT,FALSE);
    Sense_Button(hrpt_window,LAB_CH1,FALSE);
    Sense_Button(hrpt_window,LAB_CH2,FALSE);
    Sense_Button(hrpt_window,LAB_CH3,FALSE);
    Sense_Button(hrpt_window,LAB_CH4,FALSE);
    Sense_Button(hrpt_window,LAB_CH5,FALSE);
    Sense_Button(hrpt_window,LAB_CHA,FALSE);

    Recordpp_Hrpt(hrpt_window,main_window,rdi,pci,vwi);

    Sense_Button(hrpt_window,LAB_Exit,TRUE);
    Sense_Button(hrpt_window,LAB_Save,TRUE);
    Sense_Button(hrpt_window,LAB_HRPT,TRUE);
    Sense_Button(hrpt_window,LAB_CHRPT,TRUE);
    Sense_Button(hrpt_window,LAB_CH1,TRUE);
    Sense_Button(hrpt_window,LAB_CH2,TRUE);
    Sense_Button(hrpt_window,LAB_CH3,TRUE);
    Sense_Button(hrpt_window,LAB_CH4,TRUE);
    Sense_Button(hrpt_window,LAB_CH5,TRUE);
    Sense_Button(hrpt_window,LAB_CHA,TRUE);
    rdi->recording=FALSE;
    Set_Button(hrpt_window,LAB_Start,FALSE);
  }

  if ((!strcmp(name,LAB_Start)) && (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))))
  {
    rdi->recording=FALSE;
  }

  if (!strcmp(name,LAB_Stop))
  {
    rdi->recording=FALSE;
  }

  if (!strcmp(name,LAB_Exit))
  {
    Close_Window(hrpt_window);
    Sense_Button(main_window,MENU_HRI,TRUE);
    Sense_Button(main_window,MENU_RAWD,TRUE);
  }

/* Write new select to decoder */
  if (rdi->hwsel_type)
    rdi->sel_hw=(rdi->file_type==CHRPT1? TRUE : FALSE);
  else
    rdi->sel_hw=rdi->gen1_dec0;

  if ((rdi->port_type==parport) && (channelselect_changed))
  {
/* Open port connection */
    if (!Open_HWConnect(rdi))
    {
      if (rdi->port_type==usb)            // NVT als geen kanaalselectie
      {
        if (main_window)
          Create_Message("Error","Can't open USB port.");
        else
          fprintf(stderr,"Can't open USB port.");
      }
      else if (rdi->port_type==ethernet) // NVT als geen kanaalselectie
      {
        if (main_window)
          Create_Message("Error","Can't open ethernet port.");
        else
          fprintf(stderr,"Can't open ethernet port.");
      }
      else
      {
        if (main_window)
          Create_Message("Error",PAR_ERR_MESSAGE,addr_portd);
        else
          fprintf(stderr,PAR_ERR_MESSAGE,addr_portd);
      }
      return;
    }
    select_channel(chan,rdi,reprogramming_fpga);
    Close_HWConnect(rdi);
    reprogramming_fpga=FALSE;
  }
}

static void close_wnd(GtkWidget *wnd)
{
  GtkWidget *mwnd=Find_Window(gtk_widget_get_toplevel(wnd),Lab_Main);;
  Sense_Button(mwnd,MENU_HRI,TRUE);
  Sense_Button(mwnd,MENU_RAWD,TRUE);
  destroy_window(wnd);
}

/*************************************
 * Record window. 
 * This function is started from the main menu.
 *************************************/
void Menu_Rec_Hrpt(GtkWidget *widget)
{
  GtkWidget *w1,*w2,*w3;
  GtkWidget *wa,*wb,*wc,*wd,*we,*wf,*wg,*wh,*wq,*wx,*wy,*wz;
  GtkWidget *hrpt_window;
  GtkWidget *main_window=Find_Parent_Window(widget);
  RECORDINFO *rdi=Get_Recordinfo_db(widget);
  GtkWidget *drawing_area=Find_Widget(main_window,"GTK_DRAWING_AREA");

  GdkColor startbut_clr,startbutsclr;
  startbut_clr.red=0xffff; startbut_clr.green=0x5555; startbut_clr.blue=0x5555;
  startbutsclr.red=0xffff; startbutsclr.green=0x8888; startbutsclr.blue=0x8888;

  hide_but=NULL;
  if (!(hrpt_window=Create_Window(main_window,0,0,Lab_Hrptwnd,close_wnd))) return;

/* Set policy to 'auto-resize' */
  gtk_window_set_policy((GtkWindow *)hrpt_window,FALSE,FALSE,TRUE);

  rdi->sat_type=HRPT;
  rdi->file_type=HRPT1;

  if (rdi->port_type==usb)
  {
    int i;
    for (i=0; i<=5; i++) rdi->channels_recording[i]=TRUE;
  }
  else if (rdi->port_type==ethernet)
  {
    int i;
    for (i=0; i<=5; i++) rdi->channels_recording[i]=TRUE;
  }
  else
  {
    rdi->channels_recording[0]=TRUE;
  }
  if (!rdi->height) rdi->height=drawing_area->allocation.height;

/* ==================== Button groups ==================== */
/* -------------------- Channel choice ------------------- */
  wa=Create_ButtonArray("Channel selection",Record_Hrpt_Commands,3,
                        TOGGLE,LAB_CH1,rdi->channels_recording[1],
                        TOGGLE,LAB_CH2,rdi->channels_recording[2],
                        TOGGLE,LAB_CH3,rdi->channels_recording[3],
                        TOGGLE,LAB_CH4,rdi->channels_recording[4],
                        TOGGLE,LAB_CH5,rdi->channels_recording[5],
                        BUTTON,LAB_CHA,
                        0
                       );

/* -------------------- HRPT/CHRPT  ------------------- */
  w1=Create_Radio(LAB_HRPT,TRUE,Record_Hrpt_Commands);
  w2=Create_Radio(LAB_CHRPT,FALSE,Record_Hrpt_Commands);
  wb=Pack("Sat type",'v',w1,0,w2,0,NULL);


/* --------------------START/STOP----------------------- */
  w1=Create_Toggle(LAB_Start,Record_Hrpt_Commands,FALSE);
  Set_Color(w1,GTK_RC_BG,NULL,&startbutsclr,&startbut_clr,&startbut_clr,NULL);

/*
  w2=Create_Button(LAB_Stop,Record_Hrpt_Commands);
*/
  w2=Create_Toggle(LAB_DIR_NSN,Record_Hrpt_Commands,FALSE);
  wc=SPack("","hH",w1,"fe3",w2,"fe3",NULL);

/* --------------------- STATUS ------------------------- */
  w1=Create_Entry(LAB_Recording,NULL,"%-10s","Off");
  w2=Create_Entry(LAB_Satname  ,NULL,"%-10s","");
  w3=Create_Entry(LAB_RECLINES,NULL,"%4d",0);
  wd=Pack("Status",'v',w1,2,w2,2,w3,2,NULL);

/* ==================== Advanced Button groups ==================== */
/* --------------------- DEC/GEN ---------------------- */
  if (!rdi->hwsel_type)
  {
    w1=Create_Radio(LAB_DEC,TRUE,Record_Hrpt_Commands);
    w2=Create_Radio(LAB_GEN,FALSE,Record_Hrpt_Commands);
    we=Pack("Hardware select",'h',w1,0,w2,0,NULL);
  }
  

/* --------------------- SHOW DURING REC -------------- */
  w1=Create_Optionmenu(LAB_VCHR,Record_Hrpt_Commands,rdi->sh_recchan-1,
                       LAB_CH1_V,LAB_CH2_V,LAB_CH3_V,LAB_CH4_V,LAB_CH5_V,
                       LAB_CH6_V,LAB_CH7_V,LAB_CH8_V,LAB_CH9_V,LAB_CH10_V,
                       0);


  w2=Create_Spin(LAB_MAXY,Record_Hrpt_Commands,"%d%d%d",rdi->height,1,10000);
  wf=Pack("Show during record",'h',w1,5,w2,1,NULL);


/* --------------------SAVE----------------------- */
  w1=Create_Check(LAB_Save,Record_Hrpt_Commands,rdi->save_flag);
  w2=Create_Check(LAB_KEP,Record_Hrpt_Commands,rdi->auto_get_kepler);
  wg=Pack("",'h',w1,1,w2,2,NULL);


/* --------------------Advanced / Exit----------------------- */
  wh=Create_Button(LAB_Exit,Record_Hrpt_Commands);
  wq=Create_Toggle(LAB_ADVANCED,Record_Hrpt_Commands,FALSE);


/* --------------------PACK----------------------- */
  wx=Pack(NULL,'h',wa,2,wb,1,NULL);
  wx=Pack(NULL,'v',wx,5,wc,5,wd,1,NULL);

  if (rdi->hwsel_type)
  {
    wy=Pack(LAB_ADV_SET,'v',wf,5,wg,1,NULL);
  }
  else
  {
    wy=Pack(LAB_ADV_SET,'v',we,5,wf,5,wg,1,NULL);
  }
  wz=SPack("","Hh",wq,"fe10",wh,"fe1",NULL);

  wz=Pack(NULL,'v',wx,5,wy,5,wz,1,NULL);

  gtk_container_add(GTK_CONTAINER(hrpt_window),wz);

  gtk_widget_show_all(hrpt_window);
  place_window(hrpt_window,0,0,prefs.sub_wnd_geom);

  // Get channel info from decoder.
  // Currently only supported for parport.
  if (rdi->port_type==parport)
  {
    if (Open_HWConnect(rdi))
    {                                 // no err returned -> OK
      if (get_hwselected(rdi->parport_base,rdi))
	rdi->sel_hw=FALSE;
      else
	rdi->sel_hw=TRUE;

      if (rdi->hwsel_type)
	Set_Button(hrpt_window,LAB_CHRPT,rdi->sel_hw);
      else
	Set_Button(hrpt_window,LAB_GEN,rdi->sel_hw);

      Set_Button(hrpt_window,LAB_GEN,rdi->gen1_dec0);
      Close_HWConnect(rdi);
    }
  }
  Sense_Button(main_window,MENU_HRI,FALSE);
  Sense_Button(main_window,MENU_RAWD,FALSE);

/* Restore show-channel-during-record state */
  Set_Button(hrpt_window,LAB_VCHR,rdi->sh_recchan-1);

  Record_Hrpt_Commands(wq, LAB_ADVANCED);
  Set_Button(hrpt_window,LAB_DIR_NSN,(rdi->scan_ver==S_N? TRUE : FALSE));
}

/*************************************
 * Record auto. 
 *************************************/
void Recordpp_Hrpt_Auto(GtkWidget *main_window,char *chan,SAT_TYPE sattype)
{
  GtkWidget *recwnd;
  char labstr[20];
  Menu_Rec_Hrpt(main_window);
  recwnd=Find_Window(main_window,Lab_Hrptwnd);
  Set_Button(recwnd,LAB_DEC,1);

  if (strchr(chan,'c')) sattype=CHRPT;

  if (sattype==CHRPT)
  {
    Set_Button(recwnd,LAB_CHRPT,1);
  }

  for (; *chan; chan++)
  {
    if (strchr("1234567890",*chan))
    {
      int ichan=*chan-'0';
      if (!ichan) ichan+=10;
      if (sattype==CHRPT)
      {
        ichan=(ichan-1)/2 + 1;
      }
      if ((ichan>=1) && (ichan<=5))
      {
        sprintf(labstr,"%s%d  ",LAB_CHX,ichan);
        Set_Button(recwnd,labstr,1);
/* Toggle buttons: func NOT activated using Set_Button, so do it here! */
        Record_Hrpt_Commands(Find_Widget(recwnd,LAB_Start), labstr);
      }
    }
    else if (*chan=='a')
    {
/* Do 'all-channel select'. (Toggle-button; don't click it extra!) */
      Set_Button(recwnd,LAB_CHA,1);
    }
    else if (*chan=='g')
    {
      Set_Button(recwnd,LAB_GEN,1);
    }
    else
      continue;
  }

  Set_Button(recwnd,LAB_Start,1);
/* Toggle buttons: func NOT actibated using Set_Button, so do it here! */
  Record_Hrpt_Commands(Find_Widget(recwnd,LAB_Start), LAB_Start);
}
