#include <malloc.h>
#include "wsat.h"
#include "hrpt_funcs.h"

#ifdef __ADD_ETH__ 

#ifdef __WIN32__
#define MSG_DONTWAIT 0
#endif

#define GENERATORn
#define ATMEGAn

#define PBUFLEN 140
/*
11: in-sync
10: start-frame
9...0: data
*/
#define STARTFRAME_ETH(reg) ((reg & 0x0400))
#define INSYNC_ETH(reg) ((reg & 0x0800))
int eth_wait_for_insync(RECORDINFO *rdi,int threshold)
{
  return 1;
}

int eth_wait_for_outsync(RECORDINFO *rdi,int threshold)
{
  return 0;
}

#define UDPPORT 0x0400
int eth_init_connect(RECORDINFO *rdi)
{
  SOCKADDR_IN addr;
  unsigned short iPort = UDPPORT;
  int prot='u';
  if (rdi->eth.s) return rdi->eth.s;
  addr.sin_family = AF_INET;
  addr.sin_port = htons(iPort);
  addr.sin_addr.s_addr = INADDR_ANY;
#ifdef __WIN32__
  {
    WSADATA wsda;
    WSAStartup(MAKEWORD(1,1), &wsda);
  }
  if (prot=='u')
  {
    unsigned long iMode=1;
    rdi->eth.s=socket(AF_INET,SOCK_DGRAM, IPPROTO_UDP);
    if (rdi->eth.s == SOCKET_ERROR) return -1;
    ioctlsocket(rdi->eth.s,FIONBIO,&iMode);
    if ((bind(rdi->eth.s, (struct sockaddr *)&addr, sizeof(addr)))== SOCKET_ERROR) return -1;
  }
  else
#endif
  {
#ifndef __WIN32__
    rdi->eth.s=socket(AF_PACKET,SOCK_RAW,htons(ETH_P_ALL));
#endif
  }

  if (!rdi->eth.pbuf)
    rdi->eth.pbuf = (void*)malloc(PBUFLEN+10); /* buffer for ethernet frame */
  rdi->eth.prot='u';

  return rdi->eth.s;
}

/*
 in buffer:
   6+6+2=14 bytes header
   1 byte frame counter
   1 byte packet counter
   max. 1110 data-bytes = 555 words 
*/
int get_eth_data(RECORDINFO *rdi,int *word,int *fn,int *pn)
{
  static int nrbytes;
  static int length;
  static unsigned char *data;
  SOCKADDR_IN remote_addr;
  ETH *eth=&rdi->eth;
  int iRemoteAddrLen = sizeof(remote_addr);
  if (!word)
  {
    nrbytes=0;
    return 0;
  }

  if (!nrbytes)
  {
    int done=0;
    while (!done)
    {
      while ((eth->plen = recvfrom(eth->s, eth->pbuf, PBUFLEN, MSG_DONTWAIT, 
                                     (struct sockaddr *) &remote_addr, &iRemoteAddrLen))<0)
      {

        while (g_main_iteration(FALSE));
        if (!rdi->recording) break;
      }
      if (!eth->plen) return 0;

      #if __GTK_WIN32__ == 1
        data=rdi->eth.pbuf;
        length=eth->plen;
        done=1;
      #else
        #define UDP_THDRLEN 42
        #define ETH_THDRLEN 14
        if (eth->prot=='u')
        {
          if (eth->pbuf[14+9]==0x11)    // protocol UDP
          {
            data=rdi->eth.pbuf+UDP_THDRLEN;
            length=eth->plen-UDP_THDRLEN;
            done=1;
          }
        }
        else
        {
          data=rdi->eth.pbuf+ETH_THDRLEN;
          length=eth->plen-ETH_THDRLEN;
          done=1;
        }
      #endif
    }
    *pn=data[0];       // packetnumber
    *fn=data[1];       // framenumber
    nrbytes=length-2;  // 
  }
#ifdef GENERATOR
  *word=(data[length-nrbytes])<<8;
  nrbytes--;
  if (nrbytes>0)
  {
    (*word)|=(((data[length-nrbytes])&0xff));
    nrbytes--;
  }
#else
  {
  // 0x0400=fs
  // 0x0800=!oof
    unsigned char d0,d1;
    d0=(data[length-nrbytes])&0xff;
    nrbytes--;
    if (nrbytes>0)
    {
      d1=(data[length-nrbytes])&0xff;
      nrbytes--;
    }
#ifdef ATMEGA
    *word=((d0&0x78)<<3) + ((d0&0x03)<<4) + ((d1&0x1e)>>1);
    if (d0&0x80) (*word)+=0x0400;
    if (d1&0x01) (*word)+=0x0800;
#else
    *word=(d1<<8)+d0;
#endif
  }
#endif
//printf("%d  word=%x\n",nrbytes,*word);
  return 1;
}

int eth_get_hrpt_frame(guint16    *frame,RECORDINFO *rdi)
{
  int datstat;
  int words_remaining;
  gboolean start_frame_flag=FALSE;
  int cnt_nosig=0;
  int fn,pn;
  get_eth_data(rdi,NULL,NULL,NULL);

  words_remaining=rdi->fri.words_per_frame;
  while (words_remaining > 0)              /* Read all words in a frame */
  {
/* Get data from parallel port */
    if (!(get_eth_data(rdi,&datstat,&fn,&pn)))
    {
      return 0;                            /* Signal lost! */
    }

/* Check if received word contains a frame sync pulse from decoder.
   If sync pulse received, this is the first word of a frame.
   (Normally, this is the case each time getpp_hri_frame is entered.)
   NOTE This is NOT the decoder in-sync flag!
*/
    if (STARTFRAME_ETH(datstat))
    {
      if (start_frame_flag) break;
//printf("START: %x\n",datstat);
/* Now, enable to save remaining words of frame. */
      start_frame_flag=TRUE;
    }

/* Save remaining words of frame. */
    if (start_frame_flag)
    {
      *frame=datstat & 0x3ff;
      frame++;
      words_remaining--;
      cnt_nosig=0;
    }
    else
    {
/* If no readyn: make sure that now and then gui is activated */
      cnt_nosig++;
      if (cnt_nosig>1000)
      {
        while (g_main_iteration(FALSE));
        if (!rdi->recording) break;
        cnt_nosig=0;
      }
    }
  }
//printf("FR=%x  %x\n",frame1[0],frame1[1]);
  if (!INSYNC_ETH(datstat)) return 2;        /* decoder not in-sync. */
  return 1;                                 /* decoder in-sync. */
}

#endif

// syncwoord: 0x284,0x16f,0x35c,0x19d,0x20f,0x095
// = 1010000100 0101101111 1101011100 0110011101 1000001111 0010010101
// opslaan in rah: (start bytenr 0x3e87)
// a1 5b d7 67 31     83 25 xx xx dx 
// 2e: 63cb

// syncwoorden: 3e87, 74b7 aae7 ...  (interval 0x3630=13872)
// 1e data: +937           ae90
