#include <malloc.h>
#include "wsat.h"
#include "hrpt_funcs.h"

#ifndef __ADD_USB__ 
int usb_list_ndevs(FT_DEVICE_LIST_INFO_NODE **devInfo)
{
  return 0;
}
#endif
#ifdef __ADD_USB__ 

#define STARTFRAME_USB(reg) ((reg & 0x0400))
#define INSYNC_USB(reg) ((reg & 0x0800))

int usb_list_ndevs(FT_DEVICE_LIST_INFO_NODE **devInfo)
{
  FT_STATUS ftStatus;
  DWORD numDevs;
  ftStatus = FT_CreateDeviceInfoList(&numDevs);
  if (ftStatus != FT_OK) return -1;
  if (numDevs > 0)
  {
    // allocate storage for list based on numDevs
    *devInfo = (FT_DEVICE_LIST_INFO_NODE*)calloc(sizeof(FT_DEVICE_LIST_INFO_NODE),numDevs);
    // get the device information list
    ftStatus = FT_GetDeviceInfoList(*devInfo,&numDevs);

    if(ftStatus != FT_OK)
    {
      free(*devInfo);
      *devInfo=NULL;
      return -1;
    }
  }
  return numDevs;
}

// return first in list
char *usb_list_devs()
{
  FT_STATUS ftStatus;
  int iNumDevs = 0;
  char *bufpntr[2];
  static char buf0[64];
  bufpntr[0]=buf0;
  bufpntr[1]=NULL;
  *buf0=0;
  ftStatus = FT_ListDevices(bufpntr, &iNumDevs, FT_LIST_ALL | FT_OPEN_BY_SERIAL_NUMBER);
  if(ftStatus != FT_OK)
  {
    return NULL;
  }
  return buf0;
}

int usb_init_connect(RECORDINFO *rdi)
{
  FT_HANDLE ftHandle;
  FT_STATUS ftStatus;
  char *ftd_id=(rdi->usb_use_id?rdi->usbport_id : NULL);

  if (rdi->ftHandle) return 0;
  if (!ftd_id)
  {
    if (!(ftd_id=usb_list_devs()))
    {
      return -1;
    }
  }
  if((ftStatus = FT_OpenEx(ftd_id, FT_OPEN_BY_SERIAL_NUMBER, &ftHandle)) != FT_OK)
  {
    return -1;
  }
  ftStatus=FT_SetTimeouts(ftHandle,1000,1000);
  rdi->ftHandle=ftHandle;
  return 0;
}

void usb_close_connect(RECORDINFO *rdi)
{
  if (rdi->ftHandle) FT_Close(rdi->ftHandle);
  rdi->ftHandle=0;
}


#define BUF_SIZE 1024
#define MUXSELECTBIT 0x80
static int get_usb_data(RECORDINFO *rdi,int *word)
{
  static int nrbytes;
  static DWORD dwBytesRead;
  static unsigned char pcBufRead[BUF_SIZE+10],inp;
  int done;
  int nn=0;
  static int hword;

  FT_STATUS ftStatus;
  FT_HANDLE ftHandle=rdi->ftHandle;
  if (!word)
  {
    nrbytes=0;
    return 0;
  }

  done=0;
  while (!done)
  {
    if (!nrbytes)
    {
      ftStatus = FT_Read(ftHandle, pcBufRead, BUF_SIZE, &dwBytesRead);
      if ((ftStatus != FT_OK) || (dwBytesRead<BUF_SIZE))
      {
        while (g_main_iteration(FALSE));
        if (!rdi->recording) break;
      }
      nrbytes=dwBytesRead;
    }
    if (nrbytes>0)
    {
      inp=pcBufRead[dwBytesRead-nrbytes];
      nn++;
      if (!(inp&MUXSELECTBIT))
      {
        hword=(inp&0x0f)+((inp&0x30)<<4);
	*word=0;
      }
      else
      {
        *word=hword | ((inp&0x0f)<<4) | ((inp&0x30)<<6);
        done=1;
      }
      nrbytes--;  // 
    }
  }
//if (nn!=2) printf("nn=%d\n",nn);
  return 1;
}

gboolean usb_wait_for_insync(RECORDINFO *rdi,int threshold)
{
  int datstat;
  guint32 count=0;
  guint32 threshold_word;
  int n=0;
  int was_open=(int)rdi->ftHandle;

  if ((!was_open) && (usb_init_connect(rdi))) return FALSE;
/* Translate #frames-to-wait to #words-to-wait */
  threshold_word=threshold*rdi->fri.words_per_frame;
  do
  {
    if (!rdi->recording)
    {
      if (!was_open) usb_close_connect(rdi);
      return FALSE;           /* Stop if recording aborted */
    }
    if ((get_usb_data(rdi,&datstat)))
    {
      if (INSYNC_USB(datstat))
        count++;
      else
        count=0;
    }
    else
    {
/*
   waited too long for new data; problem with readyn? 
   Try again,but first do g_main_iteration to catch e.g. 'Stop' event.
*/
    }

/* Evaluate gui-events now and then. (Not too often.) */
    if (n++>10000)
    {
      while (g_main_iteration(FALSE));
      n=0;
    }
  } while (count<threshold_word);

  if (!was_open) usb_close_connect(rdi);
  return TRUE;
}

int usb_wait_for_outsync(RECORDINFO *rdi,int threshold)
{
  int datstat;
  guint32 count=0;
  guint32 threshold_word;
  int n=0;
  int was_open=(int)rdi->ftHandle;

  if (usb_init_connect(rdi)) return FALSE;
/* Translate #frames-to-wait to #words-to-wait */
  threshold_word=threshold*rdi->fri.words_per_frame;
  do
  {
    if (!rdi->recording)
    {
      if (!was_open) usb_close_connect(rdi);
      return FALSE;           /* Stop if recording aborted */
    }
    if ((get_usb_data(rdi,&datstat)))
    {
      if (!INSYNC_USB(datstat))
        count++;
      else
        count=0;
    }
    else
    {
/*
   waited too long for new data; problem with readyn? 
   Try again,but first do g_main_iteration to catch e.g. 'Stop' event.
*/
    }

/* Evaluate gui-events now and then. (Not too often.) */
    if (n++>100)
    {
      while (g_main_iteration(FALSE));
      n=0;
    }
  } while (count<threshold_word);

  if (!was_open) usb_close_connect(rdi);
  return TRUE;
}

int usb_get_hrpt_frame(guint16    *frame,RECORDINFO *rdi)
{
  int datstat;
  int words_remaining;
  gboolean start_frame_flag=FALSE;
  int cnt_nosig=0;

  words_remaining=rdi->fri.words_per_frame;
  while (words_remaining > 0)              /* Read all words in a frame */
  {
/* Get data from parallel port */
    if (!(get_usb_data(rdi,&datstat)))
    {
      return 0;                            /* Signal lost! */
    }

/* Check if received word contains a frame sync pulse from decoder.
   If sync pulse received, this is the first word of a frame.
   (Normally, this is the case each time getpp_hri_frame is entered.)
   NOTE This is NOT the decoder in-sync flag!
*/
    if (STARTFRAME_USB(datstat))
    {
      if (start_frame_flag) break;
//printf("START: %x\n",datstat);
/* Now, enable to save remaining words of frame. */
      start_frame_flag=TRUE;
    }

/* Save remaining words of frame. */
    if (start_frame_flag)
    {
      *frame=datstat & 0x3ff;
      frame++;
      words_remaining--;
      cnt_nosig=0;
    }
    else
    {
/* If no readyn: make sure that now and then gui is activated */
      cnt_nosig++;
      if (cnt_nosig>1000)
      {
        while (g_main_iteration(FALSE));
        if (!rdi->recording) break;
        cnt_nosig=0;
      }
    }
  }
//printf("FR=%x  %x\n",frame1[0],frame1[1]);
  if (!INSYNC_USB(datstat)) return 2;        /* decoder not in-sync. */
  return 1;                                 /* decoder in-sync. */
}

// syncwoord: 0x284,0x16f,0x35c,0x19d,0x20f,0x095
// = 1010000100 0101101111 1101011100 0110011101 1000001111 0010010101

#ifdef TODO: channel selection
int usb_write(RECORDINFO *rdi)
{
  FT_STATUS ftStatus;
  FT_HANDLE ftHandle=rdi->ftHandle;
  DWORD dwBytesToWrite=1;
  static unsigned char pcBufWrite[BUF_SIZE+10];
  DWORD lpdwBytesWritten;
/*
  usb_init_connect(rdi);
ftHandle=rdi->ftHandle;
//while (1)
{
//  ftStatus=FT_Write(ftHandle, pcBufWrite,dwBytesToWrite, &lpdwBytesWritten);
printf("%d  %d  %d  lpdwBytesWritten=%d\n",ftHandle,ftStatus,pcBufWrite[0],lpdwBytesWritten);
  pcBufWrite[0]++;
//sleep(1);
}
usb_close_connect(rdi);
*/
}

#endif

#endif

