/**************************************************
 * RCSId: $Id$
 *
 * Old HRPT format definitions
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log$
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 WSAT. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/
#ifndef HRPT1_HDR
#define HRPT1_HDR

#include "wsat.h"

/* Header signs */
#define LENSIGN_H 4
#define HRPT1SIGN  "HRPT"
#define CHRPT1SIGN "CHRP"
#define SEAW1SIGN  "SEAW"

/* Header version */
#define HRPTHDRVERSION "EXT_V004"

/* End of header sign*/
#define ENDFLG 8
#define ENDHDR "END_HDR_"

/* nr. bytes between 2 frames */
#define NR_INTER_BYTES 7

/* Code between 2 frames */
#define FRAMESTART "FRAME"

/* Error codes in HRPT1 file format */
#define MASK_HRPT1_READ_OK_PRE  0x01
#define MASK_HRPT1_REC_OK_PRE   0x02
#define MASK_HRPT1_SYNC_OK_PRE  0x0c
#define MASK_HRPT1_REC_OK_POST  0x10
#define MASK_HRPT1_SYNC_OK_POST 0xe0

/* Max. length sat-name */
#define LENSATNAME_HDR1 16

/* Length reserved for color maps into header */
#define CMAPLEN10B 1024

/* Time info with 2 byte integers */
struct tm_sh
{
  gint16 tm_sec,tm_min,tm_hour;
  gint16 tm_mday,tm_mon,tm_year,tm_wday,tm_yday,tm_isdst;
};

/* Kepler info as saved in HRPT header */
typedef struct 
{
  gint32 noradnr;
  gint32 internat;
  gint32 epoch_year;
  float epoch_day;
  float decay_rate;
  float inclination;
  float raan;
  float eccentricity;
  float perigee;
  float anomaly;
  float motion;
  gint32 epoch_rev;
} KEPLER1;


/* Definition HRPT header.
   NOTE: char=1 byte, float=4 bytes
*/
typedef struct
{
  char sign[LENSIGN_H];
  char sat_name[LENSATNAME_HDR1];
  char kan_tot;
  char filetype;
  gint16 width,height;
  gint16 depth;
  char offset_gmt,offset_sec;
  struct tm_sh tm_comp;
  KEPLER1 kepler;

  char hdr_version[ENDFLG];
  gint16 hdrlen;
  gint16 x_offset,y_offset;
  char satdir,reserve2;

  struct tm_sh tm_satstrt;
  char tm_comp_sat_vlg;

  float offset_lon;
  gint16 dmin[8],dmax[8];
  gint16 ro,go,bo;
  float kr[8],kg[8],kb[8];
  char channel[9];
  gint16 widthmax;
  float alphamax;
  char r[5][CMAPLEN10B],g[5][CMAPLEN10B],b[5][CMAPLEN10B];
  char reserve[348];
  char endhdr[ENDFLG];
} HRPTHDR;

/* Definition CHRPT header.
   NOTE: char=1 byte, short=2 bytes, float=4 bytes
*/
typedef struct
{
  char sign[LENSIGN_H];
  char sat_name[LENSATNAME_HDR1];
  char kan_tot;
  char filetype;
  gint16 width,height;
  gint16 depth;
  char offset_gmt,offset_sec;
  struct tm_sh tm_comp;
  KEPLER1 kepler;

  char hdr_version[ENDFLG];
  gint16 hdrlen;
  gint16 x_offset,y_offset;
  char satdir,reserve2;

  struct tm_sh tm_satstrt;
  char tm_comp_sat_vlg;

  float offset_lon;
  gint16 dmin[10],dmax[10];
  gint16 ro,go,bo;
  float kr[10],kg[10],kb[10];
  char channel[11];
  gint16 widthmax;
  float alphamax;
  char r[5][CMAPLEN10B],g[5][CMAPLEN10B],b[5][CMAPLEN10B];
  char reserve[330];
  char endhdr[ENDFLG];
} CHRPTHDR;

/* Definition hrpt1-related functions */
HRPTHDR *read_hrpt1hdr();
int read_hrpt1_hdr();
int read_hrpt1_line();
int write_hrpt1_hdr();
int write_hrpt1_line();

#endif
