/**************************************************
 * RCSId: $Id$
 *
 * HRPT recording via parallel port
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log$
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 WSAT. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/
#ifndef PP_DEC_INTERFACE_HDR
#define PP_DEC_INTERFACE_HDR
#define PAR_ERR_MESSAGE "Can't access \nParallel port with address %x !"


/*************************************
 * Masks for parallel port registers
 * NOTE: Following bits are inverted in the parallel port hardware:
 * control reg: PIN_17, PIN_14,  PIN_1
 * status reg:  PIN_11
 *************************************/

/* Pin masks control registers parallel port */
#define PIN_1  0x01               /* PP Strobe */
#define PIN_14 0x02               /* PP Auto feed */
#define PIN_16 0x04               /* PP Init */
#define PIN_17 0x08               /* PP Select in */
#define PCD    0x20               /* EPP: Parallel Control Direction (0=out) */          

/* Pin masks status register parallel port */
#define PIN_15 0x08               /* PP Error */
#define PIN_13 0x10               /* PP Select */
#define PIN_12 0x20               /* PP Paper end */
#define PIN_10 0x40               /* PP Ackn */
#define PIN_11 0x80               /* PP Busy */

/* Pin masks status register if catched with 16 bits read command */
#define PIN_10X (PIN_10 << 8)
#define PIN_11X (PIN_11 << 8)
#define PIN_15X (PIN_15 << 8)

/* Pin masks data register parallel port */
#define PIN_9 0x80
#define PIN_8 0x40
#define PIN_7 0x20
#define PIN_6 0x10
#define PIN_5 0x08
#define PIN_4 0x04
#define PIN_3 0x02
#define PIN_2 0x01


/*************************************
 * Mapping low level record commands to parallel port pins
 *************************************/

/* HRPT Channel selection definition */
#define PP_CHAN1_SEL  PIN_8
#define PP_CHAN2_SEL  PIN_7
#define PP_CHAN3_SEL  PIN_6
#define PP_CHAN4_SEL  PIN_5
#define PP_CHAN5_SEL  PIN_4
#define PP_FRHDR_SEL  PIN_3

/* Definitions control signals */
#define R_CLKN        PIN_1       /* read-clock; 1->0 ==> read */
#define OEN           PIN_14      /* output enable; active low */
#define WEN           PIN_16      /* write chan. select to decoder; active 0 */
#define DEC1_GEN0     PIN_17      /* select decoder (1) / generator (0) */


/*************************************
 * Mapping higher level to low level record commands
 *************************************/
/* Set transmission from par. port to decoder */
#define PP2DEC(dg) (((R_CLKN | WEN) & ~OEN  & ~PCD) | (dg? 0 : DEC1_GEN0))

/* Set transmission from decoder to par. port */
#define DEC2PP(dg)  ((R_CLKN | WEN  |  OEN  |  PCD) | (dg? 0: DEC1_GEN0))

/* Strobe on/off commands (tell decoder data is read) */
#define STROBE_ON(dg) (DEC2PP(dg))
#define STROBE_OFF(dg) (DEC2PP(dg) & ~R_CLKN)

/* Write on/off commands (for channel select) */
#define WRI_ON(dg)  (PP2DEC(dg) & ~WEN)
#define WRI_OFF(dg) (PP2DEC(dg))

/*************************************
 * Mapping highest level to higher level record commands
 *************************************/
/* Read data + status from parallel port register */
#define GET_DS_PP(d,dreg) d=inw(dreg)

/* Determine if catched data is valid */
#define READYN_PP(reg) (reg & PIN_10X)

/* Send strobe: read from pp finished */
#define STROBE_PP(sel_hw,creg)                  \
                  outb(STROBE_ON(sel_hw),creg); \
                  outb(STROBE_OFF(sel_hw),creg)

/* Determine if catched data is start of frame */
#define STARTFRAME_PP(reg) (!(reg & PIN_11X))

/* Determine if in-sync */
#define INSYNC_PP(reg) ((reg & PIN_15X))

/* Write channel select. For fast par-port some delays are added. */
#define SEL_CHAN_PP(channels,sel_hw,dreg,creg)   \
                    outb(channels,dreg);         \
                    sleep_ms(1);                 \
                    outb(WRI_ON(sel_hw),creg);   \
                    sleep_ms(1);                 \
                    outb(WRI_OFF(sel_hw),creg);  \
                    sleep_ms(1);                 \
                    outb(0xff,dreg)


#endif
