/**************************************************
 * RCSId: $Id$
 *
 * Channel select related functions 
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log$
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 R. Alblas. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/

#include "wsat.h"
#include "menu_wsat.h"
#include <malloc.h>

extern PREFS prefs;

/*************************************
 * Update channel selection for active file.
 *************************************/
void Channel_Select(GtkWidget *window,
                    PICINFO   *pci,
                    VIEWINFO  *vwi)
{
  int i;
  gboolean selected=FALSE;

/* Deselect channels not recorded. */
  for (i=0; i<NR_CHANNELS_MAX; i++)
  {
    if (!pci->channels_recorded[i])
      vwi->channel_selected[i]=FALSE;
  }

/* Determine if a channel is selected */
  for (i=1; i<NR_CHANNELS_MAX; i++)
  {
    if (vwi->channel_selected[i]) { selected=TRUE; break; }
  }

/* If no channels selected determine which channel to select */
  if (!selected)
  {
    for (i=1; i<NR_CHANNELS_MAX; i++)
    {
      if (pci->channels_recorded[i])
      {
        vwi->channel_selected[i]=TRUE;

/* HRI can have different sizes for each channel. 
   Load size of selected channel 
*/
        if (pci->file_type==HRI1)
        {
          pci->width=pci->hri_pci.channel_info[i].width;
          pci->height=pci->hri_pci.channel_info[i].height;
        }
        break;
      }
    }
  }

  Update_channel_state(window,pci,vwi,MENU_CH,"");
}

void Update_channel_state(GtkWidget *window,
                          PICINFO *pci,
                          VIEWINFO *vwi,
                          char *name_prefix,
                          char *name_suffix)
{
  GtkWidget *button;
  char tmp[20];
  int i;

/* Update channel buttons */
  if (window)
  {
    for (i=1; i<NR_CHANNELS_MAX; i++)                  /* For all channels */
    {
      sprintf(tmp,"%s%d%s",name_prefix,i,name_suffix); /* Construct button name */
      if ((button=Find_Widget(window,tmp)))            /* Get button widget */
      {
        if ((pci) && (pci->channels_recorded[i]))      /* If channel recorded */
          gtk_widget_set_sensitive(button,TRUE);       /* Then Enable button */
        else
          gtk_widget_set_sensitive(button,FALSE);      /* Else Disable button */

        if ((vwi) && (vwi->channel_selected[i]))       /* If channel selected */
          Set_Button(window,tmp,TRUE);                 /* Then set button */
      }
    }
  }
}


/*************************************
 * Select channel to view
 * Activated by menu channel items
 *************************************/
void Menu_Channel(GtkWidget *widget,
                  gpointer  callback_data,
                  guint     callback_action)
{
  VIEWINFO *vwi=Get_Viewinfo_db(widget);
  PICINFO *pci=Get_Picinfo_db(widget);
  int nstate=GTK_CHECK_MENU_ITEM(widget)->active;
  char *sel_name;
  char chan_name[20];
  int i;

  sel_name=(char *)gtk_object_get_data(GTK_OBJECT(widget),WDGT_ID);

/* Copy channel button states to channel_selected */
  for (i=0; i<NR_CHANNELS_MAX; i++)
  {
    sprintf(chan_name,"%s%d",MENU_CH,i);
    if (!strcmp(sel_name,chan_name))
    {
      vwi->channel_selected[i]=(nstate? TRUE: FALSE);
    }
  }
  if (nstate) Update_Main(widget);
}

/*************************************
 * Menu-item 'sat-info'
 *************************************/
#define LAB_Prev "Prev"
#define LAB_Next "Next"
#define LAB_FRNR "Frame nr"

void Menu_Info(GtkWidget *widget,gpointer data)
{
  GtkWidget *window=Find_Parent_Window(widget);
  VIEWINFO *vwi=Get_Viewinfo_db(widget);
  PICINFO *pci=Get_Picinfo_db(widget);
  char *name=(char *)data;

  GtkWidget *info,*hdr_p,*hdr_n,*hdr_f,*hdr=NULL;
  static GtkWidget *info_hold;
  static int nr_frm;
  int i;
  int fs=prefs.font_size;

  if (!(Find_Window(window,"Sat-info")))
  {
    hdr_p=Create_Button(LAB_Prev,Menu_Info);
    hdr_n=Create_Button(LAB_Next,Menu_Info);
    hdr_f=Create_Entry(LAB_FRNR,NULL,"%-5d",0);
    hdr=Pack("Frame header",'h',hdr_p,5,hdr_n,5,hdr_f,5,NULL);
  }

  if ((info=Create_Info(widget,"Sat-info",400,300,Menu_Info,hdr)))
  {
    nr_frm=0;
    info_hold=info;
  }
  else                       /* info already exist; use prev. defined one */
  {
    info=info_hold;
    if (!info) return;
    Clear_Text(info);
  }
  Add_Text(info,fs," File      : %s\n",pci->file_name);
  Add_Text(info,fs," Type      : %s",(pci->file_type==HRI1?    "HRI":
                                     pci->file_type==HRPT1?   "HRPT":
                                     pci->file_type==CHRPT1?  "CHRPT":
                                     pci->file_type==SEAWIF1? "Seastar":
                                     pci->file_type==GIF?     "GIF":
                                     pci->file_type==SUN?     "SUN raster":
                                     pci->file_type==PPM?     "PPM (24 bits)":
                                     pci->file_type==PGM?     "PGM ":
                                                              "Unknown"));
  if (pci->file_type==PGM) Add_Text(info,fs,"(%d bits)",pci->depth);
  Add_Text(info,fs,"\n");

  if (pci->file_type==unknown_filetype) return;
  if ((IS_HRPT_FAM(pci->file_type)) || (pci->file_type==HRI1))
  {
    Add_Text(info,fs," Satellite : %s\n",pci->sat_name);
    if (pci->picture_time.tm_year)
      Add_Text(info,fs," Picture time: %s\n",asctime(&pci->picture_time));

    Add_Text(info,fs," Sat time  : %s",asctime(&pci->orbit.start_tm_sat));
    Add_Text(info,fs," Rec. time : %s",asctime(&pci->orbit.start_tm_comp));
    Add_Text(info,fs," Used time : %s",asctime(&pci->orbit.start_tm));

    Add_Text(info,fs," Channels  :");
    for (i=0; i<10; i++)
      if (pci->channels_recorded[i])
        Add_Text(info,fs," %d",i);
    Add_Text(info,fs,"\n");
    Add_Text(info,fs," Offset    : %4d x %4d\n",pci->ox,pci->oy);
  }
  Add_Text(info,fs," Size      : %4d x %4d\n",pci->width,pci->height);

  if (!pci->channels_recorded[0])
  {
    if (IS_HRPT_FAM(pci->file_type))
      Add_Text(info,fs," No frame header recorded.\n");
    return;
  }
  if (!strcmp(name,LAB_Prev)) nr_frm--;
  if (!strcmp(name,LAB_Next)) nr_frm++;
  nr_frm=MAX(nr_frm,0);
  Set_Entry(window,LAB_FRNR,"%d",nr_frm);

  if ((IS_HRPT_FAM(pci->file_type)) || (pci->file_type==HRI1))
  {
    if ((!strcmp(name,LAB_Prev)) || (!strcmp(name,LAB_Next)))
    {
      guint16 *line;
      Add_Text(info,fs," ---------------------------------\n");

      line=(guint16 *)calloc(pci->fri.words_per_line,2);
      if (!(read_line(pci,vwi,nr_frm,line,NULL,NULL)))
      {
        Add_Text(info,fs," End of file.\n");
        nr_frm--;
        Set_Entry(window,LAB_FRNR,"%d",nr_frm);
        return;
      }
      Add_Text(info,fs," Frame     : %d\n",nr_frm);
      Add_Text(info,fs," Sync      : %x %x %x %x %x %x\n",
                        line[0],line[1],line[2],line[3],line[4],line[5]);

      switch(pci->file_type)
      {
        case CHRPT1:
          Add_Text(info,fs," Space craft address: %x --> ",(line[6]>>6)&0xf);
          Add_Text(info,fs,"%s\n",xrptid2name(line[6],CHRPT));

        break;
        case HRPT1:
          Add_Text(info,fs," Space craft address: %x --> ",(line[6]>>3)&0xf);
          Add_Text(info,fs,"%s\n",xrptid2name(line[6],HRPT));

          if (!(line[6]&0x2))
          {
            if (line[6]&0x1)
              Add_Text(info,fs," Channel 3 type: Ch3B");
            else
              Add_Text(info,fs," Channel 3 type: Ch3A");
          }
          Add_Text(info,fs,"\n");
        break;
        default:
        break;
      }

      {
        int day=line[8]>>1;
        gint32 msec=((long)(line[9]&0x7f))<<20 | ((long)(line[10])<<10) | line[11];
        gint32 sec=msec/1000;
        gint32 min=sec/60;
        gint32 uur=sec/3600;
        min=min-(uur*60);
        sec=sec-(min*60)-(uur*3600);
        Add_Text(info,fs,"\n");
        Add_Text(info,fs," TIME     : day %d  %ld:%ld:%ld",
             day,uur,min,sec);
      }

      free(line);
    }
  }
}
