/**************************************************
 * RCSId: $Id$
 *
 * Show sat-position and draw lon/lat lines
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log$
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 R. Alblas 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/

#include <math.h>
#include <time.h>
#include <malloc.h>

#include "wsat.h"
#include "menu_wsat.h"
#include "orbit.h"
#include "hrpt.h"


/*************************************
 * Update coordinates mouse position
 *************************************/
void Update_Coord(GtkWidget *wnd,int h,int v,PICINFO *pci,VIEWINFO *vwi)
{
  GtkWidget *drawing_area=Find_Widget(wnd,"GTK_DRAWING_AREA");
  RGBI *rgbi=Get_RGBI(drawing_area);
  WPOINT pos_rel;
  GdkColor clr;
  int x,y,z;

/* Translate position in window to position in picture */
  x=scrx2picx(h,vwi,pci);
  y=scry2picy(v,vwi,pci);

/* Calculate current position and show. If failed erase. */

  if ((pci->orbit.valid) && (xy2lonlat(x,y,pci,&pos_rel)) && (!vwi->coordxy))
  {
    Set_Entry(wnd,LAB_COORD,"%5.2f,%5.2f",R2D(pos_rel.lon),R2D(pos_rel.lat));
  }
  else /* Failed! Show x/y-values instead. */
  {
    Set_Entry(wnd,LAB_COORD,"[x=%d,y=%d]",x,y);
  }

/* Pixel value and position. */
  get_rgbpoint(rgbi,&clr,h,v);
  if ((IS_HRPTIR(vwi)) && (vwi->map_temp))
  {
    z=inv_maptemp(clr,vwi,rgbi->size);
    Set_Entry(wnd,LAB_PIXVAL,"%d C",z);
  }
  else if (vwi->gamma!=1.)
  {
    float lmin=pow((float)vwi->lum_min,1./vwi->gamma);
    float lmax=pow((float)vwi->lum_max,1./vwi->gamma);
    float fz;
    z=(clr.red+clr.green+clr.blue)/3;
    fz=z*(lmax-lmin)/rgbi->size+lmin;
    z=pow(fz,vwi->gamma);
    Set_Entry(wnd,LAB_PIXVAL,"%d",z);
  }
  else
  {
    z=(clr.red+clr.green+clr.blue)/3;
    z=z*(vwi->lum_max-vwi->lum_min)/rgbi->size+vwi->lum_min;
    Set_Entry(wnd,LAB_PIXVAL,"%d",z);
  }
}

/*************************************
 * Draw lon/lat lines
 * Method: Scan all pixels and test on lon/lat=value to draw
 *************************************/
void Draw_Lonlatpix(int h,int v,         /* current position in window */
                    WPOINT *pos,         /* current lon/lat */
                    WPOINT *pospx,       /* previous lon/lat in x-dir. */
                    WPOINT *pospy,       /* previous lon/lat in y-dir. */
                    int dlon,int dlat,   /* interval between 2 lon/lats to draw */
                    RGBI *rgbi,          /* contains drawing buffer */
                    GdkColor *grid_clr)  /* color to draw in */
{
  int ilon =(int)R2D(pos->lon);
  int ilonpx=(int)R2D(pospx->lon);
  int ilonpy=(int)R2D(pospy->lon);
  int ilat =(int)R2D(pos->lat);
  int ilatpx=(int)R2D(pospx->lat);
  int ilatpy=(int)R2D(pospy->lat);
  int max_abs_ilonx=MAX(ABS(ilon),ABS(ilonpx));
  int max_abs_ilony=MAX(ABS(ilon),ABS(ilonpy));
  int max_abs_ilatx=MAX(ABS(ilat),ABS(ilatpx));
  int max_abs_ilaty=MAX(ABS(ilat),ABS(ilatpy));
  GdkColor white;
  white.red=white.green=white.blue=0xff;

/* Detect if lon is passed which has to be drawn */
  if ( ((ilon!=ilonpx) && (!(max_abs_ilonx%dlon))) ||
       (SIGN(pos->lon)!=SIGN(pospx->lon)) ||
       ((ilon!=ilonpy) && (!(max_abs_ilony%dlon))) ||
       (SIGN(pos->lon)!=SIGN(pospy->lon)) )
  {
/* This position contains lon line to draw, so set pixel */

    draw_rgbpoint(rgbi,grid_clr,h,v);

/* Add value */
    if (v==1)
    {
      char tmp[10]; sprintf(tmp,"%d",nint(R2D(pos->lon)));
      draw_rgbstring(rgbi,&white,h+3,6,tmp);

    }
  }

/* Detect if lat is passed which has to be drawn */
  if ( ((ilat!=ilatpx) && (!(max_abs_ilatx%dlat))) ||
       (SIGN(pos->lat)!=SIGN(pospx->lat)) ||
       ((ilat!=ilatpy) && (!(max_abs_ilaty%dlat))) ||
       (SIGN(pos->lat)!=SIGN(pospy->lat)) )
  {
/* This position contains lat line to draw, so set pixel */

    draw_rgbpoint(rgbi,grid_clr,h,v);

/* Add value */
    if (h==1)
    {
      char tmp[10]; sprintf(tmp,"%d",nint(R2D(pos->lat)));
      draw_rgbstring(rgbi,&white,8,v+5,tmp);
    }
  }
}


/*************************************
 * Draw lon/lat lines
 * Method: Scan all pixels and test on lon/lat=value to draw
 *************************************/
int draw_lonlat(PICINFO *pci,VIEWINFO *vwi,GtkWidget *window)
{
  GtkWidget *drawing_area=Find_Widget(window,"GTK_DRAWING_AREA");
  RGBI *rgbi=Get_RGBI(drawing_area);
  int wnd_width =drawing_area->allocation.width;
  int wnd_height=drawing_area->allocation.height;
  WPOINT pos,pos_px,pos_py;
  int lonmin=9999,lonmax=-9999,latmin=9999,latmax=-9999;
  int dlon=1,dlat=1;
  int h,v;
  GdkColor *grid_clr=&vwi->grid_clr;
  WPOINT *pos_prevline;
/*
GdkFont *font=gdk_font_load("*-*-*-*-r-*--12-*");
*/

/* Abort if no valid Kepler data */
  if (!pci->orbit.valid) return 0;

  pos_prevline=(WPOINT *)malloc(wnd_width*sizeof(*pos_prevline));

/* Set cursor on "watch" */
  Create_Cursor(drawing_area,GDK_WATCH);

/* 1. Determine how many lon/lats to draw: 0/1/2/... or 0/2/4/.. or 0/5/10/... */
/* 1a. Determine min/max lon/lat in current view. Just look at 4 corners. */
  for (h=0; h<wnd_width; h+=(wnd_width-1))
  {
    for (v=0; v<wnd_height; v+=(wnd_height-1))
    {
      xy2lonlat(scrx2picx(h,vwi,pci),scry2picy(v,vwi,pci),pci,&pos);
      lonmin=MIN(lonmin,R2D(pos.lon));
      lonmax=MAX(lonmax,R2D(pos.lon));
      latmin=MIN(latmin,R2D(pos.lat));
      latmax=MAX(latmax,R2D(pos.lat));
    }
  }

/* 1b. Determine lon/lat interval: 1, 2 or 5 */
  dlon=ABS(lonmax-lonmin)/8; if (dlon<1) dlon=1; if (dlon>2) dlon=5;
  dlat=ABS(latmax-latmin)/8; if (dlat<1) dlat=1; if (dlat>2) dlat=5;

/* 2.
   Scan all h/v of current window 
   Compare lon/lat of each pix. with pix. left ("posx") and pix. above ("posy")
   to detect positions crossed by lon/lat lines.
   Scanning is done in x-dir; after 1 scan, y is incremented. 
   So, posx is simply previous pos, but for posy previous line is needed.
   This line is saved in pos_prevline.
 ........y..............
 .......xo..............

*/

/* 2a. To speed up: Increment by "DH" in X-direction.
       Increment in y-dir is 1; >1 is more difficult, and there is 
       not much improve because other calcs have now the upper hand.
*/ 
#define DH 5

/* 2b. Scan picture and draw. */
  for (v=0; v<wnd_height-1; v+=1)
  {
    int h1;
    WPOINT pos1,posp;
    if ((KeyPressed()) == GDK_Escape) break;    /* stop if escape pressed */

/* In h-dir: 2 loops; first: increment DH; second: fill the DH-gap.
   To make sure complete area is scanned the upper-limit
     of first loop is incremented with DH.
*/
    for (h1=0; h1<wnd_width-1+DH; h1+=DH)
    {
/* Calc. current lon/lat */
      xy2lonlat(scrx2picx(h1,vwi,pci),scry2picy(v,vwi,pci),pci,&pos1);

/* Interpolate to fill the DH-gap */
      if (h1) for (h=h1-DH; h<h1; h++)
      {
        if (h>=wnd_width) break; /* Run out of picture */

/* Interpolate lon/lat between the 2 by xy2lonlat calculated positions */
        pos.lon=posp.lon+(pos1.lon-posp.lon)*(h-(h1-DH))/DH;
        pos.lat=posp.lat+(pos1.lat-posp.lat)*(h-(h1-DH))/DH;

/* Copy lon/lat of pixel in prev. line (present in pos_prevline) to pos_py */
        pos_py=pos_prevline[h];  
/* Save current pos to pos_prevline, needed to process next line 
   (old pos_prevline not needed anymore) 
*/
        pos_prevline[h]=pos;   

/* Set a point if it is on a lon or lat line */
        if ((h>0) && (v>0))
        {
          Draw_Lonlatpix(h,v,&pos,&pos_px,&pos_py,dlon,dlat,rgbi,grid_clr);
        }
        pos_px=pos;
      }
      posp=pos1;
    }

/* After 1 line is drawn: Make it visible */
    Refresh_Rect(drawing_area,0,v,wnd_width,1);
  }

/* Ready, reset cursor. */
  Create_Cursor(drawing_area,GDK_TOP_LEFT_ARROW);
  free(pos_prevline);
  return 1;
}


