/**************************************************
 * RCSId: $Id$
 *
 * Menu/gui for changin/saving orbit/kepler vars.
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log$
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 R. Alblas 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/

#include <string.h>
#include <math.h>
#include "wsat.h"
#include "menu_wsat.h"
#include "orbit.h"
#include "hrpt1.h"
#include "hrpt.h"

extern PREFS prefs;

/*************************************
 * Copy current Kepler/Orbit vars to buttons. 
 * kepler or orbit may be NULL, related vars will not be updated.
 *************************************/
void Set_Keplervals(GtkWidget *wnd,KEPLER *kepler,ORBIT *orbit)
{
  GtkWidget *kepler_wnd=Find_Window(wnd,Lab_Keplerwnd);
  GtkWidget *orbit_wnd=Find_Window(wnd,Lab_Orbitwnd);

/* Update Offsets and time choice */
  if ((orbit) && (orbit_wnd))
  {
    Set_Adjust(orbit_wnd,LAB_OH,"%d",orbit->offset_gmt);
    Set_Adjust(orbit_wnd,LAB_OS,"%d",orbit->offset_sec);
    Set_Adjust(orbit_wnd,LAB_OL,"%f",orbit->offset_lon);

    {
      static gboolean flag;          /* To prevent endless nested calls */
      if (!flag)
      {
        flag=TRUE;
        if (orbit->use_sattime)
          Set_Button(orbit_wnd,LAB_SATTM,TRUE);
        else
          Set_Button(orbit_wnd,LAB_COMPTM,TRUE);
        flag=FALSE;
      }
    }
  }

/* Update Kepler data */
  if ((kepler) && (kepler_wnd))
  {
    Set_Adjust(kepler_wnd,LAB_EY,"%d",kepler->epoch_year);
    Set_Adjust(kepler_wnd,LAB_ED,"%f",kepler->epoch_day);
    Set_Adjust(kepler_wnd,LAB_DR,"%f",kepler->decay_rate);
    Set_Adjust(kepler_wnd,LAB_IN,"%f",R2D(kepler->inclination));
    Set_Adjust(kepler_wnd,LAB_RA,"%f",R2D(kepler->raan));
    Set_Adjust(kepler_wnd,LAB_EC,"%f",kepler->eccentricity);
    Set_Adjust(kepler_wnd,LAB_PE,"%f",R2D(kepler->perigee));
    Set_Adjust(kepler_wnd,LAB_AN,"%f",R2D(kepler->anomaly));
    Set_Adjust(kepler_wnd,LAB_MO,"%f",kepler->motion);

    if (orbit)
      Set_Entry(kepler_wnd,LAB_HE,"%f",orbit->height/1000);
  }


/* Update start time */
  if ((orbit) && (kepler_wnd))
  {
    Set_Adjust(kepler_wnd,LAB_RFY,"%d",orbit->start_tm.tm_year);
    Set_Adjust(kepler_wnd,LAB_RFN,"%d",orbit->start_tm.tm_mon+1);
    Set_Adjust(kepler_wnd,LAB_RFD,"%d",orbit->start_tm.tm_mday);
    Set_Adjust(kepler_wnd,LAB_RFH,"%d",orbit->start_tm.tm_hour);
    Set_Adjust(kepler_wnd,LAB_RFM,"%d",orbit->start_tm.tm_min);
    Set_Adjust(kepler_wnd,LAB_RFS,"%d",orbit->start_tm.tm_sec);
  }
}

/*************************************
 * Manage buttons in Kepler edit window.  
 * Connected to Create_kepleredt_wnd
 *************************************/
void kepleredt_cmds(GtkWidget *widget, gpointer data)
{
  GtkWidget *window=NULL;
  char *name=(char *)data;

/* Determine kepler-window */
  if (GTK_IS_WIDGET(widget))
    window=gtk_widget_get_toplevel(widget);

/* Exit kepler-window */
  if (!strcmp(name,LAB_Exit))
  {
    Close_Window(window);
  }
}


/*************************************
 * Manage buttons in orbit edit window 
 * Connected to Create_kepleredt_wnd and Create_orbitedt_wnd
 *************************************/
void orbitedt_cmds(GtkWidget *widget, gpointer data)
{
  GtkWidget *window=NULL;
  GtkWidget *kepler_wnd=NULL, *orbit_wnd=NULL;
  VIEWINFO *vwi=Get_Viewinfo_db(widget);
  PICINFO *pci=Get_Picinfo_db(widget);
  char *name=(char *)data;

/* Vars used to restore original values */
  static KEPLER kepler_pre;
  static struct tm start_tm_pre;
  static struct tm start_tm_sat_pre;
  static struct tm start_tm_comp_pre;
  static int offset_gmt_pre;
  static int offset_sec_pre;
  static int offset_lon_pre;
  static gboolean use_sattime_pre;

  static int time_y;                  /* time extracted from line 'y' */
  static HRPT_FRAMEHDR frmhdr;        /* frameheader vars */

/* help booleans for undo/cancel */
  static gboolean start=TRUE;
  static gboolean updated;


/* Determine kepler-window */
  if (GTK_IS_WIDGET(widget))
    window=gtk_widget_get_toplevel(widget);
  if (GTK_IS_ADJUSTMENT(widget))
    window=Find_Parent_Window(widget);

/* Save old kepler values at start */
  if (start)
  {
    time_y=0;
    kepler_pre=pci->kepler;
    start_tm_pre=pci->orbit.start_tm;
    start_tm_sat_pre=pci->orbit.start_tm_sat;
    start_tm_comp_pre=pci->orbit.start_tm_comp;

    offset_gmt_pre=pci->orbit.offset_gmt;
    offset_sec_pre=pci->orbit.offset_sec;
    offset_lon_pre=pci->orbit.offset_lon;
    use_sattime_pre=pci->orbit.use_sattime;
    updated=FALSE;
    start=FALSE;
  }

/* ===== Manage Kepler window buttons ===== */
/* Take over entered kepler values */
  if (!strcmp(name,LAB_EY))
  {
    pci->kepler.epoch_year=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_ED))
  {
    pci->kepler.epoch_day=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_DR))
  {
    pci->kepler.decay_rate=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_IN))
  {
    pci->kepler.inclination=D2R(GTK_ADJUSTMENT(widget)->value);
  }
  if (!strcmp(name,LAB_RA))
  {
    pci->kepler.raan=D2R(GTK_ADJUSTMENT(widget)->value);
  }
  if (!strcmp(name,LAB_EC))
  {
    pci->kepler.eccentricity=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_PE))
  {
    pci->kepler.perigee=D2R(GTK_ADJUSTMENT(widget)->value);
  }
  if (!strcmp(name,LAB_AN))
  {
    pci->kepler.anomaly=D2R(GTK_ADJUSTMENT(widget)->value);
  }
  if (!strcmp(name,LAB_MO))
  {
    pci->kepler.motion=GTK_ADJUSTMENT(widget)->value;
  }

/* ===== Manage start time buttons ===== */
  if (!strcmp(name,LAB_RFY))
  {
    pci->orbit.start_tm.tm_year=GTK_ADJUSTMENT(widget)->value;
  }

  if (!strcmp(name,LAB_RFN))
  {
    pci->orbit.start_tm.tm_mon=GTK_ADJUSTMENT(widget)->value-1;
  }
  if (!strcmp(name,LAB_RFD))
  {
    pci->orbit.start_tm.tm_mday=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_RFH))
  {
    pci->orbit.start_tm.tm_hour=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_RFM))
  {
    pci->orbit.start_tm.tm_min=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_RFS))
  {
    pci->orbit.start_tm.tm_sec=GTK_ADJUSTMENT(widget)->value;
  }

/* ===== Manage Orbit offset buttons ===== */
  if (!strcmp(name,LAB_OH))
  {
    pci->orbit.offset_gmt=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_OS))
  {
    pci->orbit.offset_sec=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_OL))
  {
    pci->orbit.offset_lon=GTK_ADJUSTMENT(widget)->value;
  }


/* Recalculate constants */
  calc_orbitconst(&pci->kepler,&pci->orbit);

/* Update satellite height */
  if (kepler_wnd) 
    Set_Entry(kepler_wnd,LAB_HE,"%f",pci->orbit.height/1000.);

/* ** Manage Orbit window buttons ** */
/* Select time: computer or sat. 
   Note: Connected to 2 radio buttons; switching will 
   activate both buttons (one going off, the other going on) 
*/
/* 1. Computer-time. */
  if (!strcmp(name,LAB_COMPTM))
  {
    if (!pci->orbit.use_sattime)                    /* changing to sat-time */
    {
      pci->orbit.start_tm_comp=pci->orbit.start_tm; /* so save comp-time */
    }
    else                                            /* changing to comp-time */
    {
      pci->orbit.start_tm=pci->orbit.start_tm_comp; /* so get comp-time */
      pci->orbit.start_ms=0;
      pci->orbit.use_sattime=FALSE;
      Set_Keplervals(orbit_wnd,NULL,&pci->orbit);
    }

    Sense_Button(orbit_wnd,LAB_OH,TRUE);
    Sense_Button(orbit_wnd,LAB_OS,TRUE);
  }

/* 2. Satellite-time. */
  if (!strcmp(name,LAB_SATTM))
  {
    if (pci->orbit.use_sattime)                     /* changing to comp-time */
    {
      pci->orbit.start_tm_sat=pci->orbit.start_tm;  /* so save sat-time */
      pci->orbit.start_ms_sat=pci->orbit.start_ms;
    }
    else                                            /* changing to sat-time */
    {
      pci->orbit.start_tm=pci->orbit.start_tm_sat;  /* so get sat-time */
      pci->orbit.start_ms=pci->orbit.start_ms_sat;

      pci->orbit.use_sattime=TRUE;
      Set_Keplervals(orbit_wnd,NULL,&pci->orbit);
    }

    Sense_Button(orbit_wnd,LAB_OH,FALSE);
    Sense_Button(orbit_wnd,LAB_OS,FALSE);
  }

/* ** Extract satellite time from HRPT header. ** */

/* Start search */
  if (!strcmp(name,LAB_SRCHSATTM))
  {
    char tmp[20];
    time_y=search_first_reliable_frame(time_y,pci,vwi,&frmhdr);

/* Show result search */
    if (!time_y)
    {
      sprintf(tmp,"Failed!");
      Sense_Button(orbit_wnd,LAB_USESATTM,FALSE);
    }
    else
    {
      sprintf(tmp,"at line %d",time_y);
      Sense_Button(window,LAB_USESATTM,TRUE);
      interpret_framehdr(&frmhdr,pci->orbit.start_tm_comp);
      Set_Entry(window,LAB_SATNMF,"%s",frmhdr.sat_name);
    }
    Set_Entry(window,LAB_SRCHRSLT,"%s",tmp);
  }

/* Use found time */
  if (!strcmp(name,LAB_USESATTM))
  {
/* Parse frameheader; use year of computer */
    if ((interpret_framehdr(&frmhdr,pci->orbit.start_tm_comp)) == 'd')
      Create_Message("Warning","Satellite data doesn't contain day count.\n Day taken from computer-time.");

/* Load into start time */
    pci->orbit.start_tm_sat=frmhdr.sat_time; 

/* Substract time elapsed from start picture */
    if (pci->lines_per_sec)
      pci->orbit.start_tm_sat.tm_sec-=(time_y/pci->lines_per_sec);

/* Correct tm-struct */
    mktime_ntz(&pci->orbit.start_tm_sat);
    pci->orbit.start_ms_sat=0;

/* Redefine 'Start time' and set time select to 'Satellite time' */
    Set_Button(window,LAB_SATTM,TRUE);
    pci->orbit.start_tm=pci->orbit.start_tm_sat;

    Set_Keplervals(window,&pci->kepler,&pci->orbit);
  }

/* Show selected time */
  if (orbit_wnd) 
  {
    char tmp[100];
    strcpy(tmp,asctime(&pci->orbit.start_tm));
    Set_Entry(orbit_wnd,LAB_TM,"%s",tmp+4);
  }

/* Redraw picture */
  if (!strcmp(name,LAB_Redraw))
  {
    Update_Main(window);
    updated=TRUE;
  }

/* Undo changes */
  if (!strcmp(name,LAB_Undo))
  {
    pci->kepler=kepler_pre;
    pci->orbit.start_tm=start_tm_pre;
    pci->orbit.start_tm_sat=start_tm_sat_pre;
    pci->orbit.start_tm_comp=start_tm_comp_pre;

    pci->orbit.offset_gmt=offset_gmt_pre;
    pci->orbit.offset_sec=offset_sec_pre;
    pci->orbit.offset_lon=offset_lon_pre;
    pci->orbit.use_sattime=use_sattime_pre;
    Set_Keplervals(window,&pci->kepler,&pci->orbit);
  }

/* Reload hrpt header. */
  if (!strcmp(name,LAB_Reload))
  {
    Reload_Kepler(pci);
    Set_Keplervals(window,&pci->kepler,&pci->orbit);
  }


/* Save changes */
  if (!strcmp(name,LAB_Save))
  {
    Rewrite_hdr(pci);
  }

/* Exit kepler-window */
  if (!strcmp(name,LAB_Exit))
  {
    start=TRUE;
    Close_Window(window);
  }

/* Cancel: undo kepler edits and exit. */
  if (!strcmp(name,LAB_Cancel))
  {
    pci->kepler=kepler_pre;
    pci->orbit.start_tm=start_tm_pre;
    pci->orbit.start_tm_sat=start_tm_sat_pre;
    pci->orbit.start_tm_comp=start_tm_comp_pre;

    pci->orbit.offset_gmt=offset_gmt_pre;
    pci->orbit.offset_sec=offset_sec_pre;
    pci->orbit.offset_lon=offset_lon_pre;
    pci->orbit.use_sattime=use_sattime_pre;

    start=TRUE;
    Close_Window(window);
  }

}

/*************************************
 * Reload Kepler data from hrpt-file 
 *************************************/
void Reload_Kepler(PICINFO *pci)
{
  switch(pci->file_type)
  {
    case CHRPT1: case HRPT1:
      read_hrpt1_hdr(pci);
    break;
    default:
    break;
  }
}


/*************************************
 * Define orbit advanced edit window.  
 *************************************/
GtkWidget *Create_kepleredt_wnd(GtkWidget *main_window,
                                KEPLER *kepler,ORBIT *orbit)
{
  GtkWidget *wa,*wb,*wc,*wy,*wz;
  GtkWidget *w1;
  GtkWidget *kepler_wnd;

/* Create new window or pop-up and refresh it if it already exist. */
  if (!(kepler_wnd=Create_Window(main_window,0,0,Lab_Keplerwnd,NULL))) 
  {
    kepler_wnd=Find_Window(main_window,Lab_Keplerwnd);
    if (kepler_wnd) Set_Keplervals(kepler_wnd,kepler,orbit);
    return NULL;
  }

/* Go on with defining the Kepler edit window */
  gtk_signal_connect(GTK_OBJECT(kepler_wnd), "key_press_event",
		              GTK_SIGNAL_FUNC(orbitedt_cmds), NULL);

/* Add Kepler edit buttons */
  wa=Create_ButtonArray("Kepler",orbitedt_cmds,3,
              SPIN,LAB_EY,"%9d%9d%9d",kepler->epoch_year,1,300,
              SPIN,LAB_ED,"%9.4f%9.4f%9.4f",kepler->epoch_day,0.,366.,
              SPIN,LAB_DR,"%9.4f%9.4f%9.4f",kepler->decay_rate,0.,1.,
              SPIN,LAB_IN,"%9.4f%9.4f%9.4f",R2D(kepler->inclination),-360.,360.,
              SPIN,LAB_RA,"%9.4f%9.4f%9.4f",R2D(kepler->raan),-180.,180.,
              SPIN,LAB_EC,"%9.4f%9.4f%9.4f",kepler->eccentricity,0.,1.,
              SPIN,LAB_PE,"%9.4f%9.4f%9.4f",R2D(kepler->perigee),-360.,360.,
              SPIN,LAB_AN,"%9.4f%9.4f%9.4f",R2D(kepler->anomaly),-360.,360.,
              SPIN,LAB_MO,"%9.4f%9.4f%9.4f",kepler->motion,0.,30.,
              0
              );

  if (orbit)
    wb=Create_Entry(LAB_HE,NULL,"%f",orbit->height/1000.);

/* Add time edit buttons */
  if (orbit)
  {
    wc=Create_ButtonArray("Selected start time",orbitedt_cmds,3,
                    SPIN,LAB_RFY,"%d%d%d",orbit->start_tm.tm_year,0,200,
                    SPIN,LAB_RFN,"%d%d%d",orbit->start_tm.tm_mon+1,1,12,
                    SPIN,LAB_RFD,"%d%d%d",orbit->start_tm.tm_mday,1,31,
                    SPIN,LAB_RFH,"%d%d%d",orbit->start_tm.tm_hour,0,23,
                    SPIN,LAB_RFM,"%d%d%d",orbit->start_tm.tm_min,0,59,
                    SPIN,LAB_RFS,"%d%d%d",orbit->start_tm.tm_sec,0,59,
                    0
                   );
  }

/* Exit button */
  w1=Create_Button(LAB_Exit,kepleredt_cmds);
  wy=Pack("",'h',w1,5,NULL);

  if (orbit)
    wz=Pack(NULL,'v',wa,5,wb,5,wc,5,wy,5,NULL);
  else
    wz=Pack(NULL,'v',wa,5,wy,5,NULL);

  gtk_container_add(GTK_CONTAINER(kepler_wnd),wz);
  gtk_widget_show(kepler_wnd);
  place_window(kepler_wnd,prefs.sub_wnd_x,prefs.sub_wnd_y,prefs.sub_wnd_geom);

  return kepler_wnd;
}


/*************************************
 * Activate Kepler/start-time edit window.  
 *************************************/
void Menu_kepleredt(GtkWidget *widget)
{
  GtkWidget *main_window=Find_Parent_Window(widget);
  PICINFO *pci=Get_Picinfo_db(widget);
  Create_kepleredt_wnd(main_window,&pci->kepler,&pci->orbit);
}

/*************************************
 * Define orbit edit window.  
 *************************************/
GtkWidget *Create_orbitedt_wnd(GtkWidget *main_window,KEPLER *kepler,ORBIT *orbit)
{
  GtkWidget *w1,*w2,*w3,*w4,*w5;
  GtkWidget *wa,*wb,*wc,*wd,*we,*wu,*wv,*wy,*wz;
  GtkWidget *kepler_window;

/* Create new window or pop-up and refresh it if it already exist. */
  if (!(kepler_window=Create_Window(main_window,0,0,Lab_Orbitwnd,NULL))) 
  {
    kepler_window=Find_Window(main_window,Lab_Orbitwnd);
    Set_Keplervals(kepler_window,kepler,orbit);
    return NULL;
  }

/* Go on with defining the Kepler edit window */
  gtk_signal_connect(GTK_OBJECT(kepler_window), "key_press_event",
		                 GTK_SIGNAL_FUNC(orbitedt_cmds), NULL);


/* Add orbit edit buttons */
  if (orbit)
  {
    char tmp[100];

/* Time and longitude offsets */
    w1=Create_ButtonArray(NULL,orbitedt_cmds,4,
                    SPIN,LAB_OH,"%d%d%d",orbit->offset_gmt,-24,24,
                    SPIN,LAB_OS,"%d%d%d",orbit->offset_sec,-900,900,
                    SPIN,LAB_OL,"%2.1f%f%f",orbit->offset_lon,-180.,180.,
                    0
                   );

    w2=Create_Button(LAB_Advwnd,Menu_kepleredt);
    wa=Pack("Corrections",'h',w1,5,w2,1,NULL);

/* Time selection */
    strcpy(tmp,asctime(&orbit->start_tm)); tmp[strlen(tmp)-1]=0;
    w1=Create_Entry(LAB_TM,NULL,"%s",tmp+4);

    w2=Create_ButtonArray("Select",orbitedt_cmds,1,
                    RADIO,LAB_COMPTM,
                    RADIO,LAB_SATTM,
                    0
                   );

    wb=Pack("Start time",'v',w1,5,w2,5,NULL);

/* Extract time */
    w1=Create_Button(LAB_SRCHSATTM,orbitedt_cmds);
    w2=Create_Entry(LAB_SRCHRSLT,NULL,"%11s","");
    w3=Create_Entry(LAB_SATNMF,NULL,"%11s","");
    w4=Pack(NULL,'v',w1,1,w2,1,w3,1,NULL);

    w5=Create_Button(LAB_USESATTM,orbitedt_cmds);

    wc=Pack("Extract time",'v',w4,5,w5,1,NULL);

    wd=Pack(NULL,'h',wb,5,wc,5,NULL);
  }

/* Redraw, undo */
  w1=Create_Button(LAB_Redraw,orbitedt_cmds);
  w2=Create_Button(LAB_Undo,orbitedt_cmds);
  we=Pack(NULL,'h',w1,5,w2,20,NULL);

/* Add HRPT file related buttons: reload, save */
  w1=Create_Button(LAB_Reload,orbitedt_cmds);
  w2=Create_Button(LAB_Save,orbitedt_cmds);
  wu=Pack("File",'h',w1,5,w2,5,NULL);

/* Add exit/cancel buttons */
  w1=Create_Button(LAB_Exit,orbitedt_cmds);
  w2=Create_Button(LAB_Cancel,orbitedt_cmds);
  wv=Pack("This window",'h',w1,5,w2,5,NULL);

  wy=Pack(NULL,'h',wu,5,wv,20,NULL);

  if (orbit)
    wz=Pack(NULL,'v',wa,5,wd,5,we,5,wy,1,NULL);
  else
    wz=Pack(NULL,'v',wa,5,wu,5,wv,1,NULL);

  gtk_container_add(GTK_CONTAINER(kepler_window),wz);
  gtk_widget_show(kepler_window);
  place_window(kepler_window,prefs.sub_wnd_x,prefs.sub_wnd_y,prefs.sub_wnd_geom);


  Sense_Button(kepler_window,LAB_USESATTM,FALSE);
  if (orbit->use_sattime)
    Set_Button(wd,LAB_SATTM,TRUE);
  else
    Set_Button(wd,LAB_COMPTM,TRUE);

  return kepler_window;
}

/*************************************
 * Create Kepler window
 * Activated by menu: Edit->Kepler
 *************************************/
void Menu_orbitedt(GtkWidget *widget)
{
  GtkWidget *main_window=Find_Parent_Window(widget);
  PICINFO *pci=Get_Picinfo_db(widget);
  Create_orbitedt_wnd(main_window,&pci->kepler,&pci->orbit);
}


#define Lab_Hdrwnd "Header edit"
#define LAB_SRCHFRMINFO "search frame info"
#define LAB_SRCHRSLTH "Found"

#define LAB_USESATNM "Use it"
#define LAB_SATNMF0 "^Satellite name"
#define LAB_SATNMF1 "Edit name"

#define LAB_SCANDIR "^Scan direction"
#define LAB_N_S "North->south"
#define LAB_S_N "South->north"

void hdredt_cmds(GtkWidget *widget, gpointer data)
{
  GtkWidget *window=gtk_widget_get_toplevel(widget);
  GtkWidget *main_window=Find_Window(window,Lab_Main);
  GtkWidget *hdr_wnd=Find_Window(widget,Lab_Hdrwnd);
  VIEWINFO *vwi=Get_Viewinfo_db(widget);
  PICINFO *pci=Get_Picinfo_db(widget);
  char *name=(char *)data;
  static int time_y;                  /* time extracted from line 'y' */
  static HRPT_FRAMEHDR frmhdr;        /* frameheader vars */

  if (!pci) return; /* create_options_menu activates func before everything is settled! */

/* Start search */
  if (!strcmp(name,LAB_SRCHFRMINFO))
  {
    char tmp[20];

    time_y=search_first_reliable_frame(time_y,pci,vwi,&frmhdr);

/* Show result search */
    if (!time_y)
    {
      sprintf(tmp,"Failed!");
      Sense_Button(hdr_wnd,LAB_USESATNM,FALSE);
    }
    else
    {
      sprintf(tmp,"at line %d",time_y);
      Sense_Button(hdr_wnd,LAB_USESATNM,TRUE);
      interpret_framehdr(&frmhdr,pci->orbit.start_tm_comp);
      Set_Entry(window,LAB_SATNMF0,"%s",frmhdr.sat_name);
    }
    Set_Entry(window,LAB_SRCHRSLTH,"%s",tmp);
  }
  if (!strcmp(name,LAB_USESATNM))
  {
    strcpy(pci->sat_name,frmhdr.sat_name);
    Set_Entry(window,LAB_SATNMF1,"%s",frmhdr.sat_name);
    Set_Entry(main_window,LAB_SATNM,"%s",frmhdr.sat_name);
  }

  if (!strcmp(name,LAB_SATNMF1))
  {
    strcpy(pci->sat_name,
      Get_Entry(gtk_widget_get_toplevel(widget),LAB_SATNMF1));
    Set_Entry(main_window,LAB_SATNM,"%s",pci->sat_name);
  }

  if (!strcmp(name,LAB_N_S))
  {
    pci->scan_ver=N_S;
    pci->scan_hor=W_E;
  }
  if (!strcmp(name,LAB_S_N))
  {
    pci->scan_ver=S_N;
    pci->scan_hor=E_W;
  }

  if (!strcmp(name,LAB_Save))
  {
    Rewrite_hdr(pci);
  }
  if (!strcmp(name,LAB_Exit))
  {
    Close_Window(window);
  }

}

/*************************************
 * Define frameheader edit window.  
 *************************************/
GtkWidget *Create_hdredt_wnd(GtkWidget *main_window,PICINFO *pci)
{
  GtkWidget *w1,*w2;
  GtkWidget *wa,*wb,*wc,*wu,*wz;
  GtkWidget *hdr_window;

/* Create new window or pop-up and refresh it if it already exist. */
  if (!(hdr_window=Create_Window(main_window,0,0,Lab_Hdrwnd,NULL))) 
  {
    hdr_window=Find_Window(main_window,Lab_Hdrwnd);
    return NULL;
  }

  gtk_signal_connect(GTK_OBJECT(hdr_window), "key_press_event",
		                 GTK_SIGNAL_FUNC(hdredt_cmds), NULL);



/* Extract frame info */
  w1=Create_Button(LAB_SRCHFRMINFO,hdredt_cmds);
  w2=Create_Entry(LAB_SRCHRSLTH,NULL,"%-8s","");
  wa=Pack("Extract frame header",'h',w1,1,w2,1,NULL);

  w1=Create_Entry(LAB_SATNMF0,NULL,"%8s","");
  w2=Create_Button(LAB_USESATNM,hdredt_cmds);
  w2=SPack(NULL,"v",w2,"E1",NULL);
  wb=Pack("Translated from space craft code",'h',w1,10,w2,1,NULL);

  wc=Create_Entry(LAB_SATNMF1,hdredt_cmds,"%-12s",pci->sat_name);

  wa=Pack("Satellite name",'v',wa,10,wb,10,wc,10,NULL);

  wb=Create_Optionmenu(LAB_SCANDIR,hdredt_cmds,(pci->scan_ver==S_N? 1 : 0),
                       LAB_N_S,
                       LAB_S_N,
                       0
                      );
  wb=SPack(NULL,"h",wb,"5",NULL);

/* Add save/exit buttons */
  w1=Create_Button(LAB_Save,hdredt_cmds);
  w2=Create_Button(LAB_Exit,hdredt_cmds);
  wu=Pack("",'h',w1,5,w2,5,NULL);

  wz=SPack(NULL,"v",wa,"5",wb,"5",wu,"5",NULL);

  gtk_container_add(GTK_CONTAINER(hdr_window),wz);
  gtk_widget_show(hdr_window);
  place_window(hdr_window,prefs.sub_wnd_x,prefs.sub_wnd_y,prefs.sub_wnd_geom);

  return hdr_window;
}

void Menu_Hdredt(GtkWidget *widget)
{
  GtkWidget *main_window=Find_Parent_Window(widget);
  PICINFO *pci=Get_Picinfo_db(widget);
  Create_hdredt_wnd(main_window,pci);
}
