/**************************************************
 * RCSId: $Id$
 *
 * File-save related functions 
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log$
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 R. Alblas. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include "wsat.h"
#include "menu_wsat.h"

// #undef _AHRPT1_

extern PREFS prefs;

#if _SUN_
#  include "sun.h"
#endif
#if _TIFF_
#  include "tiff.h"
#endif
#if _PPM_
#  include "ppm.h"
#endif
#if _GIF_
#  include "gif.h"
#endif
#if _HRPT1_
#  include "hrpt1.h"
#endif
#if _HRI1_
#  include "hri1.h"
#endif

#define FILENAME_PREFIX "dump"

#define LAB_PPM_MCH "Multi-channel"

#define LAB_CH " "             /* prefix number */
#define LAB_CHSUF   " "        /* suffix number */
#define LAB_CH1     " 1 "
#define LAB_CH2     " 2 "
#define LAB_CH3     " 3 "
#define LAB_CH4     " 4 "
#define LAB_CH5     " 5 "
#define LAB_CH6     " 6 "
#define LAB_CH7     " 7 "
#define LAB_CH8     " 8 "
#define LAB_CH9     " 9 "
#define LAB_CH10    " 10 "
#define LAB_CHALL   "All"
#define LAB_FILENAME "^Save to"
#define LAB_BITSHIFT "^Bitshift"

#define LAB_Genfn "New name"
#define LAB_Save "Save"
#define LAB_Exit "Exit"
#define LAB_HRPT "(C)HRPT"
#define LAB_SUN  "Sun"
#define LAB_TIFF "Tiff"
#define LAB_PPM  "PPM"
#define LAB_PGM  "PGM"
#define LAB_PGM1 "PGM1"
#define LAB_GIF  "Gif"
#define LAB_AHRPT  "AHRPT"

#define FN_EXT(t) (t==SUN?      "sun" : \
                   t==TIFF?     "tif" : \
                   t==PPM?      "ppm" : \
                   t==PGM?      "pgm" : \
                   t==PGM1?     "pgm" : \
                   t==GIF?      "gif" : \
                   t==HRPT1?    "10b" : \
                   t==CHRPT1?   "10b" : \
                   t==SEAWIF1?  "10b" : \
                   t==HRI1?     "hri" : \
                                "dat")

/*************************************
 * Write file header.  
 *************************************/
int write_header(PICINFO *pci)
{
  int err=0;
  switch(pci->file_type)
  {
#if _SUN_
    case SUN:
      err=write_sun_header(pci);
    break;
#endif
#if _TIFF_
    case TIFF:
      err=write_tiff_header(pci);
    break;
#endif
#if _PPM_
    case PPM: case PGM: case PGM1:
      err=write_ppm_header(pci);
    break;
#endif
#if _GIF_
    case GIF:
      err=write_gif_header(pci);
    break;
#endif
#if _HRPT1_
    case HRPT1: case CHRPT1: case SEAWIF1:
      err=write_hrpt1_hdr(pci);
    break;
#endif
#if _AHRPT1_
    case AHRPT1:
      err=write_ahrpt1_hdr(pci);
    break;
#endif
    default: break;
  }
  return err;
}

/*************************************
 * Finish writing if needed.
 *************************************/
void write_finish(PICINFO *pci)
{
#if _TIFF_
  if (pci->file_type==TIFF)
    write_tiff_trailer(pci);
#endif
#if _GIF_
  if (pci->file_type==GIF)
    write_gif_end(&pci->gif_pci, pci->fp);
#endif
}

/*************************************
 * Write a line.
 *************************************/
int write_line(guint16 *l,              /* B/W or red */
               guint16 *l_g,            /* green */
               guint16 *l_b,            /* blue */
               PICINFO *pci,
               int     dec_status)      /* sync status decoder cur. line */
{
  switch(pci->file_type)
  {
#if _SUN_
    case SUN:
      pci->fri.bytes_per_line=pci->width;
      write_sun_line(pci,l);
    break;
#endif
#if _TIFF_
    case TIFF:
      pci->fri.bytes_per_line=pci->width;
      write_tiff_line(pci,l,l_g,l_b);
    break;
#endif
#if _PPM_
    case PPM: case PGM: case PGM1:
      pci->fri.bytes_per_line=pci->width;
      write_ppm_line(pci,l,l_g,l_b);
    break;
#endif
#if _GIF_
    case GIF:
      write_gif_line(pci,l);
    break;
#endif
#if _HRPT1_
    case HRPT1: case CHRPT1: case SEAWIF1:
      write_hrpt1_line(pci,l,dec_status);
    break;
#endif
#if _AHRPT1_
    case AHRPT1:
      write_ahrpt1_line(pci,l);
    break;
#endif
    default:
    break;
  }
  return 0;
}

/*************************************
 * Determine channel(s) to write.
 *************************************/
int load_chan(PICINFO *pci_r,VIEWINFO *vwi_w,int *chan_arr)
{
  int nr_rch=0;
  int nr_wch=0;
  int ch;
  for (ch=1; ch<NR_CHANNELS_MAX; ch++)
  {
    if (pci_r->channels_recorded[ch])
    {
      if (vwi_w->channel_selected[ch])
      {
        chan_arr[nr_wch]=nr_rch;
        nr_wch++;
      }
      nr_rch++;
    }
  }
  return nr_wch;
}

/*************************************
 * Save.  
 * 
 *************************************/
void Save_it(GtkWidget *window,
             PICINFO    *pci_r,
             PICINFO    *pci_w,
             char       *file_name,
             VIEWINFO   *vwi_r)     /* only needed for HRI */
{
  GtkWidget *progress_bar;
  guint *xmap=NULL;
  guint *ymap=NULL;
  VIEWINFO vwi_w=*vwi_r;
  guint16 **lines;
  guint16 *line_from;
  guint16 *line_to,*line_to_g,*line_to_b;
  gboolean *line_grid=NULL;
  int x,y,ch;
  int nr_wch;
  int nr_ilines;
  int dec_status;
  int chan_arr[NR_CHANNELS_MAX];
  int picstart;
  int bitshift=0;
  guint16 *pix_from;
  gboolean cancelled=FALSE;
  gboolean grid;

/* Open file for write */
  if (!(pci_w->fp=fopen(file_name,"wb")))
  {
    Create_Message("Error","Can't open file %s.",pci_w->file_name);
    return;
  }

/* Determine where first pixel is in line */
  picstart=0;

  if (IS_HRPT_FAM(pci_w->file_type))        /* To HRPT */
  {
    vwi_w.linearize=FALSE;
    vwi_w.scan_hor=W_E;
    vwi_w.scan_ver=N_S;
    if (pci_r->channels_recorded[0])
      picstart=pci_r->fri.header_len;

    if (!IS_HRPT_FAM(pci_r->file_type))     /* From non-HRPT */
      bitshift=2;
  }
  else                                      /* To non-HRPT */
  {
    if ((IS_HRPT_FAM(pci_r->file_type)) ||  /* From HRPT */
        (pci_r->file_type==PGM))            /* From 16-bits PGM */
    {
      if (pci_w->file_type==PGM)            /* HRPT to 2-bytes PGM */
        bitshift=0;
      else                                  /* HRPT to 1-byte pp format */
      {
        if (vwi_r->lum_max<256)
          bitshift=0;
        else if (vwi_r->lum_max<512)
          bitshift=-1;
        else
          bitshift=-2;
      }
    }
  }
  Set_Entry(window,LAB_BITSHIFT,"%d",bitshift);

/* Determine channels to write */
  for (ch=1; ch<NR_CHANNELS_MAX; ch++)
    vwi_w.channel_selected[ch]=pci_w->channels_recorded[ch];
  nr_wch=load_chan(pci_r,&vwi_w,chan_arr);

/* Determine sizes of zoomed picture to save */
  pci_w->width=pci_r->width/vwi_r->zval.zx;
  pci_w->height=pci_r->height/vwi_r->zval.zy;
  pci_w->clrmap.size=pci_r->clrmap.size;
  if (pci_w->file_type==PGM1) pci_w->clrmap.size=256; 
  if (pci_w->file_type==PGM1) pci_w->depth=8; 

/* Write the file header */
  write_header(pci_w);

/* Generate mapping arrays xmap and ymap */
  map_xy(pci_w->width,pci_w->height,FALSE,pci_r,&vwi_w,&xmap,&ymap);

/* Allocate memory for read and write lines */
  nr_ilines=vwi_r->max_vshift-vwi_r->min_vshift+1;
  if (vwi_r->despeckle) nr_ilines*=3;

  lines=(guint16 **)alloc_lines(pci_r->fri.words_per_line,nr_ilines);
  line_from=lines[0];

  line_to=calloc(pci_r->fri.words_per_line,sizeof(guint16));   /* red, B/W */
  line_to_g=calloc(pci_r->fri.words_per_line,sizeof(guint16)); /* green */
  line_to_b=calloc(pci_r->fri.words_per_line,sizeof(guint16)); /* blue */
  if (vwi_r->grid)
    line_grid=(gboolean *)calloc(pci_r->fri.words_per_line,sizeof(*line_grid));

/* Start progress bar */
  progress_bar=Create_Progress(window,"Saving...",TRUE);

/* If source is GIF then rewind it */
  if (pci_r->file_type==GIF) rewind_gif(pci_r);

/* Save the picture */
  for (y=0; y<pci_w->height; y++)
  {
    int x_w=0;
/* Read 1 or more lines needed to produce 1 output line */

    dec_status=read_lines(vwi_r,pci_r,ymap,y,nr_ilines,
                                            lines,NULL,NULL,line_grid);

/* Copy HRPT header to line_to if HRPT --> HRPT */

    if ((pci_r->channels_recorded[0]) && (pci_w->channels_recorded[0]))
    {
      for (x=0; x<picstart; x++)
      {
        *(line_to+x)=*(line_from+x);
        *(line_to_g+x)=*(line_from+x);
        *(line_to_b+x)=*(line_from+x);
      }
    }

    if (vwi_r->process_data)
    {
/* Translate input line to rgb */
      Translate_Line(vwi_r,pci_r,bitshift,NULL,FALSE,lines,nr_ilines,
                      line_to+picstart,line_to_g+picstart,line_to_b+picstart);
    }
    else
    {
/* Add all pixels of all channels to save to "line_to" */
      for (x=0; x<pci_w->width; x++)          /* for all pixels in a line */
      {
        for (ch=0; ch<nr_wch; ch++)           /* for all channels to save */
        {
/* Get pixel taking into account x-offset */
          pix_from=(line_from+*(xmap+x)+chan_arr[ch]-chan_arr[0]);

/* Get grid-info. */
          if (line_grid)
            grid=*(line_grid+*(xmap+x)+chan_arr[ch]-chan_arr[0]);

          {
            guint16 bw=*pix_from;
            guint16 bw1,bw2,bw3;

/* Translate pix between 10bits <--> 8bits words */
            if (bitshift<0)
              bw>>=(bitshift*-1);
            else
              bw<<=bitshift;

/* Replace pix by grid, if requested and gridflag is true. 
   (line_grid=true if vwi->grid=true) 
*/
            if ((line_grid) && (grid))
            {
              if ((pci_w->file_type==TIFF) || (pci_w->file_type==PPM))
                 /* use colored grid. */
                add_gridpix(vwi_r,&bw1,&bw2,&bw3);
              else                        /* For bw format use white grid. */
                add_gridpix(NULL,&bw1,&bw2,&bw3);
            }
            else
            {
              bw1=bw2=bw3=bw;
            }

            *(line_to + x_w + picstart)   = bw1;
            *(line_to_g + x_w + picstart) = bw2;
            *(line_to_b + x_w + picstart) = bw3;
          }

/* Increment write-X. Different from x if multi-channels are saved. */
          x_w++;
        }
      }
    }

/* Copy HRPT trailer to line_to if HRPT --> HRPT */
    if ((pci_r->channels_recorded[0]) && (pci_w->channels_recorded[0]))
    {
      int picend;
      picend=pci_r->fri.words_per_frame-100;
      for (x=picend; x<(picend+100); x++)
      {
        *(line_to + x_w + picstart)  =*(line_from+x);
        *(line_to_g + x_w + picstart)=*(line_from+x);
        *(line_to_b + x_w + picstart)=*(line_from+x);
        x_w++;
      }
    }

/* Update progress bar and detect if cancelled */
    if (Update_Progress(progress_bar,y,pci_w->height))
    {
      cancelled=TRUE;
      break;
    }

/* Write line */
    if (vwi_r->process_data)
    {
      int ox;
      if (vwi_r->scan_hor==E_W)
        ox=pci_r->width-vwi_r->zval.ox - pci_w->width;
      else
        ox=vwi_r->zval.ox;
      ox=MAX(ox,0);
      write_line(line_to+ox,line_to_g+ox,line_to_b+ox,pci_w,dec_status);
    }
    else
      write_line(line_to,line_to_g,line_to_b,pci_w,dec_status);
  }

/* Close progress bar */
  Close_Window(gtk_widget_get_toplevel(progress_bar));

  write_finish(pci_w);

  free_lines(lines,nr_ilines);
  free(line_to);
  free(line_to_g);
  free(line_to_b);
  if (line_grid) free(line_grid);
  free(xmap);
  free(ymap);
  fclose(pci_w->fp);

  if (cancelled)
    Create_Message("Info","Cancelled; %d of %d lines saved in %s.",
               y,pci_w->height,pci_w->file_name);
  else
    Create_Message("Info","Saved in %s; bitshift=%d.",
                                           pci_w->file_name,bitshift);
}

/*************************************
 * Manage buttons.  
 *************************************/
void save_commands(GtkWidget *widget, gpointer data)
{
  GtkWidget *save_window=gtk_widget_get_toplevel(widget);
  char *name=(char *)data;
  char *fn,*p,ext[10];
  PICINFO *pci=Get_Picinfo_db(widget);
  VIEWINFO *vwi=Get_Viewinfo_db(widget);
  static int busy;
  if (busy) return;
  busy=1;

  fn=Get_Entry(gtk_widget_get_toplevel(widget),LAB_FILENAME);

/* Exit button */
  if (!strcmp(name,LAB_Exit))
  {
    Close_Window(save_window);
  }

/* Catch selected format. */
  if ((!strcmp(name,LAB_Genfn)) ||
      (!strcmp(name,LAB_HRPT))  ||
      (!strcmp(name,LAB_PPM))  ||
      (!strcmp(name,LAB_PGM))  ||
      (!strcmp(name,LAB_PGM1))  ||
      (!strcmp(name,LAB_GIF)))
  {
    FILE_TYPE file_type=0;
    if (Get_Button(save_window,LAB_SUN))  file_type=SUN;
    if (Get_Button(save_window,LAB_TIFF)) file_type=TIFF;
    if (Get_Button(save_window,LAB_PPM))  file_type=PPM;
    if (Get_Button(save_window,LAB_PGM))  file_type=PGM;
    if (Get_Button(save_window,LAB_PGM1)) file_type=PGM1;
    if (Get_Button(save_window,LAB_GIF))  file_type=GIF;
    if (Get_Button(save_window,LAB_HRPT)) file_type=HRPT1;

    if ((p=strchr(fn,'.'))) *p=0;
    sprintf(ext,".%s",FN_EXT(file_type));
    create_new_filename(fn,pci->work_location,FILENAME_PREFIX,ext);
    Set_Entry(gtk_widget_get_toplevel(widget),LAB_FILENAME,fn);

    if ((vwi->process_data) && (Get_Button(save_window,LAB_PPM)))
    {
      Set_Led(save_window,LAB_PPM_MCH,0xf00);
    }
    else
    {
      Set_Led(save_window,LAB_PPM_MCH,0x777);
    }
  }

  if (!strcmp(name,LAB_CHALL))
  {
    int i;
    for (i=1; i<NR_CHANNELS_MAX; i++)
    {
      GtkWidget *b;
      char tmp[20];
      sprintf(tmp,"%s%d%s",LAB_CH,i,LAB_CHSUF);           /* Construct button name */
      if ((b=Find_Widget(save_window,tmp)))
          Set_Button(save_window,tmp,TRUE);
    }
  }

/* Channel button. If no HRPT selected then just 1 channel may be selected. */
  if ((!strncmp(name,LAB_CH,strlen(LAB_CH))) || (!strcmp(name,LAB_CHALL)))
  {
    if ((Get_Button(save_window,LAB_SUN)) ||
        (Get_Button(save_window,LAB_GIF)) ||
        (Get_Button(save_window,LAB_PPM)) ||
        (Get_Button(save_window,LAB_PGM)) ||
        (Get_Button(save_window,LAB_PGM1)))
    {
      int i;
      for (i=1; i<NR_CHANNELS_MAX; i++)
      {
        GtkWidget *b;
        char tmp[20];
        sprintf(tmp,"%s%d%s",LAB_CH,i,LAB_CHSUF);           /* Construct button name */
        if ((b=Find_Widget(save_window,tmp)))
          if (b!=widget)
            Set_Button(save_window,tmp,FALSE);
      }
    }
  }


/* Save button */
  if ((fn) && (!strcmp(name,LAB_Save)))
  {
    FILE_TYPE file_type=0;
    PICINFO pci_w=*pci;
    char file_name[400];
    int i;
    int nr_chan=0;

/* Get choices */
    if (Get_Button(save_window,LAB_SUN))  file_type=SUN;
    if (Get_Button(save_window,LAB_TIFF)) file_type=TIFF;
    if (Get_Button(save_window,LAB_PPM))  file_type=PPM;
    if (Get_Button(save_window,LAB_PGM))  file_type=PGM;
    if (Get_Button(save_window,LAB_PGM1)) file_type=PGM1;
    if (Get_Button(save_window,LAB_GIF))  file_type=GIF;
    if (Get_Button(save_window,LAB_HRPT))
    {
      if (IS_HRPT_FAM(pci->file_type)) 
        file_type=pci->file_type;             /* Use same format as source */
      else
        file_type=HRPT1;
    }
    if (Get_Button(save_window,LAB_AHRPT)) file_type=AHRPT1;

    pci_w.file_type=file_type;
    for (i=1; i<NR_CHANNELS_MAX; i++)
    {
      GtkWidget *button;
      char tmp[20];
      pci_w.channels_recorded[i]=FALSE;
      sprintf(tmp,"%s%d%s",LAB_CH,i,LAB_CHSUF);               /* Construct button name */
      if ((button=Find_Widget(save_window,tmp)))       /* Get button widget */
      {
        pci_w.channels_recorded[i]=Get_Button(save_window,tmp);
        if (pci_w.channels_recorded[i]) nr_chan++;
      }
    }
    if ((!nr_chan) && (!vwi->process_data))
    {
      Create_Message("Error","No channels selected.");
      return;
    }

    if ((pci_w.file_type!=HRPT1) && (pci_w.file_type!=CHRPT1) && 
        (pci_w.file_type!=AHRPT1))
      pci_w.channels_recorded[0]=FALSE;

    pci_w.fri.bytes_per_line=pci->width;
    pci_w.fri.words_per_line=pci->width;
    strcpy(pci_w.file_name,fn);
    sprintf(file_name,"%s%s",pci_w.work_location,pci_w.file_name);
    if ((pci_w.fp=fopen(file_name,"rb")))
    {
      fclose(pci_w.fp);
      if ((Create_Choice("Warning",2,"Cancel","OK"," File %s exist. Overwrite?",
                                                   pci_w.file_name))==1)
        return;
    }

    Save_it(save_window,pci,&pci_w,file_name,vwi);
  }
  busy=0;
}

/*************************************
 * Set state of channel buttons 
 *************************************/
void Update_channel_save(GtkWidget *main_window,PICINFO *pci,VIEWINFO *vwi)
{
  GtkWidget *save_window=Find_Window(main_window,LAB_Save);
  if (vwi->process_data)
  {
    Update_channel_state(save_window,NULL,NULL,LAB_CH,LAB_CHSUF);
  }
  else
  {
    Update_channel_state(save_window,pci,vwi,LAB_CH,LAB_CHSUF);
  }

  if ((vwi->process_data) && (Get_Button(save_window,LAB_PPM)))
    Set_Led(save_window,LAB_PPM_MCH,0xf00);
  else
    Set_Led(save_window,LAB_PPM_MCH,0x777);
}

/*************************************
 * Save file 
 * Menu: File -> Save as
 *************************************/
void MenuSave(GtkWidget *widget)
{
  GtkWidget *main_window=Find_Parent_Window(widget);
  GtkWidget *save_window;
  GtkWidget *wa[4],*wb,*wc,*wd,*we,*wx,*wy,*wz;
  GtkWidget *w1,*w2,*w3;
  GtkWidget *wt[11];
  int i,n=0;
  char fn[100],ext[10];
  PICINFO *pci=Get_Picinfo_db(widget);
  VIEWINFO *vwi=Get_Viewinfo_db(widget);

  sprintf(ext,".%s",FN_EXT(pci->file_type));
  create_new_filename(fn,pci->work_location,FILENAME_PREFIX,ext);

  if (!(save_window=Create_Window(main_window,0,0,LAB_Save,NULL))) return;

  gtk_signal_connect(GTK_OBJECT(save_window), "key_press_event",
		      GTK_SIGNAL_FUNC(save_commands), NULL);

  wa[1]=Create_Led(LAB_PPM_MCH,0x7ff);

  wa[2]=Create_ButtonArray(NULL,save_commands,5,
                             TOGGLE,LAB_CH1,FALSE,
                             TOGGLE,LAB_CH2,FALSE,
                             TOGGLE,LAB_CH3,FALSE,
                             TOGGLE,LAB_CH4,FALSE,
                             TOGGLE,LAB_CH5,FALSE,
                             TOGGLE,LAB_CH6,FALSE,
                             TOGGLE,LAB_CH7,FALSE,
                             TOGGLE,LAB_CH8,FALSE,
                             TOGGLE,LAB_CH9,FALSE,
                             TOGGLE,LAB_CH10,FALSE,
                             0
                          );

  {
    GdkColor startbut_clr,startbutsclr;
    GtkWidget *wbut;
    startbut_clr.red=0xffff; startbut_clr.green=0x5555; startbut_clr.blue=0x5555;
    startbutsclr.red=0xffff; startbutsclr.green=0x8888; startbutsclr.blue=0x8888;
    for (i=1; i<=10; i++)
    {
      char tmp[10];
      sprintf(tmp,"%s%d%s",LAB_CH,i,LAB_CHSUF);
      wbut=Find_Widget(wa[2],tmp);
      Set_Color(wbut,GTK_RC_BG,NULL,&startbutsclr,&startbut_clr,&startbut_clr,NULL);
    }
  }
  wa[3]=Create_Button(LAB_CHALL,save_commands);
  wa[2]=Pack("Channel select",'v',wa[2],5,wa[3],5,NULL);

  wt[n++]=Create_Radio(LAB_HRPT,TRUE,save_commands);
#if _SUN_
  wt[n++]=Create_Radio(LAB_SUN,FALSE,save_commands);
#endif
#if _TIFF_
  wt[n++]=Create_Radio(LAB_TIFF,FALSE,save_commands);
#endif
#if _PPM_
  wt[n++]=Create_Radio(LAB_PPM,FALSE,save_commands);
  wt[n++]=Create_Radio(LAB_PGM,FALSE,save_commands);
  wt[n++]=Create_Radio(LAB_PGM1,FALSE,save_commands);
#endif
#if _GIF_
  wt[n++]=Create_Radio(LAB_GIF,FALSE,save_commands);
#endif
#if _AHRPT1_
  wt[n++]=Create_Radio(LAB_AHRPT,FALSE,save_commands);
#endif
  wb=Pack(NULL,'v',NULL);
  for (i=0; i<n; i++)
    wb=Pack(NULL,'v',wb,0,wt[i],0,NULL);
  wb=Pack("File type",'v',wb,0,NULL);

  wd=Create_Entry(LAB_FILENAME,save_commands,"%s",fn);
  we=Create_Entry(LAB_BITSHIFT,NULL,"%3d",0);

  w1=Create_Button(LAB_Genfn,save_commands);
  w2=Create_Button(LAB_Save,save_commands);
  w3=Create_Button(LAB_Exit,save_commands);
  wc=SPack("","h",w1,"5ef",w2,"5ef",w3,"5ef",NULL);

  wd=Pack(NULL,'h',wd,5,we,5,NULL);
  wx=Pack(NULL,'v',wa[1],1,wa[2],10,wd,5,NULL);
  wy=Pack(NULL,'h',wx,10,wb,10,NULL);
  wz=Pack(NULL,'v',wy,10,wc,10,NULL);

  gtk_container_add(GTK_CONTAINER(save_window),wz);
  gtk_widget_show(save_window);
  place_window(save_window,prefs.sub_wnd_x,prefs.sub_wnd_y,prefs.sub_wnd_geom);

  if (IS_HRPT_FAM(pci->file_type))
  {
    Set_Button(save_window,LAB_HRPT,TRUE);
  }
  else switch(pci->file_type)
  {
    case PPM : Set_Button(save_window,LAB_PPM,TRUE); break;
    case PGM : Set_Button(save_window,LAB_PGM,TRUE); break;
    case PGM1: Set_Button(save_window,LAB_PGM1,TRUE); break;
    case GIF : Set_Button(save_window,LAB_GIF,TRUE); break;
    case HRI1: Set_Button(save_window,LAB_GIF,TRUE); break;
    default  : Set_Button(save_window,LAB_GIF,TRUE); break;
  }

  Update_channel_save(main_window,pci,vwi);
}

/*************************************
 * Rewrite file header using info in pci 
 *************************************/
int Rewrite_hdr(PICINFO *pci)
{
  int rslt=0;
  switch(pci->file_type)
  {
/* Just these formats; NO hri! */
    case CHRPT1: case HRPT1: case SEAWIF1:
      if ((pci->fp=freopen(pci->file_name,"rb+",pci->fp)))
      {
        if (write_hrpt1_hdr(pci))
        {
          Create_Message("Error","Rewriting header in %s failed.",
                                                          pci->file_name);
          rslt=0;
        }
        else
        {
          Create_Message("Info","Rewriting header in %s succeeded.",
                                                          pci->file_name);
          rslt=1;
        }
        if (!(pci->fp=freopen(pci->file_name,"rb",pci->fp)))
        {
          Create_Message("Error","Can't reopen %s for reading",
                                                          pci->file_name);
          rslt=0;
        }
      }
      else
      {
        Create_Message("Error","Can't reopen %s for writing",pci->file_name);
        rslt=0;
      }
    break;
    default:
    break;
  }
  return rslt;
}
