/**************************************************
 * RCSId: $Id$
 *
 * Key actions.
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log$
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 R. Alblas. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/
#include "wsat.h"
#include "menu_wsat.h"
 gboolean sched_slide(gpointer data);

/*************************************
 * Handle keyboard actions
 * Connected to canvas 
 *************************************/
gboolean key_actions(GtkWidget *widget, GdkEventKey *event)
{
  GtkWidget *main_window=gtk_widget_get_toplevel(widget);
  PICINFO *pci=Get_Picinfo_db(widget);
  VIEWINFO *vwi=Get_Viewinfo_db(widget);
  GtkWidget *drawing_area=Find_Widget(main_window,"GTK_DRAWING_AREA");
  int wnd_width=drawing_area->allocation.width;  /* width window */
  int wnd_height=drawing_area->allocation.height; /* height window */

  gboolean new_zoom=FALSE;
  ZOOMVALS tzval;

  vwi->fixzooming=FALSE;

/* Prevent focus on gamma spin */
  gtk_widget_grab_focus(Find_Widget(widget,LAB_COORD));
/*
  if (!GTK_WIDGET_HAS_FOCUS(Find_Widget(widget,LAB_GAMMAVAL)))
  {
    gtk_widget_grab_focus(Find_Widget(widget,LAB_COORD));
  }
  else return;
*/

/* Detect control-key, force event to 'Z' = fixed-zooming. */
  if (event->keyval==gdk_keyval_from_name("Control_L"))
    event->keyval='Z';

  tzval=vwi->zval;

  switch ((guchar)event->keyval)
  {
    case GDK_Escape&0xff:
      if (vwi->slide_mode) vwi->slide_mode=FALSE;
    break;
    case 'r':
      new_zoom=TRUE;
    break;
    case 'Z':
      vwi->fixzooming=TRUE;
    break;
/* Zoom with mouse */
    case 'z':
      Create_Cursor(drawing_area,GDK_PLUS);
      vwi->zooming=TRUE;
    break;
/* Full Zoom */
    case 'f':
      vwi->zval=zoom_full();
      new_zoom=TRUE;
    break;
/* Zoom out with fixed value */
    case 'o':
      vwi->zval.ox=vwi->zval.ox-
                   nint(pci->width/vwi->zval.zx*(vwi->fix_zoomfactor-1)/2);
      vwi->zval.oy=vwi->zval.oy-
                   nint(pci->height/vwi->zval.zy*(vwi->fix_zoomfactor-1)/2);
      vwi->zval.zx=vwi->zval.zx/vwi->fix_zoomfactor;
      vwi->zval.zy=vwi->zval.zy/vwi->fix_zoomfactor;
      new_zoom=TRUE;
    break;
/* Zoom in with fixed value */
    case 'i':
      vwi->zval.zx=vwi->zval.zx*vwi->fix_zoomfactor;
      vwi->zval.zy=vwi->zval.zy*vwi->fix_zoomfactor;
      vwi->zval.ox=vwi->zval.ox+
                   nint(pci->width/vwi->zval.zx*(vwi->fix_zoomfactor-1)/2);
      vwi->zval.oy=vwi->zval.oy+
                   nint(pci->height/vwi->zval.zy*(vwi->fix_zoomfactor-1)/2);
      new_zoom=TRUE;
    break;
    case 'l':
      tzval=vwi->zval;
      vwi->zval=vwi->zval_pre;
      vwi->zval_pre=tzval;
      new_zoom=TRUE;
    break;
    case 'c':
      vwi->zval.dx=vwi->zval.dy; 
      vwi->zval.zx=vwi->zval.dx/((float)wnd_width/(float)pci->width);
      new_zoom=TRUE;
    break;
    case 'C':
      vwi->zval.dy=vwi->zval.dx;
      vwi->zval.zy=vwi->zval.dy/((float)wnd_height/(float)pci->height);
      new_zoom=TRUE;
    break;

/* shift left */
    case GDK_Left&0xff:
      if (vwi->scan_hor==E_W)
        vwi->zval.ox+=(pci->width/vwi->zval.zx/2);
      else
        vwi->zval.ox-=(pci->width/vwi->zval.zx/2);
      new_zoom=TRUE;
    break;
/* shift down */
    case GDK_Up&0xff:
      if (vwi->scan_ver==S_N)
        vwi->zval.oy+=(pci->height/vwi->zval.zy/2);
      else
        vwi->zval.oy-=(pci->height/vwi->zval.zy/2);
      new_zoom=TRUE;
    break;
/* shift right */
    case GDK_Right&0xff:
      if (vwi->scan_hor==E_W)
        vwi->zval.ox-=(pci->width/vwi->zval.zx/2);
      else
        vwi->zval.ox+=(pci->width/vwi->zval.zx/2);
      new_zoom=TRUE;
    break;
/* shift up */
    case GDK_Down&0xff:
      if (vwi->scan_ver==S_N)
        vwi->zval.oy-=(pci->height/vwi->zval.zy/2);
      else
        vwi->zval.oy+=(pci->height/vwi->zval.zy/2);
      new_zoom=TRUE;
    break;

/* Maximize/restore window size. */
    case 'm':
      window_toggle_size(main_window);
    break;

    case 's':
      vwi->slide_mode=TRUE;
      do_slide(widget,FALSE);
    break;
  }

  if (new_zoom)
  {
    vwi->zval_pre=tzval;
    correct_zoomval(drawing_area,vwi,pci);

/* Update windows */
    Update_Zoom(main_window);
    Update_Main(widget);
  }
  return TRUE;
}


/*************************************
 * Detect key-pressed.
 * If pressed: return 1.
 *************************************/
int KeyPressed()
{
  int val=0;
  if (gdk_events_pending())
  {
    GdkEvent *event=gdk_event_get();
    if (event)
    {
      if (event->type == GDK_KEY_PRESS)
        val=event->key.keyval;
      else
        gdk_event_put(event);   /* If key-press then put event back. */
    }
  }
  return val;
}
