/**************************************************
 * RCSId: $Id$
 *
 * Luminance related functions
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log$
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 R. Alblas. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/

#include <stdlib.h>

#include "wsat.h"

#define LAB_LMIN   "Lmin"
#define LAB_LMAX   "Lmax"
#define LAB_LZM    "Ampl"

#define LAB_Close  " Close "
#define LAB_Rest   "Restore"
#define LAB_Redraw "Redraw"
#define LAB_LMIMA  "Lum thr."

#define LAB_Stat   "Recalc"
#define Lab_Lumwnd "Luminance"

extern PREFS prefs;

static int lumzoom;

/*************************************
 * Determine lum min/max.
 * Search starting from lum=1 until # pixels is above threshold "LOW_THRESH".
 * ==> lum_min
 * Search starting from lum=max until # pixels is above threshold "LOW_THRESH".
 * ==> lum_max
 *************************************/
#define LOW_THRESH nr_pix_maxl/50  /* #pix threshold */
void calc_lum_thresholds(VIEWINFO *vwi,int nrclr)
{
  int nr_pix_maxl=0;
  int i;
/* Determine max. # pixels with certain lum. */
  for (i=1; i<nrclr; i++)
  {
    nr_pix_maxl=MAX(nr_pix_maxl,vwi->lumstat[i]);
  }

/* Set lower lum threshold. 
   Skip lum=0 because maybe this isn't the actual min. lum 
*/
  for (i=1; i<nrclr; i++)
  {
    if (vwi->lumstat[i] > LOW_THRESH)
    {
      vwi->lum_min=i;
      break;
    }
  }

/* Set upper lum threshold */
  for (i=nrclr-1; i>=1; i--)
  {
    if (vwi->lumstat[i] > LOW_THRESH)
    {
      vwi->lum_max=i;
      break;
    }
  }
}

/*************************************
 * Draw luminance statistics.
 *************************************/
void draw_stat(GtkWidget *window,PICINFO *pci,VIEWINFO *vwi)
{

  GtkWidget *drawing_area=Find_Widget(window,"GTK_DRAWING_AREA");
  GdkPixmap *pixmap=Get_Pixmap(window);
  GdkGC **gc=Get_Gc(window);

  int x,y,c,xp=0,yp=0;
  int w_xmax=drawing_area->allocation.width-1;
  int w_ymax=drawing_area->allocation.height-1;

/* Determine xmax (=color axis) and ymax */
  int xmax=pci->clrmap.size-1;

  int ymax=0;

  if (!gc) return;              /* If gc not defined (yet) abort. */

/* Don't take into account '0' (too much black in picture 
   doesn't give nice lumi graph) 
*/
  for (x=1; x<=xmax; x++)
  {
    ymax=MAX(ymax,vwi->lumstat[x]);
  }
  if (!ymax) return;

/* Now draw the graphic */
    lumzoom=MAX(lumzoom,1);
  for (c=1; c<=xmax; c++)
  {
    x=c*w_xmax/xmax;
    y=vwi->lumstat[c]*lumzoom;
    y=MIN(y,ymax);
    y=w_ymax-(y*w_ymax)/ymax;
    if (x!=xp)
    {
      gdk_draw_line(pixmap,gc[1],xp,yp,x,y);
      yp=y;
    }
    xp=x;
  }

/* Draw lum_min and lum_max boundary lines */
  x=vwi->lum_min*w_xmax/xmax;
  gdk_draw_line(pixmap,gc[2],x,0,x,w_ymax);

  x=vwi->lum_max*w_xmax/xmax;
  gdk_draw_line(pixmap,gc[2],x,0,x,w_ymax);
}

/*************************************
 * Redraw window 
 *************************************/
gint configure_event_lum(GtkWidget *widget,GdkEventConfigure *event)
{
  GtkWidget *window=gtk_widget_get_toplevel(widget);
  PICINFO *pci=Get_Picinfo_db(widget);
  VIEWINFO *vwi=Get_Viewinfo_db(widget);

  Renew_Pixmap(window,TRUE);
  draw_stat(window,pci,vwi);

  return TRUE;
}


/*************************************
 * Refresh the lum window
 *************************************/
void Update_Lum(GtkWidget *window,PICINFO *pci,VIEWINFO *vwi)
{
  GtkWidget *drawing_area;

  if (!window) return;
  window=Find_Parent_Window(window);
  window=Find_Window(window,Lab_Lumwnd);
  if (!window) return;

  drawing_area=Find_Widget(window,"GTK_DRAWING_AREA");

  configure_event_lum(drawing_area,NULL);
  expose_event(drawing_area,NULL);

/* to prevent strange behav. if no pic is read */
  vwi->lum_min=MAX(vwi->lum_min,0);
  vwi->lum_max=MAX(vwi->lum_max,0);

  if (vwi->lum_min>vwi->lum_max)
  {
    int lum;
    lum=vwi->lum_min;
    vwi->lum_min=vwi->lum_max;
    vwi->lum_max=lum;
  }

  Set_Adjust(window,LAB_LMIN,"%d",vwi->lum_min);
  Set_Adjust(window,LAB_LMAX,"%d",vwi->lum_max);
}

/*************************************
 * Copy entered values in spin buttons to viewinfo
 * Activated by buttons
 * Also called by mouse_lum_actions, with data=NULL, to set previous lum-vals.
 *************************************/
void lum_commands(GtkWidget *widget, gpointer data)
{
  GtkWidget *main_window=Find_Window(Find_Parent_Window(widget),Lab_Main);
  GtkWidget *lum_window=Find_Window(main_window,Lab_Lumwnd);
  PICINFO *pci=Get_Picinfo_db(widget);
  VIEWINFO *vwi=Get_Viewinfo_db(widget);

  static VIEWINFO vwi_pre;
  GdkEventKey *event=(GdkEventKey *)data;
  char *name=(char *)data;
  static gboolean start=TRUE;

/* Save old lum values at start */
  if (start)
  {
    vwi_pre=*vwi;
    start=FALSE;
  }
  if (!data) return;

/* Update window if 'enter' is hit */
  if ((guchar)event->keyval==(GDK_Return&0xff))
  {
    Update_Lum(lum_window,pci,vwi);
    Update_Main(widget);
  }

/* Redraw main-window */
  if (!strcmp(name,LAB_Redraw))
  {
    Update_Main(widget);
  }

/* Determine lum min/max */
  if (!strcmp(name,LAB_LMIMA))
  {
    calc_lum_thresholds(vwi,pci->clrmap.size);
    Update_Lum(lum_window,pci,vwi);
    Update_Main(widget);
  }


/* Restore: recover previous lum values */
  if (!strcmp(name,LAB_Rest))
  {
    vwi->lum_min=vwi_pre.lum_min;
    vwi->lum_max=vwi_pre.lum_max;
    Update_Lum(lum_window,pci,vwi);
    Update_Main(widget);
  }

/* Take over entered lum values */
  if (!strcmp(name,LAB_LMIN))
  {
    vwi->lum_min=GTK_ADJUSTMENT(widget)->value;
    Update_Lum(lum_window,pci,vwi);
  }
  if (!strcmp(name,LAB_LMAX))
  {
    vwi->lum_max=GTK_ADJUSTMENT(widget)->value;
    Update_Lum(lum_window,pci,vwi);
  }

  if (!strcmp(name,LAB_LZM))
  {
    lumzoom=GTK_ADJUSTMENT(widget)->value;
    Update_Lum(lum_window,pci,vwi);
  }

/* Close lum-window */
  if (!strcmp(name,LAB_Close))
  {
    start=TRUE;
    Close_Window(lum_window);
  }
}

/*************************************
 * Handle mouse actions in lum canvas
 *************************************/
void mouse_lum_actions(GtkWidget *widget, GdkEventMotion *event)
{
  GtkWidget *lum_window=gtk_widget_get_toplevel(widget);
  GtkWidget *drawing_area=Find_Widget(lum_window,"GTK_DRAWING_AREA");
  PICINFO *pci=Get_Picinfo_db(widget);
  VIEWINFO *vwi=Get_Viewinfo_db(widget);
  int x,y,lum;
  GdkModifierType state;
  int w_xmax,xmax;

/* Just to set previous values lum! */
  lum_commands(widget,NULL);

  w_xmax=drawing_area->allocation.width-1;

/* Determine xmax */
  xmax=pci->clrmap.size-1;


/* Get mouse position */
  if (event->is_hint)
  {
    gdk_window_get_pointer(event->window,&x,&y,&state);
  }
  else
  {
    x=event->x;
    y=event->y;
    state=event->state;
  }

  if (state & GDK_BUTTON1_MASK)
  {
/* Catch neares line */
    lum=x*xmax/w_xmax;
    lum=MAX(0,MIN(lum,xmax));
    if (ABS(lum-vwi->lum_max) < ABS(lum-vwi->lum_min))
      vwi->lum_max=lum;
    else
      vwi->lum_min=lum;

    Update_Lum(lum_window,pci,vwi);
  }
}

/*************************************
 * Calculate lum thresholds
 *************************************/
void Menu_Lumthr(GtkWidget *widget)
{
  VIEWINFO *vwi=Get_Viewinfo_db(widget);
  PICINFO *pci=Get_Picinfo_db(widget);
  calc_lum_thresholds(vwi,pci->clrmap.size);
/*  Update_Lum(widget,pci,vwi);  /* Update lum-window, if open */
  Update_Main(widget);         /* Update pic */
}

#define LUMWINDOW_W 220
#define LUMWINDOW_H 180

/*************************************
 * Lum window
 * Activated by menu: View->Lum
 *************************************/
void Menu_Lummap(GtkWidget *widget)
{
  VIEWINFO *vwi=Get_Viewinfo_db(widget);
  PICINFO *pci=Get_Picinfo_db(widget);
  GtkWidget *w1,*w2,*w3,*w4;
  GtkWidget *wa,*wb,*wz;
  GtkWidget *lum_window;
  GtkWidget *main_window;
  GtkWidget *canvas;

  GdkColor clr[3];
  GdkColormap clrmap;

  main_window=Find_Parent_Window(widget);

  clrmap.colors=clr;
  clrmap.size=3;
/* Window color (?) */
  clr[0].red=0;
  clr[0].green=0;
  clr[0].blue=0xffff;

/* Graph. color */
  clr[1].red=0xffff;
  clr[1].green=0;
  clr[1].blue=0;

/* boundary color */
  clr[2].red=0;
  clr[2].green=0xffff;
  clr[2].blue=0;

/* Create window */
/* width depends on # buttons. Hard-defined here because 
   auto-size. (width=-1 ==> no gtk_window_set_default_size) doesn't work
   in combination with place_window.(???)
*/
  if (!(lum_window=Create_Window(main_window,260,200,Lab_Lumwnd,NULL))) return;

  gtk_signal_connect(GTK_OBJECT(lum_window), "key_press_event",
		      GTK_SIGNAL_FUNC(lum_commands), NULL);

  canvas=Create_Canvas(lum_window,0,0,configure_event_lum,
                                  NULL,mouse_lum_actions,mouse_lum_actions);

/* Add buttons */
  wa=Create_ButtonArray("Luminance",lum_commands,3,
                    SPIN,LAB_LMIN,"%3d%d%d",vwi->lum_min,0,pci->clrmap.size-1,
                    SPIN,LAB_LMAX,"%3d%d%d",vwi->lum_max,0,pci->clrmap.size-1,
                    SPIN,LAB_LZM ,"%2d%d%d",1,1,512,
                    0);

  w1=Create_Button(LAB_Redraw,lum_commands);
  w2=Create_Button(LAB_LMIMA,lum_commands);

  w3=Create_Button(LAB_Rest,lum_commands);
  w4=Create_Button(LAB_Close,lum_commands);

/* Pack buttons and canvas into window */
  wb=SPack("","h",w1,"5ef",w2,"5ef",w3,"5ef",w4,"5ef",NULL);

  wz=Pack(NULL,'v',canvas,3,wa,5,wb,5,NULL);
  gtk_container_add(GTK_CONTAINER(lum_window),wz);

  Create_GCPalette(lum_window,&clrmap);

  gtk_widget_show(lum_window);
  gtk_widget_realize(lum_window);
  place_window(lum_window,prefs.sub_wnd_x,prefs.sub_wnd_y,prefs.sub_wnd_geom);
}

