/**************************************************
 * RCSId: $Id$
 *
 * Misc functions.
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log$
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 R. Alblas. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/

#include <string.h>
#include <stdlib.h>
#include "wsat.h"
#include "menu_wsat.h"

/*************************************
 * Print error and abort program 
 *************************************/
void fatal_error()
{
  fprintf(stderr,"Fatal error! Program aborted.\n");
  exit(-1);
}

/*************************************
 * Print error 
 *************************************/
void error(char *str)
{
  fprintf(stderr,"Error: %s\n",str);
}

/*************************************
 * Allocate and copy
 *************************************/
char *strcpya(char *s)
{
  char *r;
  if (!(r=malloc(strlen(s)+2))) return NULL;
  strcpy(r,s);
  return r;
}

/*************************************
 * translate days-in-year in tm struct into month and day
 *************************************/
void yday2mday_mon(struct tm *tmref)
{
  int i;
  int days[]={31,28,31,30,31,30,31,31,30,31,30,31};
  if (tmref->tm_year/4==tmref->tm_year/4.) days[1]=29;

  tmref->tm_mday=tmref->tm_yday+1;
  tmref->tm_mon=0; 
  for (i=0; i<12; i++)
  {
    if (tmref->tm_mday<=days[i]) break;
    tmref->tm_mon++;
    tmref->tm_mday-=days[i];
  }

}

gboolean check_range(int a,int min,int max)
{
  if (a>max) return TRUE;
  if (a<min) return TRUE;
  return FALSE;
}

/*************************************
 * Check tm struct for invalid values.
 * Invalid values will be set on min. valid value.
 * (mktime can take very long exec. time if tm-struct 
 * contains ridiculous out-of-range values!)
 *************************************/
gboolean check_tm(struct tm *tm)
{
  int days[]={31,28,31,30,31,30,31,31,30,31,30,31};
  gboolean ret=FALSE;

  if (check_range(tm->tm_sec,0,60))                { ret=TRUE; tm->tm_sec=0; }
  if (check_range(tm->tm_min,0,60))                { ret=TRUE; tm->tm_min=0; }
  if (check_range(tm->tm_hour,0,23))               { ret=TRUE; tm->tm_hour=0; }
  if (check_range(tm->tm_mon,0,11))                { ret=TRUE; tm->tm_mon=0; }
  if (check_range(tm->tm_mday,1,days[tm->tm_mon])) { ret=TRUE; tm->tm_mday=0; }
  if (check_range(tm->tm_year,0,200))              { ret=TRUE; tm->tm_year=0; }

  return ret;
}

/*************************************
 * Correct tm-struct and return #seconds since 1970.
 * ntz stands for "no time-zone";
 * similar to mktime, except that dayligt saving is ignored.
 *************************************/
gint32 mktime_ntz(struct tm *tm)
{
  gint32 secs;
  gint32 rdays;
  int i;
  int days[]={31,28,31,30,31,30,31,31,30,31,30,31};

  while (tm->tm_sec >= 60) { tm->tm_min++;  tm->tm_sec-=60; }
  while (tm->tm_sec <   0) { tm->tm_min--;  tm->tm_sec+=60; }
  while (tm->tm_min >= 60) { tm->tm_hour++; tm->tm_min-=60; }
  while (tm->tm_min <   0) { tm->tm_hour--; tm->tm_min+=60; }
  while (tm->tm_hour>= 24) { tm->tm_mday++; tm->tm_hour-=24; }
  while (tm->tm_hour<   0) { tm->tm_mday--; tm->tm_hour+=24; }
  while (tm->tm_mon >= 12) { tm->tm_year++; tm->tm_mon-=12; }
  if (tm->tm_year/4==tm->tm_year/4.) days[1]=29;

  while (tm->tm_mday>days[tm->tm_mon])
  {
    tm->tm_mday-=days[tm->tm_mon]; tm->tm_mon++;
    while (tm->tm_mon>= 12)  { tm->tm_year++; tm->tm_mon-=12; }
    if (tm->tm_year/4==tm->tm_year/4.) days[1]=29; else days[1]=28;
  }
  secs=tm->tm_sec+60*(tm->tm_min+60*(tm->tm_hour+24*(tm->tm_mday-1)));
  for (i=0; i<tm->tm_mon; i++) secs+=(days[i]*24*3600);
  rdays=(tm->tm_year-70)*365;
  rdays+=(tm->tm_year-70+1)/4;
  secs+=(rdays*24*3600);
  tm->tm_wday=rdays%7;
  tm->tm_yday=0;
  for (i=0; i<tm->tm_mon; i++) tm->tm_yday+=days[i];
  tm->tm_yday+=(tm->tm_mday-1);

  return secs;
}


/*************************************
 * Get number in name after prefix
 *************************************/
int get_isuf(char *name,char *pre)
{
  if (!strncmp(name,pre,strlen(pre)))
    return atoi(name+strlen(pre));
  return 0;
}

/*************************************
 * translate to nearest integer
 *************************************/
int nint(float i)
{
  if (i>=0) return (int)(i+0.5);
  return (int)(i-0.5);
}

/*************************************
 * Extract picinfo from gtk-window.
 * 'w' can be any widget
 *************************************/
PICINFO *Get_Picinfo_db(GtkWidget *w)
{
  GtkWidget *window;

/* Find the parent window of the widget */  
  window=Find_Window(Find_Parent_Window(w),Lab_Main);
  if (!window) return NULL;

/* Return pointer to 'picinfo' */
  return (PICINFO *)gtk_object_get_data(GTK_OBJECT(window),WSAT_PCI);
}

/*************************************
 * Extract viewinfo from gtk-window.
 * 'w' can be any widget
 *************************************/
VIEWINFO *Get_Viewinfo_db(GtkWidget *w)
{
  GtkWidget *window;

/* Find the parent window of the widget */  
  window=Find_Window(Find_Parent_Window(w),Lab_Main);
  if (!window) return NULL;

/* Return pointer to 'viewinfo' */
  return (VIEWINFO *)gtk_object_get_data(GTK_OBJECT(window),WSAT_VWI);
}

/*************************************
 * Extract recordinfo from gtk-window.
 * 'w' can be any widget
 *************************************/
RECORDINFO *Get_Recordinfo_db(GtkWidget *w)
{
  GtkWidget *window;

/* Find the main window of the widget */  
  window=Find_Window(Find_Parent_Window(w),Lab_Main);
  if (!window) return NULL;

/* Return pointer to 'recordinfo' */
  return (RECORDINFO *)gtk_object_get_data(GTK_OBJECT(window),WSAT_RDI);
}

/*************************************
 * 'Sleep' function with ms resolution.
 * Just 1 sleep_ms allowed at a time, 
 * otherwise gtk_main_quit / gtk_main balance could get lost(?)...
 *************************************/
static gboolean busy_sleep;
gboolean ret_false()
{
  gtk_main_quit();
  return FALSE;
}

void sleep_ms(int ms)
{
  if (busy_sleep) return;          /* Don't allow more than 1 sleep_ms */
  busy_sleep=TRUE;
  gtk_timeout_add(ms,(GtkFunction)ret_false,0); /* Start time-out function */
  gtk_main();                             /* wait */
  busy_sleep=FALSE;
}

/*************************************
 * Set menu buttons to current situation
 *************************************/
void init_menubuts(GtkWidget *wnd,PICINFO *pci,VIEWINFO *vwi)
{
  FILE_TYPE ft=pci->file_type;
  Set_Button(wnd,MENU_LIN,(IS_HRPT_FAM(ft)? vwi->linearize : FALSE));
  Sense_Button(wnd,MENU_LIN,      IS_HRPT_FAM(ft));
  Sense_Button(wnd,MENU_KEPLER,   IS_HRPT_FAM(ft));
  Sense_Button(wnd,MENU_LDKEPLER, IS_HRPT_FAM(ft));
  Sense_Button(wnd,MENU_TEMP,     IS_HRPT_FAM(ft));
  Sense_Button(wnd,MENU_CHMAP,    IS_HRPT_FAM(ft));
  Sense_Button(wnd,MENU_GRIDADD,  IS_PRIM_DATA(ft));
  Sense_Button(wnd,MENU_SKIP_FLTS,IS_PRIM_DATA(ft));
  Sense_Button(wnd,MENU_SKIP_PP  ,IS_HRPT_FAM(ft));   /* sense doesn't work */ 
  Sense_Button(wnd,MENU_SKIP_PRE  ,IS_HRPT_FAM(ft));  /* so do it for sub-items */
  Sense_Button(wnd,MENU_SKIP_POST ,IS_HRPT_FAM(ft));
  Sense_Button(wnd,MENU_MARK_FLTS,IS_PRIM_DATA(ft));
  Set_Button(wnd,MENU_SATDIR,(ft==GIF? FALSE : (vwi->scan_ver==S_N? TRUE : FALSE)));
  Sense_Button(wnd,MENU_SATDIR,(ft==GIF? FALSE : TRUE));
  if (!pci->fp) Sense_Button(wnd,MENU_CLOSE,FALSE);
}

/* Detect if file exist */
int exist_file(char *fn)
{
  FILE *fp;
  if (!(*fn)) return 0;
  if (!(fp=fopen(fn,"r"))) return 0;
  fclose(fp);
  return 1;
}

extern PREFS prefs;
#define WSAT_LOG "wsat.log"
/* Generate log messages */
void log_message(char *frmt,...)
{
  FILE *fp=stdout;
  time_t t=time(NULL);
  char ts[30],*p;
  static char *logfile=NULL;
  va_list arg;
  va_start(arg,frmt);
  strcpy(ts,asctime(gmtime(&t)));
  if ((p=strchr(ts,'\n'))) *p=0;
  if (prefs.log)
  {
    if (!logfile)
    {
      logfile=malloc(strlen(prefs.home_dir)+strlen(WSAT_LOG)+5);
      if (logfile) sprintf(logfile,"%s%c%s",prefs.home_dir,DIR_SEPARATOR,WSAT_LOG);
    }
    if (logfile) fp=fopen(logfile,"a");
  }
  if (!fp) fp=stdout;
  
  fprintf(fp,"%s  ",ts);
  vfprintf(fp,frmt,arg);
  fprintf(fp,"\n");
  if (logfile) fclose(fp);
  va_end(arg);
}
