/**************************************************
 * RCSId: $Id$
 *
 * Preferences
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log$
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 R. Alblas. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/

#include <stdlib.h>
#include <string.h>

#include "wsat.h"
#include "menu_wsat.h"

/* Define button labels */
#define Lab_Prefwnd "Preferences"
#define LAB_Exit   " Exit "
#define LAB_Use    " Use  "
#define LAB_Save   " Save "
#define LAB_LOPREF "Local preference file"
#define LAB_GLPREF "Used preference file"

#define LAB_WNDPOS "^Window positions"
#define LAB_WNDSIZE "Main window size"
#define LAB_WNDPOS_DFLT "Default"
#define LAB_WNDPOS_SMART "Smart"
#define LAB_WND_WIDTH "^Width"
#define LAB_WND_HEIGHT "^Height"

#define LAB_INTERF "Decoder interface"
#define LAB_PAR "Parallel"
#define LAB_ETH "Ethernet"
#define LAB_USB "USB"

#define LAB_USBPORT "USB Port"
#define LAB_USBCHOICE "Use ID or first found:"
#define LAB_TESTUSB "Search FTD devices"
#define LAB_USB_USEFIRST "First found"
#define LAB_USB_USEID "Use this: "

#define LAB_USBID "USB ID"

#define LAB_PARPORT "Parallel port"
#define LAB_P1 "Port 1"
#define LAB_P2 "Port 2"
#define LAB_PN "Port n"
#define LAB_PE "Address Port n"
#define LAB_FASTPPACC "Fast parport access"

#define LAB_SEL_HWTYPE "Select HW:"
#define LAB_SEL_DECGEN "decoder/generator"
#define LAB_SEL_DECTYPE "decoder type"

#define LAB_LINEARIZE "Linearize"

#define LAB_DESPEC_LIM "Despeckle limits"
#define LAB_SPECK_SUL "^Sur. U-limit"
#define LAB_SPECK_SPL "^Pix. L-limit"
#define LAB_FIXZOOM "^Fixed zoom fact"

#define LAB_HRPTRECTIMES "Record times (frames)"
#define LAB_HRPTFILENAMES "File names"
#define LAB_HRPT_START "^Start after."
#define LAB_HRPT_STOP "^Stop after."
#define LAB_HRPTFILE_BASE "^Basename."
#define LAB_HRPTFILE_EXT "^Extension."

#define LAB_LPS "^Lines/screen"
#define LAB_SCANDIR "^Scan direction"
#define LAB_DIR_NSN "South bound/North bound"

#define LAB_VCHR "!Show channel"
#define LAB_CHX_V   "Chan"
#define LAB_CH1_V   "Chan 1"
#define LAB_CH2_V   "Chan 2"
#define LAB_CH3_V   "Chan 3"
#define LAB_CH4_V   "Chan 4"
#define LAB_CH5_V   "Chan 5"
#define LAB_CH6_V   "Chan 6"
#define LAB_CH7_V   "Chan 7"
#define LAB_CH8_V   "Chan 8"
#define LAB_CH9_V   "Chan 9"
#define LAB_CH10_V  "Chan 10"

#define LAB_SAVREC   "Save record"
#define LAB_KEPREC   "Load kepler"


#define LAB_HRIRECTIMES "Record times (frames)"
#define LAB_HRIFILENAMES "File names"
#define LAB_HRI_START "^Start after"
#define LAB_HRI_STOP "^Stop after"
#define LAB_HRIFILE_BASE "^Basename"
#define LAB_HRIFILE_EXT "^Extension"

#define LAB_DIR_SAVE "Working directory"

#define LAB_DIR_KEP  "Dir.   "
#define LAB_FILE_KEP "Name"


#define LAB_FSIZE "Font size"
#define LAB_GRIDCLR "Grid colour"
#define LAB_GRID_RED "^Red"
#define LAB_GRID_GREEN "^Green"
#define LAB_GRID_BLUE "^Blue"

#define SEPARATE " \n	="        /* parse word separator */

/* Defaults */
#define PREF_FILE ".wsatrc"
#define PARPORT_NUMBER 2          /* Default par port! */
#define GRID_CLR 0xff0000         /* Default grid color: red */

#define DEF_HRPTBASE "hrpt"
#define DEF_HRPTEXT ".dat"

#define DEF_HRIBASE "hri"
#define DEF_HRIEXT ".dat"

#define DEF_RECDIR ""

extern PREFS prefs;


/*************************************
 * Translate parport number into address  
 *************************************/
int ppnr_to_ppaddr(int parport_nr)
{
  int parport_base=0;
  switch(parport_nr)
  {
    case 1 : parport_base=0x378;      break;
    case 2 : parport_base=0x278;      break;
    default: parport_base=parport_nr; break;
  }
  return parport_base;
}

/*************************************
 * Copy some preferences
 *************************************/
void Load_Prefs(RECORDINFO *rdi,VIEWINFO *vwi,PICINFO *pci)
{
  prefs.parport_base=ppnr_to_ppaddr(prefs.parport_nr);
  if (rdi)
  {
    rdi->port_type=prefs.port_type;
    rdi->parport_base=prefs.parport_base;
    rdi->fast_parportaccess=prefs.fast_parportaccess;
    rdi->hwsel_type=prefs.hwsel_type;

    rdi->usb_use_id=prefs.usb_use_id;
    strncpy(rdi->usbport_id,prefs.usbport_id,30);

    rdi->hrpt_start_after_in_sync=prefs.hrpt_start_after_in_sync;
    rdi->hrpt_stop_after_out_sync=prefs.hrpt_stop_after_out_sync;
    rdi->hri_start_after_in_sync=prefs.hri_start_after_in_sync;
    rdi->hri_stop_after_out_sync=prefs.hri_stop_after_out_sync;

    rdi->height=prefs.hrpt_lines_per_screen;
    rdi->scan_ver=prefs.hrpt_scan_ver;
    rdi->scan_hor=(prefs.hrpt_scan_ver==N_S? W_E: E_W);

    strcpy(rdi->file_hrptbase,prefs.file_hrptbase);
    strcpy(rdi->file_hrptext,prefs.file_hrptext);

    strcpy(rdi->file_hribase,prefs.file_hribase);
    strcpy(rdi->file_hriext,prefs.file_hriext);
    strcpy(rdi->work_location,prefs.dir_name);

    sprintf(rdi->kepler_file,"%s%s",prefs.kep_dirname,prefs.kep_filename);
    rdi->sh_recchan=prefs.sh_recchan;
    rdi->save_flag=prefs.save_flag;
    rdi->auto_get_kepler=prefs.auto_get_kepler;
  }

  if (vwi)
  {
    vwi->grid_clr.red=prefs.grid_clr.red;
    vwi->grid_clr.green=prefs.grid_clr.green;
    vwi->grid_clr.blue=prefs.grid_clr.blue;
    vwi->sur_ulim=prefs.sur_ulim;
    vwi->dsp_llim=prefs.dsp_llim;
    vwi->linearize=prefs.linearize;
    vwi->fix_zoomfactor=prefs.fix_zoomfactor;
  }

  if (pci)
  {
    strcpy(pci->work_location,prefs.dir_name);
    strcpy(pci->kep_location,prefs.kep_dirname);
    pci->syncsignal_del=-1;
  }

}

void Load_Prefdef()
{
  prefs.parport_nr=PARPORT_NUMBER;
  prefs.fast_parportaccess=FALSE;
  prefs.hwsel_type=FALSE;

  prefs.hrpt_start_after_in_sync=6;
  prefs.hrpt_stop_after_out_sync=90;
  prefs.hri_start_after_in_sync=20;
  prefs.hri_stop_after_out_sync=20;

  prefs.hrpt_lines_per_screen=0;
  prefs.hrpt_scan_ver=N_S;
  prefs.sh_recchan=2;
  prefs.save_flag=TRUE;
  prefs.auto_get_kepler=FALSE;

  strcpy(prefs.file_hrptbase,DEF_HRPTBASE);
  strcpy(prefs.file_hrptext,DEF_HRPTEXT);

  strcpy(prefs.file_hribase,DEF_HRIBASE);
  strcpy(prefs.file_hriext,DEF_HRIEXT);

  strcpy(prefs.dir_name,DEF_RECDIR);

  prefs.font_size=0;
  prefs.grid_clr.red=(GRID_CLR>>16)&0xff;
  prefs.grid_clr.green=(GRID_CLR>>8)&0xff;
  prefs.grid_clr.blue=(GRID_CLR>>0)&0xff;

  prefs.sur_ulim=20;
  prefs.dsp_llim=10;
  prefs.fix_zoomfactor=FIX_ZOOM;
  prefs.linearize=FALSE;

/* Let window system determine where to place the windows */
  prefs.main_wnd_geom=deflt_wndpos;
  prefs.sub_wnd_geom=deflt_wndpos;

  prefs.main_wnd_x=0;
  prefs.main_wnd_y=0;
  prefs.main_wnd_w=0;
  prefs.main_wnd_h=0;

}

void get_dirs(char *progname,char **cur_dir,char **prog_dir,char **home_dir)
{
  if (*cur_dir)  free(*cur_dir);  *cur_dir=NULL;
  if (*prog_dir) free(*prog_dir); *prog_dir=NULL;
  if (*home_dir) free(*home_dir); *home_dir=NULL;

  *cur_dir=g_get_current_dir();
  *prog_dir=get_path(progname);
  *home_dir=g_get_home_dir();
  #if __GTK_WIN32__ == 1
  {
    if (!(*home_dir))
    {
      if (!strncmp(*cur_dir+1,":\\",2))
      {
        *home_dir=malloc(10);
        strncpy(*home_dir,*cur_dir,2);
        *((*home_dir)+2)=0;
      }
    }
  }
  #endif
}

void search_file(char *rootfile,char *file,
                 char *cur_dir,char *home_dir,char *prog_dir)
{
/* Search pref-file */
  if (cur_dir)
    sprintf(file,"%s%c%s",cur_dir,DIR_SEPARATOR,rootfile);

  if (!(exist_file(file)))
  {
    *file=0;
    if (home_dir)
      sprintf(file,"%s%c%s",home_dir,DIR_SEPARATOR,rootfile);

    if (!(exist_file(file)))
    {
      *file=0;
      if (prog_dir)
        sprintf(file,"%s%c%s",prog_dir,DIR_SEPARATOR,rootfile);

      if (!(exist_file(file)))
      {
        *file=0;
      }
    }
  }
  return;
}

/*************************************
 * Read preferences from file; first load defaults 
 *************************************/
int Read_Prefs(char *progname)
{
  FILE *fp;
  char l[100],*w1,*w2,*w3;

/* Load directory names: current, program location etc. */
  get_dirs(progname,&prefs.cur_dir,&prefs.prog_dir,&prefs.home_dir);

/* Load defaults */
  Load_Prefdef();

  *prefs.used_preffile=0;
  search_file(PREF_FILE,prefs.used_preffile,
              prefs.cur_dir,prefs.home_dir,prefs.prog_dir);


/* Open preference file */
  if (!(fp=fopen(prefs.used_preffile,"r")))
  {
    return 0;
  }

/* Start reading/parsing preferences file */
  while (fgets(l,100,fp))
  {
    if (!(w1=strtok(l,SEPARATE))) continue;
    if (!(w2=strtok(NULL,SEPARATE))) continue;

    if (!strcmp(w1,"main_window_size"))
    {
      if ((w3=strtok(w2,"x")))
      {
        prefs.main_wnd_w=atoi(w3);
        if ((w3=strtok(NULL,"x\n"))) prefs.main_wnd_h=atoi(w3);
      }
    }

    if (!strcmp(w1,"main_window_position"))
    {
      if ((w3=strtok(w2,"@")))
      {
        if (!strcmp(w3,"abs_wndpos")) prefs.main_wnd_geom=abs_wndpos;
        if (!strcmp(w3,"smart_wndpos")) prefs.main_wnd_geom=smart_wndpos;
        if (!strcmp(w3,"center_wndpos")) prefs.main_wnd_geom=center_wndpos;
        if ((w3=strtok(NULL,"x\n"))) prefs.main_wnd_x=atoi(w3);
        if ((w3=strtok(NULL,"x\n"))) prefs.main_wnd_y=atoi(w3);
      }
    }

    if (!strcmp(w1,"sub_window_position"))
    {
      if ((w3=strtok(w2,"@")))
      {
        if (!strcmp(w3,"abs_wndpos")) prefs.sub_wnd_geom=abs_wndpos;
        if (!strcmp(w3,"smart_wndpos")) prefs.sub_wnd_geom=smart_wndpos;
        if (!strcmp(w3,"center_wndpos")) prefs.sub_wnd_geom=center_wndpos;
        if ((w3=strtok(NULL,"x\n"))) prefs.sub_wnd_x=atoi(w3);
        if ((w3=strtok(NULL,"x\n"))) prefs.sub_wnd_y=atoi(w3);
      }
    }

    if (!strcmp(w1,"Port_type"))
    {
      if (!strcmp(w2,"usb"))      prefs.port_type=usb;
      else if (!strcmp(w2,"eth")) prefs.port_type=ethernet;
      else                        prefs.port_type=parport;
    }

    if (!strcmp(w1,"Parallel_port"))
      prefs.parport_nr=atoi(w2);

    if (!strcmp(w1,"Parallel_port_address"))
      prefs.parport_address=strtol(w2,NULL,16);

    if (!strcmp(w1,"Fast_Parportaccess"))
      prefs.fast_parportaccess=(*w2=='y'? TRUE : FALSE); 

    if (!strcmp(w1,"Hwtype_select"))
      prefs.hwsel_type=(!strcmp(w2,"dec_type")? TRUE : FALSE); 

    if (!strcmp(w1,"use_usbid"))
      prefs.usb_use_id=(*w2=='y'? TRUE : FALSE); 

    if (!strcmp(w1,"usb_id"))
      strcpy(prefs.usbport_id,w2);

    if (!strcmp(w1,"Font_size"))
    {
      prefs.font_size= atoi(w2);
    }

    if (!strcmp(w1,"Grid_color"))
    {
      prefs.grid_clr.red  =(strtol(w2,NULL,16)>>16)&0xff;
      prefs.grid_clr.green=(strtol(w2,NULL,16)>>8)&0xff;
      prefs.grid_clr.blue =(strtol(w2,NULL,16)>>0)&0xff;
    }

    if (!strcmp(w1,"Linearize"))
    {
      prefs.linearize=(!strcmp(w2,"on")? TRUE : FALSE);
    }

    if (!strcmp(w1,"Despeckle_limits"))
    {
      prefs.sur_ulim=atoi(w2);
      if (!(w2=strtok(NULL,SEPARATE))) continue;
      prefs.dsp_llim=atoi(w2);
    }

    if (!strcmp(w1,"Fixed_zoomfactor"))
    {
      prefs.fix_zoomfactor=atof(w2);
    }

    if (!strcmp(w1,"Hrpt_Record_Start_Stop_Times"))
    {
      prefs.hrpt_start_after_in_sync=atoi(w2);
      if (!(w2=strtok(NULL,SEPARATE))) continue;
      prefs.hrpt_stop_after_out_sync=atoi(w2);
    }
    if (!strcmp(w1,"Hri_Record_Start_Stop_Times"))
    {
      prefs.hri_start_after_in_sync=atoi(w2);
      if (!(w2=strtok(NULL,SEPARATE))) continue;
      prefs.hri_stop_after_out_sync=atoi(w2);
    }

    if (!strcmp(w1,"Hrpt_Record_Lines_per_screen"))
    {
      prefs.hrpt_lines_per_screen=atoi(w2);
    }
    if (!strcmp(w1,"Hrpt_Record_Scandirection"))
    {
      prefs.hrpt_scan_ver=(!strcmp(w2,"N_S")? N_S : 
                           !strcmp(w2,"S_N")? S_N :
                           scan_unk);
    }

    if (!strcmp(w1,"Hrpt_Record_Showchan"))
    {
      prefs.sh_recchan=atoi(w2);
    }

    if (!strcmp(w1,"Hrptrec_Save_rec"))
      prefs.save_flag=(*w2=='y'? TRUE : FALSE); 
    if (!strcmp(w1,"Hrptrec_Auto_getkepler"))
      prefs.auto_get_kepler=(*w2=='y'? TRUE : FALSE); 


    if (!strcmp(w1,"Hrpt_File_Base"))
    {
      strcpy(prefs.file_hrptbase,w2);
    }
    if (!strcmp(w1,"Hrpt_File_Extension"))
    {
      strcpy(prefs.file_hrptext,w2);
    }

    if (!strcmp(w1,"Hri_File_Base"))
    {
      strcpy(prefs.file_hribase,w2);
    }
    if (!strcmp(w1,"Hri_File_Extension"))
    {
      strcpy(prefs.file_hriext,w2);
    }

    if (!strcmp(w1,"Working_Dir"))
    {
      strcpy(prefs.dir_name,w2);
      if (*(prefs.dir_name+strlen(prefs.dir_name)-1) != DIR_SEPARATOR)
        sprintf(prefs.dir_name,"%s%c",prefs.dir_name,DIR_SEPARATOR);

    }
    if (!strcmp(w1,"Kepler_File_Dir"))
    {
      strcpy(prefs.kep_dirname,w2);
      if (*(prefs.kep_dirname+strlen(prefs.kep_dirname)-1) != DIR_SEPARATOR)
        sprintf(prefs.kep_dirname,"%s%c",prefs.kep_dirname,DIR_SEPARATOR);
    }
    if (!strcmp(w1,"Kepler_File_name"))
    {
      strcpy(prefs.kep_filename,w2);
    }
  }
  fclose(fp);

  fprintf(stderr,"Preference file %s found.\n",prefs.used_preffile);

  return 1;
}

void pri_prefline(FILE *fp,char *key,char *frmt,...)
{
  va_list arg;
  va_start(arg,frmt);
  fprintf(fp,"%-30s = ",key);
  vfprintf(fp,frmt,arg);
  va_end(arg);
}

/*************************************
 * Save preferences into pref. file
 *************************************/
void Save_Prefs()
{
  FILE *fp;

  if (!(fp=fopen(PREF_FILE,"w")))
  {
    Create_Message("Error","Can't open %s!",PREF_FILE);
    return;
  }

  pri_prefline(fp,"main_window_size","%dx%d\n",prefs.main_wnd_w,prefs.main_wnd_h);

  pri_prefline(fp,"main_window_position","");
  switch (prefs.main_wnd_geom)
  {
    case abs_wndpos:
      fprintf(fp,"abs_wndpos@%dx%d\n",prefs.main_wnd_x,prefs.main_wnd_y);
    break;
    case center_wndpos:
      fprintf(fp,"center_wndpos@%dx%d\n",prefs.main_wnd_x,prefs.main_wnd_y);
    break;
    default:
      fprintf(fp,"default\n");
    break;
  }

  pri_prefline(fp,"sub_window_position","");
  switch (prefs.sub_wnd_geom)
  {
    case abs_wndpos:
      fprintf(fp,"abs_wndpos@%dx%d\n",prefs.sub_wnd_x,prefs.sub_wnd_y);
    break;
    case center_wndpos:
      fprintf(fp,"center_wndpos@%dx%d\n",prefs.sub_wnd_x,prefs.sub_wnd_y);
    break;
    case smart_wndpos:
      fprintf(fp,"smart_wndpos@%dx%d\n",prefs.sub_wnd_x,prefs.sub_wnd_y);
    break;
    default:
      fprintf(fp,"default\n");
    break;
  }

  pri_prefline(fp,"Port_type","%s\n",
                     (prefs.port_type==usb?      "usb":
                      prefs.port_type==ethernet? "eth" :
                                                 "par"));
  pri_prefline(fp,"Parallel_port","%x\n",prefs.parport_nr);
  pri_prefline(fp,"Parallel_port_address","%x\n",prefs.parport_address);
  pri_prefline(fp,"Fast_Parportaccess","%s\n",
                     (prefs.fast_parportaccess? "yes" : "no"));

  pri_prefline(fp,"Hwtype_select","%s\n",
                     (prefs.hwsel_type? "dec_type" : "dec_gen"));

  pri_prefline(fp,"use_usbid","%s\n",(prefs.usb_use_id? "yes" : "no"));
  pri_prefline(fp,"usb_id","%s\n",prefs.usbport_id);

  pri_prefline(fp,"Font_size","%d\n",prefs.font_size);

  pri_prefline(fp,"Grid_color","0x%x\n",(prefs.grid_clr.red<<16)+
                                     (prefs.grid_clr.green<<8)+
                                     (prefs.grid_clr.blue) );
  pri_prefline(fp,"Linearize","%s\n",prefs.linearize? "on" : "off");
  pri_prefline(fp,"Despeckle_limits","%d %d\n",prefs.sur_ulim,prefs.dsp_llim);
  pri_prefline(fp,"Fixed_zoomfactor","%.2f\n",prefs.fix_zoomfactor);

  pri_prefline(fp,"Hrpt_Record_Start_Stop_Times","%d %d\n",
     prefs.hrpt_start_after_in_sync,prefs.hrpt_stop_after_out_sync);
  pri_prefline(fp,"Hri_Record_Start_Stop_Times","%d %d\n",
     prefs.hri_start_after_in_sync,prefs.hri_stop_after_out_sync);

  pri_prefline(fp,"Hrpt_Record_Lines_per_screen","%d\n",
    prefs.hrpt_lines_per_screen);
  pri_prefline(fp,"Hrpt_Record_Scandirection","%s\n",
    (prefs.hrpt_scan_ver==N_S? "N_S" : prefs.hrpt_scan_ver==S_N? "S_N" : "?"));
  pri_prefline(fp,"Hrpt_Record_Showchan","%d\n",prefs.sh_recchan);

  pri_prefline(fp,"Hrptrec_Save_rec","%s\n",
                     (prefs.save_flag? "yes" : "no"));
  pri_prefline(fp,"Hrptrec_Auto_getkepler","%s\n",
                     (prefs.auto_get_kepler? "yes" : "no"));

  pri_prefline(fp,"Hrpt_File_Base","%s\n",prefs.file_hrptbase);
  pri_prefline(fp,"Hrpt_File_Extension","%s\n",prefs.file_hrptext);

  pri_prefline(fp,"Hri_File_Base","%s\n",prefs.file_hribase);
  pri_prefline(fp,"Hri_File_Extension","%s\n",prefs.file_hriext);
  pri_prefline(fp,"Working_Dir","%s\n",prefs.dir_name);
  pri_prefline(fp,"Kepler_File_Dir","%s\n",prefs.kep_dirname);
  pri_prefline(fp,"Kepler_File_name","%s\n",prefs.kep_filename);

  fclose(fp);
}

/*************************************
 * Get preferences window commands  
 *************************************/
void pref_commands(GtkWidget *widget, gpointer data)
{
  GtkWidget *pref_window=NULL;
  RECORDINFO *rdi=Get_Recordinfo_db(widget);
  PICINFO *pci=Get_Picinfo_db(widget);
  VIEWINFO *vwi=Get_Viewinfo_db(widget);
  char *name=(char *)data;

  if (GTK_IS_WIDGET(widget))
    pref_window=gtk_widget_get_toplevel(widget);

  if (!strcmp(name,LAB_Exit))
  {
    Close_Window(pref_window);
  }

  if (!strcmp(name,LAB_Use))
  {
    Load_Prefs(rdi,vwi,pci);
  }

  if (!strcmp(name,LAB_Save))
  {
    Save_Prefs();
  }

  if (!strcmp(name,LAB_WNDPOS_DFLT))
  {
    prefs.main_wnd_geom=deflt_wndpos;
    prefs.sub_wnd_geom=deflt_wndpos;
  }
  if (!strcmp(name,LAB_WNDPOS_SMART))
  {
    prefs.main_wnd_geom=abs_wndpos;
    prefs.sub_wnd_geom=smart_wndpos;
  }

  if (!strcmp(name,LAB_WND_WIDTH))
  {
    prefs.main_wnd_w=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_WND_HEIGHT))
  {
    prefs.main_wnd_h=GTK_ADJUSTMENT(widget)->value;
  }

  if (!strcmp(name,LAB_USB))
  {
    prefs.port_type=usb;
    Show_Button(widget,LAB_SEL_HWTYPE,prefs.port_type==parport);
    Show_Button(widget,LAB_PARPORT,prefs.port_type==parport);
    Show_Button(widget,LAB_USBPORT,prefs.port_type==usb);
  }
  if (!strcmp(name,LAB_ETH))
  {
    prefs.port_type=ethernet;
    Show_Button(widget,LAB_SEL_HWTYPE,prefs.port_type==parport);
    Show_Button(widget,LAB_PARPORT,prefs.port_type==parport);
    Show_Button(widget,LAB_USBPORT,prefs.port_type==usb);
  }
  if (!strcmp(name,LAB_PAR))
  {
    prefs.port_type=parport;
    Show_Button(widget,LAB_SEL_HWTYPE,prefs.port_type==parport);
    Show_Button(widget,LAB_PARPORT,prefs.port_type==parport);
    Show_Button(widget,LAB_USBPORT,prefs.port_type==usb);
  }

  if (!strcmp(name,LAB_P1))
  {
    prefs.parport_nr=1;
    rdi->parport_base=ppnr_to_ppaddr(prefs.parport_nr);
    Sense_Button(pref_window,LAB_PE,FALSE);
  }
  if (!strcmp(name,LAB_P2))
  {
    prefs.parport_nr=2;
    rdi->parport_base=ppnr_to_ppaddr(prefs.parport_nr);
    Sense_Button(pref_window,LAB_PE,FALSE);
  }
  if (!strcmp(name,LAB_PN))
  {
    prefs.parport_nr=0;
    rdi->parport_base=prefs.parport_address;
    Sense_Button(pref_window,LAB_PE,TRUE);
  }

  if (!strcmp(name,LAB_PE))
  {
    rdi->parport_base=strtol(Get_Entry(pref_window,LAB_PE),NULL,16);
    prefs.parport_address=rdi->parport_base;
  }

  if (!strcmp(name,LAB_SEL_DECGEN))
  {
    prefs.hwsel_type=FALSE;
    rdi->hwsel_type=prefs.hwsel_type;
  }
  if (!strcmp(name,LAB_SEL_DECTYPE))
  {
    prefs.hwsel_type=TRUE;
    rdi->hwsel_type=prefs.hwsel_type;
  }

  if (!strcmp(name,LAB_FASTPPACC))
  {
    prefs.fast_parportaccess=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
    rdi->fast_parportaccess=prefs.fast_parportaccess;
  }

  if (!strcmp(name,LAB_LINEARIZE))
  {
    prefs.linearize=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
  }

  if (!strcmp(name,LAB_SPECK_SUL))
  {
    prefs.sur_ulim=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_SPECK_SPL))
  {
    prefs.dsp_llim=GTK_ADJUSTMENT(widget)->value;
  }

  if (!strcmp(name,LAB_FIXZOOM))
  {
    prefs.fix_zoomfactor=GTK_ADJUSTMENT(widget)->value;
  }


  if (!strcmp(name,LAB_HRPT_START))
  {
    prefs.hrpt_start_after_in_sync=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_HRPT_STOP))
  {
    prefs.hrpt_stop_after_out_sync=GTK_ADJUSTMENT(widget)->value;
  }


  if (!strcmp(name,LAB_HRI_START))
  {
    prefs.hri_start_after_in_sync=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_HRI_STOP))
  {
    prefs.hri_stop_after_out_sync=GTK_ADJUSTMENT(widget)->value;
  }

  if (!strcmp(name,LAB_LPS))
  {
    prefs.hrpt_lines_per_screen=GTK_ADJUSTMENT(widget)->value;
  }

  if (!strcmp(name,LAB_DIR_NSN))
  {
    if (Update_Togglelabel(widget))
      prefs.hrpt_scan_ver=S_N;
    else
      prefs.hrpt_scan_ver=N_S;
  }

  if (!strcmp(name,LAB_SAVREC))
  {
    prefs.save_flag=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
    rdi->save_flag=prefs.save_flag;
  }
  if (!strcmp(name,LAB_KEPREC))
  {
    prefs.auto_get_kepler=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
    rdi->auto_get_kepler=prefs.auto_get_kepler;
  }

  {
    char *ext;
    ext=soption_menu_get_history(Find_Widget(widget,LAB_VCHR));
    if ((ext) && (!strcmp(name,ext)))
    {
      prefs.sh_recchan=atoi(name+strlen(LAB_CHX_V));
    }
  }

  if (!strcmp(name,LAB_HRPTFILE_BASE))
  {
    strcpy(prefs.file_hrptbase,
           Get_Entry(gtk_widget_get_toplevel(widget),
                                                   LAB_HRPTFILE_BASE));
  }
  if (!strcmp(name,LAB_HRPTFILE_EXT))
  {
    strcpy(prefs.file_hrptext,
           Get_Entry(gtk_widget_get_toplevel(widget),
                                                   LAB_HRPTFILE_EXT));
  }


  if (!strcmp(name,LAB_HRIFILE_BASE))
  {
    strcpy(prefs.file_hribase,
        Get_Entry(gtk_widget_get_toplevel(widget),
                                                    LAB_HRIFILE_BASE));
  }
  if (!strcmp(name,LAB_HRIFILE_EXT))
  {
    strcpy(prefs.file_hriext,
        Get_Entry(gtk_widget_get_toplevel(widget),
                                                    LAB_HRIFILE_EXT));
  }

  if (!strcmp(name,LAB_DIR_SAVE))
  {
    strcpy(prefs.dir_name,
      Get_Entry(gtk_widget_get_toplevel(widget),LAB_DIR_SAVE));
  }
  if (!strcmp(name,LAB_DIR_KEP))
  {
    strcpy(prefs.kep_dirname,
      Get_Entry(gtk_widget_get_toplevel(widget),LAB_DIR_KEP));
  }
  if (!strcmp(name,LAB_FILE_KEP))
  {
    strcpy(prefs.kep_filename,
      Get_Entry(gtk_widget_get_toplevel(widget),LAB_FILE_KEP));
  }

  if (!strcmp(name,LAB_FSIZE))
  {
    prefs.font_size=GTK_ADJUSTMENT(widget)->value;
  }

  if (!strcmp(name,LAB_GRID_RED))
  {
    prefs.grid_clr.red=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_GRID_GREEN))
  {
    prefs.grid_clr.green=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_GRID_BLUE))
  {
    prefs.grid_clr.blue=GTK_ADJUSTMENT(widget)->value;
  }
}

int usb_list_nndevs();

void usb_commands(GtkWidget *widget, gpointer data)
{
  GtkWidget *pref_window=NULL;
  char *name=(char *)data;
#ifdef __ADD_USB__ 

  if (GTK_IS_WIDGET(widget))
    pref_window=gtk_widget_get_toplevel(widget);

  if (!strcmp(name,LAB_USBID))
  {
    strcpy(prefs.usbport_id,
      Get_Entry(gtk_widget_get_toplevel(widget),LAB_USBID));
  }
  if (!strcmp(name,LAB_USB_USEID))
  {
    Sense_Button(widget,LAB_USBID,Get_Button(pref_window,LAB_USB_USEID));
    prefs.usb_use_id=TRUE;
  }
  if (!strcmp(name,LAB_USB_USEFIRST))
  {
    prefs.usb_use_id=FALSE;
  }


#define LAB_USBINFO "usblist"
  if (!strcmp(name,LAB_TESTUSB))
  {
    int n,nrdev;
    GtkWidget *txt=Find_Widget(widget,LAB_USBINFO)->parent;
    FT_DEVICE_LIST_INFO_NODE *devInfo=NULL;
    nrdev=usb_list_ndevs(&devInfo);
    Clear_Text(txt);
    if (nrdev<-1)
      Add_Text(txt,10,"%s","No USB port available.\n");
    else if (nrdev==0)
      Add_Text(txt,10,"%s","No FTD USB device connected.\n");
    else
    {
      Add_Text(txt,10,"%-23s | %s\n","SerialNumber (USB ID)"  ,"Description");
      Add_Text(txt,10,"%-23s-+-%s\n","-----------------------","-----------");
      for (n=0; n<nrdev; n++)
      {
        Add_Text(txt,12,"%-23s | %s\n",devInfo[n].SerialNumber,devInfo[n].Description);
      }
    }
    if (devInfo) free(devInfo);
  }

#endif
}

/*************************************
 * Open Preferences window
 *************************************/
void MenuPrefs(GtkWidget *widget)
{
  GtkWidget *pref_window;
  GtkWidget *main_window;
  GtkWidget *wn,*wa[3],*wb[7],*wc[4],*wd[10],*we[4],*wf[3];
  GtkWidget *wt[3],*wo[4];
  GtkWidget *wz;

  main_window=Find_Parent_Window(widget);

/* Create new window or pop-up it if it already exist. */
  if (!(pref_window=Create_Window(main_window,0,0,Lab_Prefwnd,NULL))) return;

/* Add buttons etc. at top*/
  if (exist_file(PREF_FILE))
    wt[1]=Create_Entry(LAB_LOPREF,NULL,PREF_FILE); 
  else
    wt[1]=Create_Entry(LAB_LOPREF,NULL,"Not found."); 

  if (exist_file(prefs.used_preffile))
    wt[2]=Create_Entry(LAB_GLPREF,NULL,prefs.used_preffile); 
  else
    wt[2]=Create_Entry(LAB_GLPREF,NULL,"Not found"); 

  wt[0]=Pack("Preference file",'v',wt[1],5,wt[2],5,NULL);

/* ---------- Part Window ---------- */
  wa[1]=Create_Optionmenu(LAB_WNDPOS,pref_commands,0,
                        LAB_WNDPOS_DFLT,
                        LAB_WNDPOS_SMART,
                        0
                       );

  wa[2]=Create_ButtonArray(LAB_WNDSIZE,pref_commands,2,
                         SPIN,LAB_WND_WIDTH,"%d%d%d",prefs.main_wnd_w,0,1600,
                         SPIN,LAB_WND_HEIGHT,"%d%d%d",prefs.main_wnd_h,0,1200,
                         0
                        );
  wa[0]=Pack("Window attributes",'h',wa[1],5,wa[2],5,NULL);

/* ---------- Part Interface ---------- */
//  wb[1]=Create_Check(LAB_ETH,pref_commands,prefs.port_type);
  wb[1]=Create_ButtonArray(LAB_INTERF,pref_commands,3,
                        RADIO,LAB_PAR,
#ifdef __ADD_ETH__
                        RADIO,LAB_ETH,
#endif
#ifdef __ADD_USB__
                        RADIO,LAB_USB,
#endif
                        0);

  wb[2]=Create_ButtonArray(LAB_PARPORT,pref_commands,2,
                        RADIO,LAB_P1,
                        RADIO,LAB_P2,
                        RADIO,LAB_PN,
                        ENTRY,LAB_PE,"%10s","",
                        CHECK,LAB_FASTPPACC,
                        0
                       );

  wb[3]=Create_ButtonArray(LAB_SEL_HWTYPE,pref_commands,2,
                         RADIO,LAB_SEL_DECGEN,
                         RADIO,LAB_SEL_DECTYPE,
                         0
                        );

  wb[4]=Create_ButtonArray(LAB_USBCHOICE,usb_commands,2,
                        RADIOs,LAB_USB_USEFIRST,
                        LABEL,"",
                        RADIOn,LAB_USB_USEID,
                        ENTRY,LAB_USBID,"%30s","",
                        0
                       );
#ifdef __ADD_USB__
  wb[5]=Create_Text(LAB_USBINFO,FALSE,"Courier 10");
  wb[6]=Create_Button(LAB_TESTUSB,usb_commands);
  wb[5]=Pack("Found FTD devices",'v',wb[5],1,wb[6],1,NULL);
  wb[4]=Pack(LAB_USBPORT,'v',wb[4],5,wb[5],5,NULL);
#else
  wb[4]=NULL;
#endif
  wb[0]=Pack("",'v',wb[1],10,wb[2],10,wb[3],2,wb[4],2,NULL);

/* ---------- Part Dirs/files ---------- */
  wc[1]=Create_Entry(LAB_DIR_SAVE,pref_commands,"%-30s",prefs.dir_name); 
  wc[2]=Create_Entry(LAB_DIR_KEP,pref_commands,"%-30s",prefs.kep_dirname); 
  wc[3]=Create_Entry(LAB_FILE_KEP,pref_commands,"%-30s",prefs.kep_filename); 
  wc[2]=Pack("Kepler file",'v',wc[2],3,wc[3],3,NULL);
  wc[0]=Pack("",'v',wc[1],10,wc[2],2,NULL);



/* ---------- Part Record ---------- */
  wd[1]=Create_ButtonArray(LAB_HRPTRECTIMES,pref_commands,2,
          SPIN,LAB_HRPT_START,"%d%d%d",prefs.hrpt_start_after_in_sync,0,1023,
          SPIN,LAB_HRPT_STOP,"%d%d%d",prefs.hrpt_stop_after_out_sync,0,1023,
                         0
                        );

  wd[2]=Create_ButtonArray(LAB_HRPTFILENAMES,pref_commands,2,
          ENTRY,LAB_HRPTFILE_BASE,"%-5s",prefs.file_hrptbase,
          ENTRY,LAB_HRPTFILE_EXT ,"%-5s",prefs.file_hrptext,
                         0
                        );

  wd[3]=Create_ButtonArray(LAB_HRIRECTIMES,pref_commands,2,
          SPIN,LAB_HRI_START,"%d%d%d",prefs.hri_start_after_in_sync,0,1023,
          SPIN,LAB_HRI_STOP,"%d%d%d",prefs.hri_stop_after_out_sync,0,1023,
                         0
                        );
  wd[4]=Create_ButtonArray(LAB_HRIFILENAMES,pref_commands,2,
          ENTRY,LAB_HRIFILE_BASE,"%-5s",prefs.file_hribase,
          ENTRY,LAB_HRIFILE_EXT ,"%-5s",prefs.file_hriext,
                         0
                        );

  wd[5]=Create_Spin(LAB_LPS,pref_commands,"%d%d%d",
                            prefs.hrpt_lines_per_screen,0,10000);
  wd[6]=Create_Toggle(LAB_DIR_NSN,pref_commands,FALSE);
  wd[7]=Create_Optionmenu(LAB_VCHR,pref_commands,prefs.sh_recchan-1,
                        LAB_CH1_V,LAB_CH2_V,LAB_CH3_V,LAB_CH4_V,LAB_CH5_V,
                        LAB_CH6_V,LAB_CH7_V,LAB_CH8_V,LAB_CH9_V,LAB_CH10_V,
                        0);

  wd[6]=Pack(NULL,'v',wd[6],5,wd[7],5,NULL);
  wd[5]=Pack("Show during record",'h',wd[5],1,wd[6],1,NULL);

  wd[8]=Create_Check(LAB_SAVREC,pref_commands,prefs.save_flag);
  wd[9]=Create_Check(LAB_KEPREC,pref_commands,prefs.auto_get_kepler);
  wd[8]=Pack("Action after record",'h',wd[8],1,wd[9],1,NULL);

  wd[1]=Pack("(C)HRPT parameters",'v',wd[1],5,wd[2],5,wd[5],5,wd[8],5,NULL);
  wd[2]=Pack("HRI parameters",'v',wd[3],5,wd[4],5,NULL);
  wd[0]=Pack(NULL,'h',wd[1],10,wd[2],10,NULL);


/* ---------- Part Algos ---------- */

  we[1]=Create_Check(LAB_LINEARIZE,pref_commands,prefs.linearize);
  we[2]=Create_Spin(LAB_FIXZOOM,pref_commands,"%f%f%f",
                                      prefs.fix_zoomfactor,1.2,10.);
  we[3]=Create_ButtonArray(LAB_DESPEC_LIM,pref_commands,2,
                         SPIN,LAB_SPECK_SUL,"%d%d%d",prefs.sur_ulim,0,1023,
                         SPIN,LAB_SPECK_SPL,"%d%d%d",prefs.dsp_llim,0,1023,
                         0
                        );
  we[1]=Pack("",'h',we[1],10,NULL);
  we[2]=Pack("",'h',we[2],10,NULL);
  we[0]=Pack(NULL,'h',we[1],10,we[2],10,NULL);
  we[0]=Pack(NULL,'v',we[0],10,we[3],10,NULL);

/* ---------- Part Colors ---------- */
  wf[1]=Create_Spin(LAB_FSIZE,pref_commands,"%d%d%d%d",
                            5,prefs.font_size,0,500);

  wf[2]=Create_ButtonArray(LAB_GRIDCLR,pref_commands,3,
                        SPIN,LAB_GRID_RED,"%8d%8d%8d",prefs.grid_clr.red,0,255,
                        SPIN,LAB_GRID_GREEN,"%8d%8d%8d",prefs.grid_clr.green,0,255,
                        SPIN,LAB_GRID_BLUE,"%8d%8d%8d",prefs.grid_clr.blue,0,255,
                         0
                        );
  wf[0]=Pack(NULL,'v',wf[1],10,wf[2],10,NULL);

/* The notebook */
  wn=Create_Notebook(NULL,GTK_POS_TOP,
                     "Window"         ,wa[0],
                     "Interface"      ,wb[0],
                     "Dirs/files"     ,wc[0],
                     "Record"         ,wd[0],
                     "Algos"          ,we[0],
                     "Colours & Fonts",wf[0],
                     NULL);

/* ---------- Buttons bottom ---------- */
  wo[1]=Create_Button(LAB_Use,pref_commands);
  wo[2]=Create_Button(LAB_Save,pref_commands);
  wo[3]=Create_Button(LAB_Exit,pref_commands);

  wo[0]=Pack("",'h',wo[1],10,wo[2],10,wo[3],10,NULL);

  wz=Pack(NULL,'v',wt[0],5,wn,5,wo[0],5,NULL);
  gtk_container_add(GTK_CONTAINER(pref_window),wz);

  gtk_widget_show_all(pref_window);

/* Set buttons to current state */
  switch(prefs.port_type)
  {
    case ethernet : Set_Button(pref_window,LAB_ETH,TRUE); break;
    case usb      : Set_Button(pref_window,LAB_USB,TRUE); break;
    default       : Set_Button(pref_window,LAB_PAR,TRUE); break;
  }

  Sense_Button(pref_window,LAB_PE,FALSE);
  switch(prefs.parport_nr)
  {
    case 1 : Set_Button(pref_window,LAB_P1,TRUE); break;
    case 2 : Set_Button(pref_window,LAB_P2,TRUE); break;
    default: Set_Button(pref_window,LAB_PN,TRUE);
             Sense_Button(pref_window,LAB_PE,TRUE);
    break;
  }
  Set_Entry(pref_window,LAB_PE,"%x",prefs.parport_address);

  switch(prefs.port_type)
  {
    case usb:
      Set_Button(pref_window,LAB_USB,TRUE);
    break;
    case ethernet:
      Set_Button(pref_window,LAB_ETH,TRUE);
    break;
    default:
      Set_Button(pref_window,LAB_PAR,TRUE);
      Show_Button(pref_window,LAB_USBPORT,FALSE);
    break;
  }

  if (prefs.usb_use_id)
    Set_Button(pref_window,LAB_USB_USEID,TRUE);
  else
    Set_Button(pref_window,LAB_USB_USEFIRST,TRUE);

  Set_Entry(pref_window,LAB_USBID,"%s",prefs.usbport_id);

  Set_Button(pref_window,LAB_SEL_DECTYPE,prefs.hwsel_type);
  Set_Button(pref_window,LAB_VCHR,prefs.sh_recchan-1);

  if (prefs.fast_parportaccess)
    Set_Button(pref_window,LAB_FASTPPACC,TRUE);
  else
    Set_Button(pref_window,LAB_FASTPPACC,FALSE);

  switch(prefs.main_wnd_geom)
  {
    case abs_wndpos: Set_Button(pref_window,LAB_WNDPOS,1);  break;
    default        : Set_Button(pref_window,LAB_WNDPOS,0);  break;
  }

  switch(prefs.hrpt_scan_ver)
  {
    case S_N: Set_Button(pref_window,LAB_DIR_NSN,TRUE);  break;
    default : Set_Button(pref_window,LAB_DIR_NSN,FALSE);  break;
  }

  Set_Button(pref_window,LAB_SAVREC,prefs.save_flag);
  Set_Button(pref_window,LAB_KEPREC,prefs.auto_get_kepler);

}
