/**************************************************
 * RCSId: $Id$
 *
 * Draw related functions.
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log$
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 R. Alblas. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/

#include <stdio.h>
#include <malloc.h>

#include "wsat.h"

#define SCHEID " \n	"
gboolean get_next_slide(PICINFO *pci,VIEWINFO *vwi,gboolean start)
{
  static FILE *fp;
  static gboolean is_film;
  char l[100],*p;

  if ((start) || (!fp))
  {
    if ((detect_file(pci->file_name)) != unknown_filetype)
    {
      if (!(fp=fopen(pci->file_name,"rb"))) return FALSE;
      is_film=TRUE;
    }
    else
    {
      if (!(fp=fopen(pci->file_name,"r"))) return FALSE;
      is_film=FALSE;
    }


    return TRUE;
  }

  if (is_film)
  {
    PICINFO pcix;
    FILE *fpo;
    int i;
    long start_pos;

    start_pos=ftell(fp);
    pcix.fp=fp;
    if (!(read_ppm_header(&pcix)))
    {
      int nrbytes=pcix.width*pcix.height*(pcix.depth>8? 2 : 1);
      if (pci->file_type==PPM) nrbytes*=3;
      fseek(fp,nrbytes,SEEK_CUR);
      vwi->foffset=start_pos;
      return TRUE;
    }

    return FALSE;
  }
  else
  {
    while (fgets(l,100,fp))
    {
      if (!(p=strtok(l,SCHEID))) continue;
      strcpy(pci->file_name,p);
      return TRUE;
    }
  }
  if (fp) fclose(fp); fp=NULL;
  return FALSE;
}


int run_slides(gpointer p)
{
  GtkWidget *window=Find_Window((GtkWidget *)p,Lab_Main);

  PICINFO *pci=Get_Picinfo_db(window);
  VIEWINFO *vwi=Get_Viewinfo_db(window);

  while (get_next_slide(pci,vwi,FALSE))
  {
    struct zoomvals zval=vwi->zval;
    int lmi=vwi->lum_min;
    int lma=vwi->lum_max;

    if ((!open_file(pci,vwi,NULL)))
    {
      char tmp[100];
      if (lmi!=lma)
      {
        vwi->zval=zval;
        vwi->lum_min=lmi;
        vwi->lum_max=lma;
      }
      if (!*pci->sat_name)
        sprintf(tmp,"%s    Slide %s",Lab_Main,strip_filename(pci->file_name));
      else
        sprintf(tmp,"%s    %s",Lab_Main,pci->sat_name);
      gtk_window_set_title((GtkWindow *)Find_Window(window,Lab_Main),tmp);
      configure_event_mainwt(Find_Widget(window,"GTK_DRAWING_AREA"),NULL);
    }

    if (!vwi->slide_mode) break;

    return TRUE;
  }
  gtk_window_set_title((GtkWindow *)Find_Window(window,Lab_Main),Lab_Main);

  if (vwi->slide_mode) do_slide((GtkWidget *)p,TRUE);

  vwi->foffset=0;

  return FALSE;
}


void do_slide(GtkWidget *widget,gboolean start)
{
  PICINFO *pci=Get_Picinfo_db(widget);
  VIEWINFO *vwi=Get_Viewinfo_db(widget);
  vwi->refresh_pic=TRUE;

  if (!vwi->slide_speed) vwi->slide_speed=1000;
  if (start)
    if (!(get_next_slide(pci,vwi,TRUE))) return;

  gtk_timeout_add(vwi->slide_speed,run_slides,(gpointer)widget);

}

extern PREFS prefs;
#define LAB_FILM "Film"
#define LAB_SPEED "Speed"
#define LAB_Run "Run/Stop"

void film_cmds(GtkWidget *widget, gpointer data)
{
  GtkWidget *main_window=Find_Window(Find_Parent_Window(widget),Lab_Main);
  GtkWidget *film_window=Find_Window(main_window,LAB_FILM);
  VIEWINFO *vwi=Get_Viewinfo_db(widget);
  PICINFO *pci=Get_Picinfo_db(widget);
  char *name=(char *)data;

/* Take over entered zoom values */
  if (!strcmp(name,LAB_SPEED))
  {
    vwi->slide_speed=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_Run))
  {
    Update_Togglelabel(widget);
    if (Get_Button(widget,LAB_Run))
    {
      vwi->slide_mode=TRUE;
      do_slide(widget,FALSE);
    }
    else
      vwi->slide_mode=FALSE;
  }
}

void Menu_Film(GtkWidget *widget)
{
  GtkWidget *film_window,*main_window,*w[5];
  GdkColor startbut_clr,startbutsclr;

  VIEWINFO *vwi=Get_Viewinfo_db(widget);
  PICINFO *pci=Get_Picinfo_db(widget);
  main_window=Find_Parent_Window(widget);

  startbut_clr.red=0xffff; startbut_clr.green=0x5555; startbut_clr.blue=0x5555;
  startbutsclr.red=0xffff; startbutsclr.green=0x8888; startbutsclr.blue=0x8888;

/* Create new window or pop-up it if it already exist. */
  if (!(film_window=Create_Window(main_window,0,0,LAB_FILM,NULL))) return;

/* Add buttons */
  if (!vwi->slide_speed) vwi->slide_speed=1000;
  w[1]=Create_Spin(LAB_SPEED,film_cmds,"%4d%4d%4d%4d",100,vwi->slide_speed,100,60000);
  w[2]=Create_Toggle(LAB_Run,film_cmds,FALSE);
  Set_Color(w[2],GTK_RC_BG,NULL,&startbutsclr,&startbut_clr,&startbut_clr,NULL);

  w[0]=SPack("","h",w[1],"5ef",w[2],"5ef",NULL);

  gtk_container_add(GTK_CONTAINER(film_window),w[0]);
  gtk_widget_show(film_window);
  place_window(film_window,prefs.sub_wnd_x,prefs.sub_wnd_y,prefs.sub_wnd_geom);
}

