/**************************************************
 * RCSId: $Id$
 *
 * Parallel port test
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log$
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 R. Alblas. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/

#include <asm/io.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include "wsat.h"
#include "pp_dec_interf.h"

#define Lab_Testwnd "Test"
#define LAB_Read1pp "1x Read"
#define LAB_Readcpp "Cont. Read"
#define LAB_Readcnt "!teller"

#define LAB_ZMTM "Zoom time"
#define LAB_OSTM "Offset time"
#define LAB_PRTM "Pre time"
#define LAB_TExit "Exit"

extern PREFS prefs;
static int time_zoom;
static int offset;
static int time_pre;

#define ARRLEN 10000
#define NRSIG 20

typedef struct plotsig
{
  struct plotsig *prev,*next;
  unsigned char *arr;
  char name[20];
  GdkColor clr;
} PLOTSIG;

gboolean read_parsigs(RECORDINFO *rdi,PLOTSIG *plotsig,int n,gboolean bitsplit)
{
  int addr_portd=rdi->parport_base;
  int addr_portc=rdi->parport_base+2;
  int i,j,datstat;
  int gen1_dec0=rdi->gen1_dec0;
  char *arr[NRSIG];
  PLOTSIG *ps1;

  if (ioperm(addr_portd,3,1))
  {
    Create_Message("Error",PAR_ERR_MESSAGE,addr_portd);
    return FALSE;
  }

  for (ps1=plotsig,i=0; ps1; ps1=ps1->next,i++)
  {
    if (i<NRSIG) arr[i]=ps1->arr;

  }

  for (i=0; i<time_pre; i++)
  {
    GET_DS_PP(datstat,addr_portd);

    if (!READYN_PP(datstat))
      STROBE_PP(gen1_dec0,addr_portc);
  }

  for (i=0; i<n-10; i++)
  {
    int ns=0;
    GET_DS_PP(datstat,addr_portd);

    if (!READYN_PP(datstat))
      STROBE_PP(gen1_dec0,addr_portc);

    if (bitsplit)
    {
      for (j=7; j>=0; j--)
        *(arr[ns++]+i)=(datstat&(1<<j)? 0xff : 0);
    }
    else
    {
      *(arr[ns++]+i)=datstat&0xff;
    }
    *(arr[ns++]+i)=((datstat>>12)&0x2 ? 0xff : 0);
    *(arr[ns++]+i)=((datstat>>12)&0x1 ? 0xff : 0);
    *(arr[ns++]+i)=(READYN_PP(datstat)? 0xff : 0);
    *(arr[ns++]+i)=(STARTFRAME_PP(datstat)? 0xff : 0);
    *(arr[ns++]+i)=(INSYNC_PP(datstat)? 0xff : 0);
  }
  return TRUE;
}


void draw_parsigs(GtkWidget *widget,PLOTSIG *plotsig,int n,int offset)
{
  int i,sig;
  GtkWidget *window=gtk_widget_get_toplevel(widget);
  GtkWidget *drawing_area=Find_Widget(window,"GTK_DRAWING_AREA");
  RGBI *rgbi=Get_RGBI(drawing_area);
  PLOTSIG *ps;
  int left_start=100;
  int w_xmax;
  int height;

  int sigy;
  int sig_h,sig_dy;
  int nrsig=0;

  if (!drawing_area) return;
  {
    w_xmax=drawing_area->allocation.width-1;
    height=drawing_area->allocation.height;
  }
  for (ps=plotsig; ps; ps=ps->next) nrsig++;

  sig_dy=height/nrsig;
  sig_h=sig_dy*.8;

  if (!w_xmax) return;

  for (ps=plotsig,sig=0; ps; ps=ps->next,sig++)
  {
    int xp,x,yp,y;
    sigy=sig*sig_dy;

    draw_rgbstring(rgbi,&ps->clr,10,sigy+10,ps->name);
    for (i=1; i<n; i++) 
    {
      if (i+offset>=ARRLEN) break;
      xp=(i-1)*w_xmax/n + left_start; 
      x=i*w_xmax/n + left_start;
      yp=sigy + ((255-ps->arr[i-1+offset])*sig_h)/255;
      y =sigy + ((255-ps->arr[i+offset])*sig_h)/255;
      draw_rgbline(rgbi,&ps->clr,xp,yp,xp,y);
      draw_rgbline(rgbi,&ps->clr,xp,y,x,y);
    } 

  }
  expose_event(drawing_area,NULL);
}


PLOTSIG *Create_Plotsig(PLOTSIG *ps,int arrlen,GdkColor clr,char *name)
{
  PLOTSIG *psnw;
  psnw=calloc(1,sizeof(PLOTSIG));
  psnw->arr=(unsigned char *)malloc(ARRLEN);
  psnw->clr=clr;
  strcpy(psnw->name,name);
  if (ps)
  {
    while (ps->next) ps=ps->next;
    ps->next=psnw; psnw->prev=ps;
  }

  return psnw;
}

void Remove_Plotsig(PLOTSIG *ps)
{
  PLOTSIG *psnext;
  while (ps)
  {
    psnext=ps->next;
    free(ps->arr);
    free(ps);
    ps=psnext;
  }
}

PLOTSIG *create_plotsigs(gboolean bitsplit)
{
  GdkColor clr[5];
  PLOTSIG *plotsig=NULL;
  int i;
/* sig 0 color */
  clr[0].red=0;
  clr[0].green=0xffff;
  clr[0].blue=0;

/* sig 1 color */
  clr[1].red=0xffff;
  clr[1].green=0;
  clr[1].blue=0;

/* sig 2 color */
  clr[2].red=0;
  clr[2].green=0xffff;
  clr[2].blue=0xffff;

/* sig 3 color */
  clr[3].red=0xffff;
  clr[3].green=0xffff;
  clr[3].blue=0;

/* sig 4 color */
  clr[4].red=0xffff;
  clr[4].green=0;
  clr[4].blue=0xffff;

  if (bitsplit)
  {
    for (i=7; i>=0; i--)
    {
      plotsig=Create_Plotsig(plotsig,ARRLEN,clr[0],"data");
    }
  }
  else
  {
    plotsig=Create_Plotsig(plotsig,ARRLEN,clr[0],"Data");
  }
  plotsig=Create_Plotsig(plotsig,ARRLEN,clr[4],"Pin 12");
  plotsig=Create_Plotsig(plotsig,ARRLEN,clr[4],"Pin 13");
  plotsig=Create_Plotsig(plotsig,ARRLEN,clr[1],"Readyn");
  plotsig=Create_Plotsig(plotsig,ARRLEN,clr[2],"Frame");
  plotsig=Create_Plotsig(plotsig,ARRLEN,clr[3],"Sync");

  while (plotsig->prev) plotsig=plotsig->prev;
  return plotsig;
}

static PLOTSIG *plotsig;
static gboolean bitsplit;
gboolean draw_pportpins(GtkWidget *widget)
{
  GtkWidget *window;
  GtkWidget *drawing_area;
  RECORDINFO *rdi;

  int lensig;
  static int cnt;

  window=gtk_widget_get_toplevel(widget);
  if (!window) widget=widget->parent;
  window=gtk_widget_get_toplevel(widget);
  rdi=Get_Recordinfo_db(widget);

  if (time_zoom<1) time_zoom=1;
  if (time_pre<1) time_pre=1;

  lensig=ARRLEN/time_zoom;

  drawing_area=Find_Widget(window,"GTK_DRAWING_AREA");

//  Renew_RGBBuf(window);
  Renew_RGBBuf(drawing_area);

  if (read_parsigs(rdi,plotsig,ARRLEN,bitsplit))
    draw_parsigs(widget,plotsig,lensig,offset);

  expose_event(drawing_area,NULL);

  cnt++;
  Set_Entry(window,LAB_Readcnt,"%d",cnt);
  return TRUE;
}


static GtkWidget *wtmp;
gint configure_event_test(GtkWidget *widget,GdkEventConfigure *event)
{
  Renew_RGBBuf(widget);
  draw_pportpins(widget);

  return TRUE;
}

void test_commands(GtkWidget *widget, gpointer data)
{
  GtkWidget *test_window=NULL;
  static int to=0;
  GtkWidget *drawing_area=NULL;

  char *name=(char *)data;
  if (GTK_IS_WIDGET(widget))
  {
    test_window=gtk_widget_get_toplevel(widget);
    drawing_area=Find_Widget(test_window,"GTK_DRAWING_AREA");
  }
  

  if (!strcmp(name,LAB_Read1pp))
  {
    if (to) gtk_timeout_remove(to); to=0;
    Set_Button(test_window,LAB_Readcpp,FALSE);
    draw_pportpins(widget);
  }

  if (!strcmp(name,LAB_Readcpp))
  {
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
    {
      to=gtk_timeout_add(1*1000,(GtkFunction)draw_pportpins,widget);
    }
    else
    {
      if (to) gtk_timeout_remove(to); to=0;
    }
    draw_pportpins(widget);
  }

  if (!strcmp(name,LAB_ZMTM))
  {
    time_zoom=GTK_ADJUSTMENT(widget)->value;
//    Renew_RGBBuf(gtk_widget_get_toplevel(wtmp));
    if (drawing_area) Renew_RGBBuf(drawing_area);
    draw_parsigs(wtmp,plotsig,ARRLEN/time_zoom,offset);
  }
  if (!strcmp(name,LAB_OSTM))
  {
    offset=GTK_ADJUSTMENT(widget)->value;
//    Renew_RGBBuf(gtk_widget_get_toplevel(wtmp));
    if (drawing_area) Renew_RGBBuf(drawing_area);
    draw_parsigs(wtmp,plotsig,ARRLEN/time_zoom,offset);
  }

  if (!strcmp(name,LAB_PRTM))
  {
    time_pre=GTK_ADJUSTMENT(widget)->value;
  }

/* Exit lum-window */
  if (!strcmp(name,LAB_TExit))
  {
    if (plotsig) Remove_Plotsig(plotsig); plotsig=NULL;
    Close_Window(test_window);
  }

}

void Menu_Test(GtkWidget *widget,gpointer data)
{
  GtkWidget *w1,*w2,*w3,*w3b,*w4,*w5,*w6;
  GtkWidget *wa,*wz;
  GtkWidget *test_window;
  GtkWidget *main_window;
  GtkWidget *canvas;
  RECORDINFO *rdi=Get_Recordinfo_db(widget);

  if (ioperm(rdi->parport_base,3,1))
  {
    Create_Message("Error",PAR_ERR_MESSAGE,rdi->parport_base);
    return;
  }

  main_window=Find_Parent_Window(widget);

/* Create new window or pop-up it if it already exist. */
  if (!(test_window=Create_Window(main_window,640,280,Lab_Testwnd,NULL)))
    return;

  bitsplit=TRUE;
  plotsig=create_plotsigs(bitsplit);
  canvas=Create_Canvas(test_window,0,0,configure_event_test,
                                  NULL,NULL,NULL);

  w1=Create_Button(LAB_TExit,test_commands);
  w2=Create_Button(LAB_Read1pp,test_commands);
  w3=Create_Toggle(LAB_Readcpp,test_commands,FALSE);
  w3b=Create_Entry(LAB_Readcnt,NULL,"    ");
  w4=Create_Spin(LAB_ZMTM,test_commands,"%d%d%d",1,1,500);
  w5=Create_Spin(LAB_OSTM,test_commands,"%d%d%d%d",100,0,0,ARRLEN);
  w6=Create_Spin(LAB_PRTM,test_commands,"%d%d%d%d",100,100,100,100000);


/* Pack buttons and canvas into window */
  wa=Pack("",'h',w1,10,w2,10,w3,10,w3b,10,w4,10,w5,10,w6,10,NULL);
  wz=Pack(NULL,'v',canvas,3,wa,5,NULL,5,NULL);

  gtk_container_add(GTK_CONTAINER(test_window),wz);

  gtk_widget_show(test_window);
  gtk_widget_realize(test_window);
  place_window(test_window,prefs.sub_wnd_x,prefs.sub_wnd_y,prefs.sub_wnd_geom);
}
