/**************************************************
 * RCSId: $Id$
 *
 * Main program.
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log$
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 R. Alblas. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/

#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>
#include <string.h>

#include "wsat.h"
#include "menu_wsat.h"

/* Preferences */
PREFS prefs;

/* Define version number using -DRELEASE=\"...\" in makefile. */
#ifndef RELEASE
#define RELEASE "Undefined"
#endif

#if ENABLE_DEBUG == 1
extern int __sgtkdebug__;
#endif



/*************************************
 * Activate recording directly
 *************************************/
static gboolean sched_recordnow(gpointer data)
{
  GtkWidget *widget=(GtkWidget *)data;
  RECORDINFO *rdi=Get_Recordinfo_db(widget);
#ifdef __ADD_PAR__
  char str[20],*p;
  strcpy(str,rdi->rec_chan);

  if ((p=strchr(str,':'))) { *p=0; p++; }

  if (!strcmp(str,"hri"))
  {
    if ((p) && (strchr(p,'g')))
      Recordpp_Hri_Auto(widget,TRUE);
    else
      Recordpp_Hri_Auto(widget,FALSE);
  }
  else if (!strcmp(str,"hrpt"))
  {
    Recordpp_Hrpt_Auto(widget,p,HRPT);
  }
  else if (!strcmp(str,"chrpt"))
  {
    Recordpp_Hrpt_Auto(widget,p,CHRPT);
  }
#endif
  return FALSE;
}

/*************************************
 * Activate slide show directly
 *************************************/
gboolean sched_slide(gpointer data)
{
  GtkWidget *widget=(GtkWidget *)data;
  do_slide(widget,TRUE);
  return FALSE;
}

#define CNVS_ID "Is_Canvas"
/*************************************
 * Update window; wait until finished
 *************************************/
void Update_Main_Wait(GtkWidget *widget)
{
  GtkWidget *window;
  GtkWidget *main_window,*canvas;
  PICINFO *pci;
  VIEWINFO *vwi;
  window=Find_Parent_Window(widget);
  if (!window) return;

  main_window=Find_Window(window,Lab_Main);
  canvas=Find_Widget(main_window,CNVS_ID);
  pci=Get_Picinfo_db(widget);
  vwi=Get_Viewinfo_db(widget);

  vwi->redraw=TRUE;      /* Signal that a redraw is requested. */
/* Only update if enabled. */
  if (Get_Enable_Update(canvas))
  {
    vwi->redraw=FALSE;   /* Redraw accepted, remove signal */

    Set_Enable_Update(canvas,FALSE); /* Prevent re-drawing while drawing */
    if (!draw_pic(pci,vwi,main_window))                /* draw picture */
    {
      if (vwi->grid) draw_lonlat(pci,vwi,main_window); /* draw grid */
    }

    Set_Enable_Update(canvas,TRUE);

/* If redraw was requested (during draw_pic) then honnor it now. */
    if (vwi->redraw) 
    {
      vwi->redraw=FALSE;
      Update_Main(widget);
    }
  }
  Update_Lum(widget,pci,vwi);
}


static gboolean sched_update(gpointer data)
{
  GtkWidget *widget=(GtkWidget *)data;
  Update_Main_Wait(widget);
  return FALSE;
}


/*************************************
 * Update window (scheduled)
 *************************************/
void Update_Main(GtkWidget *widget)
{
  g_idle_add(sched_update,widget);
}

/*************************************
 * Executed if window size is changed 
 *   -> picture is re-generated
 *************************************/
gint configure_event_main(GtkWidget         *widget,
                          GdkEventConfigure *event)
{
  GtkWidget *window=Find_Parent_Window(widget);
  Renew_RGBBuf(widget);
  Update_Main(widget);
  return TRUE;
}

gint configure_event_mainwt(GtkWidget         *widget,
                          GdkEventConfigure *event)
{
  GtkWidget *window=Find_Parent_Window(widget);
  Renew_RGBBuf(window);
  Update_Main_Wait(widget);
  return TRUE;
}


void Menu_Help(GtkWidget *widget,gpointer data)
{
  char cpr='c';
#if __GTK_WIN32__ == 0
/*Doesn't work with some fonts!
  cpr=GDK_copyright;
*/
#endif
  Create_Message("Program info","wsat release %s\n%c R. Alblas\nwerkgroep Kunstmanen\nNederland\n",RELEASE,cpr);
}

void usage(char *prog)
{
  fprintf(stderr,"**************************************************************\n");
  fprintf(stderr,"* Weather satellite program for HRPT/CHRPT/HRI.\n");
  fprintf(stderr,"* Release: %s\n",RELEASE);
  fprintf(stderr,"* %c R. ALblas, werkgroep Kunstmanen\n",GDK_copyright);
  fprintf(stderr,"*-------------------------------------------------------------\n");
  fprintf(stderr,"* Usage:\n");
  fprintf(stderr,"*        %s <options>\n",prog);
  fprintf(stderr,"*        <options>:\n");
  fprintf(stderr,"*          -i <file>           read <file>\n");
  fprintf(stderr,"*          -sn                 south-north (flip hor and ver)\n");
  fprintf(stderr,"*          -w <width>          Window width (default: %d)\n",
                                                                DEFAULT_WIDTH);
  fprintf(stderr,"*          -h <height>         Window height (default: %d)\n",
                                                                DEFAULT_HEIGHT);
  
  fprintf(stderr,"*          -record <sat:chan>  Auto-record; exit program if done\n");
  fprintf(stderr,"*            <sat:chan>        specify satellite and channels\n");
  fprintf(stderr,"*            <sat>             hrpt | chrpt | hri\n");
  fprintf(stderr,"*            <chan>            1-5, a, g (hrpt)\n");
  fprintf(stderr,"*                              1-0, a, g (chrpt) (0=channel 10)\n");
  fprintf(stderr,"*                              g (hri)\n");
  fprintf(stderr,"*                              a=all channels\n");
  fprintf(stderr,"*                              g=generator mode (only usefull for test)\n");
  fprintf(stderr,"*\n");
  fprintf(stderr,"*          -crecord <sat:chan> Same as -record, but record 'forever'\n");
  fprintf(stderr,"*\n");
  fprintf(stderr,"*          -batch <sat:chan>   Batch-record; exit program if done.\n");
  fprintf(stderr,"*                              Same as -record, except no graphical mode.\n");
  fprintf(stderr,"*\n");
  fprintf(stderr,"*          -cbatch <sat:chan>  Same as -batch, but record 'forever'\n");
  fprintf(stderr,"*\n");
  fprintf(stderr,"*          -log                batch messages in wsat.log in\n");
  fprintf(stderr,"*                              home dir. instead of shell\n");
  fprintf(stderr,"*Examples:\n");
  fprintf(stderr,"*         wsat                     start wsat\n");
  fprintf(stderr,"*         wsat ned.gif             show pic in file 'ned.gif'\n");
  fprintf(stderr,"*         wsat -record hrpt:124    record HRPT channels 1/2/4\n");
  fprintf(stderr,"*         wsat -record chrpt:1290  record CHRPT channels 1/2/9/10\n");
  fprintf(stderr,"*         wsat -record hrpt:a      record HRPT, all channels\n");
  fprintf(stderr,"*         wsat -record hri         record HRI\n");
  fprintf(stderr,"*         wsat -record hrpt:ag     record HRPT generator mode\n");
  fprintf(stderr,"*         wsat -batch hri          record HRI, batch-mode\n");
  fprintf(stderr,"**************************************************************\n");
}


int main(int argc, char **argv)
{
  GtkWidget *main_window,*menu,*info,*canvas;
  GtkWidget *position,*pixval,*gammaval,*satname,*wz;
  PICINFO *pci;
  VIEWINFO *vwi;
  RECORDINFO *rdi;
  gboolean load_all=FALSE;
  int i;
  int w,h;
  int scan_ver=0;

#if ENABLE_DEBUG == 1
  __sgtkdebug__=0;
#endif

/* Allocate mem for global structs */
  pci=(PICINFO *)calloc(sizeof(PICINFO),1);
  vwi=(VIEWINFO *)calloc(sizeof(VIEWINFO),1);
  rdi=(RECORDINFO *)calloc(sizeof(RECORDINFO),1);

/* Get command line arguments */
  gtk_init(&argc, &argv);
  gdk_rgb_init();

  *rdi->rec_chan=0;

/* Read preferences */
  Read_Prefs(argv[0]);
  Load_Prefs(rdi,vwi,pci);

  set_fontsize(prefs.font_size);

  for (i=1; i<argc; i++)
  {
#if ENABLE_DEBUG == 1
    if (!strcmp(argv[i],"-dbg")) { __sgtkdebug__=1; continue; }
#endif
    if ((!strcmp(argv[i-1],"-i")) || (!strcmp(argv[i-1],"-I")))
    {
      strcpy(pci->file_name,argv[i]);
      if (!strcmp(argv[i-1],"-I"))
      {
/* Dit levert nu dubbele scrolbar; moet eigenlijk gewoon weg! */
        load_all=TRUE;
      }
    }
    if (!strcmp(argv[i-1],"-w"))
      prefs.main_wnd_w=atoi(argv[i]);
    if (!strcmp(argv[i-1],"-h"))
      prefs.main_wnd_h=atoi(argv[i]);
    if (!strcmp(argv[i],"-H"))
    {
      usage(argv[0]);
      exit(0);
    }
    if (!strcmp(argv[i],"--help"))
    {
      usage(argv[0]);
      exit(0);
    }
    if (!strcmp(argv[i],"-V"))
    {
      fprintf(stderr,"* Version: %s\n",RELEASE);
      exit(0);
    }

    if (!strcmp(argv[i],"-sn"))
    {
      scan_ver=S_N;
    }

    
/* auto-rec */
    if (!strcmp(argv[i-1],"-record"))
    {
      strcpy(rdi->rec_chan,argv[i]);
      rdi->one_record=TRUE;
    }

    if (!strcmp(argv[i-1],"-crecord"))
    {
      strcpy(rdi->rec_chan,argv[i]);
      rdi->one_record=FALSE;
    }
    
/* batch-mode */
    if (!strcmp(argv[i-1],"-batch"))
    {
      strcpy(rdi->rec_chan,argv[i]);
      rdi->one_record=TRUE;
      rdi->batch_mode=TRUE;
    }

    if (!strcmp(argv[i-1],"-cbatch"))
    {
      strcpy(rdi->rec_chan,argv[i]);
      rdi->one_record=FALSE;
      rdi->batch_mode=TRUE;
    }

    if (!strcmp(argv[i],"-log"))
    {
      prefs.log=TRUE;
    }
/* slide show */
    if (!strcmp(argv[i-1],"-slide"))
    {
      vwi->slide_mode=TRUE;
    }
    if (!strcmp(argv[i-1],"-speed"))
    {
      vwi->slide_speed=atoi(argv[i]);
    }
  }
                                                                   
/* Do batch-mode */
  if (rdi->batch_mode)
  {
#ifdef __ADD_PAR__
    do_batch(rdi,pci,vwi);
#endif
    return 0;
  }

  if (!*(pci->file_name) && (argc>1) && (*argv[argc-2] !='-') && (*argv[argc-1] !='-'))
  {
    strcpy(pci->file_name,argv[argc-1]);
  }

  if (*(pci->file_name))
  {
    int err;
    if ((err=open_file(pci,vwi,NULL)))
    {
      switch(err)
      {
        case 'o': 
          fprintf(stderr,"ERROR: Can't open %s\n",pci->file_name);
        break;
        case 't': 
          fprintf(stderr,"ERROR: %s is unknown file type.\n",pci->file_name);
        break;
        default:
          fprintf(stderr,"ERROR: Problem with reading %s (code %c = %d).\n",
              pci->file_name,err,err);
        break;
      }
      free(pci);
      free(vwi);
      free(rdi);
      exit(1);
    }

    if (load_all)
    {
      prefs.canvas_w=pci->width;
      prefs.canvas_h=pci->height;
      prefs.main_wnd_w=gdk_screen_width();
      prefs.main_wnd_h=gdk_screen_height()-30;
    }
  }

  if (scan_ver) vwi->scan_ver=scan_ver;
  w=prefs.main_wnd_w;
  h=prefs.main_wnd_h;

  if ((!w) && (!h))
  {
    w=-1;
    h=-1;
  }

/* Define the main window */

  main_window=Create_Window(NULL,
                            w,h,
                            Lab_Main,gtk_main_quit);
  if (!main_window) fatal_error();

/* Add database to main window */
  gtk_object_set_data(GTK_OBJECT(main_window),WSAT_PCI,(gpointer)pci);
  gtk_object_set_data(GTK_OBJECT(main_window),WSAT_VWI,(gpointer)vwi);
  gtk_object_set_data(GTK_OBJECT(main_window),WSAT_RDI,(gpointer)rdi);

/* Create the menu bar */
  menu=Create_Menu(main_window,MENU_FILE           ,NULL          ,BUTTON,
                                 MENU_OPEN         ,MenuSelectFile,BUTTON,
                                 MENU_CLOSE        ,close_file    ,BUTTON,
                                 MENU_SAVE         ,MenuSave      ,BUTTON,
                                 MENU_INFO         ,Menu_Info     ,BUTTON,
                                 MENU_LDKEPLER     ,Menu_LDKepler ,BUTTON,
                                 MENU_SEPAR        ,NULL          ,BUTTON,
                                 MENU_QUIT         ,Close_Window  ,BUTTON,

                               MENU_EDIT           ,NULL          ,BUTTON,
                                 MENU_FHDR         ,Menu_Hdredt   ,BUTTON,
                                 MENU_KEPLER       ,Menu_orbitedt ,BUTTON,
                                 MENU_SEPAR        ,NULL          ,BUTTON,
                                MENU_PREF          ,MenuPrefs     ,BUTTON,

                               MENU_VIEW           ,NULL          ,BUTTON,
                                 MENU_SATDIR       ,Menu_Viewbuts ,CHECK,
                                 MENU_GRIDADD      ,Menu_Viewbuts ,CHECK,
                                 MENU_POSXY        ,Menu_Viewbuts ,CHECK,
                                 MENU_INVERSE      ,Menu_Viewbuts ,CHECK,
                                 MENU_TEMP         ,Menu_Viewbuts ,CHECK,
                                 MENU_LIN          ,Menu_Viewbuts ,CHECK,
                                 MENU_DESPEC       ,Menu_Viewbuts ,CHECK,
                                 MENU_MARK_FLTS    ,Menu_Viewbuts ,CHECK,
                                 MENU_SKIP_FLTS    ,Menu_Viewbuts ,CHECK,
                                 MENU_SKIP_PP      ,NULL          ,BUTTON,
                                   MENU_SKIP_PRE   ,Menu_Viewbuts ,CHECK,
                                   MENU_SKIP_POST  ,Menu_Viewbuts ,CHECK,
                                 MENU_PALETTE      ,NULL          ,BUTTON,
                                   MENU_LD_PALETTE ,load_palette  ,RADIO,
                                   MENU_GR_PALETTE ,load_greyscale,RADIO,
                                 MENU_LUM          ,Menu_Lummap   ,BUTTON,
                                 MENU_CHMAP        ,Menu_Mapchan  ,BUTTON,
                                 MENU_FILM         ,Menu_Film     ,BUTTON,

                               MENU_ZOOM           ,NULL          ,BUTTON,
                                 MENU_FULL         ,Menu_Zoombuts ,BUTTON,
                                 MENU_ZMIN         ,Menu_Zoombuts ,BUTTON,
                                 MENU_ZMOUT        ,Menu_Zoombuts ,BUTTON,
                                 MENU_ZMVAR        ,Menu_Zoombuts ,BUTTON,
                                 MENU_ZMFIX        ,Menu_Zoombuts ,BUTTON,
                                 MENU_ZMLAST       ,Menu_Zoombuts ,BUTTON,
                                 MENU_ZMDXY        ,Menu_Zoombuts ,BUTTON,
                                 MENU_ZMDYX        ,Menu_Zoombuts ,BUTTON,
                                 MENU_ZMXY         ,Menu_Zoombuts ,BUTTON,
                                 MENU_ZMYX         ,Menu_Zoombuts ,BUTTON,
                                 MENU_ZVAL         ,Menu_Zoom     ,BUTTON,

                               MENU_LUMTHR         ,Menu_Lumthr   ,BUTTON,

                               MENU_REC            ,NULL          ,BUTTON,
#if _APT_
                                 MENU_APT          ,NULL          ,BUTTON,
#endif
#if _HRPT1_
                                 MENU_HRPT         ,Menu_Rec_Hrpt ,BUTTON,
#endif
#if _WEFAX_
                                 MENU_WEFAX        ,NULL          ,BUTTON,
#endif
#ifdef __ADD_PAR__
#if _HRI1_
                                 MENU_HRI          ,Menu_Rec_Hri  ,BUTTON,
#endif
#if _RAWDT_
                                 MENU_RAWD         ,Menu_Rec_Rawd ,BUTTON,
#endif
#endif
                               MENU_CHAN           ,NULL          ,BUTTON,
                                 MENU_CH1          ,Menu_Channel  ,RADIO,
                                 MENU_CH2          ,Menu_Channel  ,RADIO,
                                 MENU_CH3          ,Menu_Channel  ,RADIO,
                                 MENU_CH4          ,Menu_Channel  ,RADIO,
                                 MENU_CH5          ,Menu_Channel  ,RADIO,
                                 MENU_CH6          ,Menu_Channel  ,RADIO,
                                 MENU_CH7          ,Menu_Channel  ,RADIO,
                                 MENU_CH8          ,Menu_Channel  ,RADIO,
                                 MENU_CH9          ,Menu_Channel  ,RADIO,
                                 MENU_CH10         ,Menu_Channel  ,RADIO,

                               MENU_HELP           ,NULL          ,BUTTON,
                                 MENU_PINFO        ,Menu_Help     ,BUTTON,
#ifdef __ADD_PAR__
                                 MENU_TEST         ,Menu_Test     ,BUTTON,
#endif

                               0);

/* Create info bar */
  position=Create_Entry(LAB_COORD,NULL,"x=%5.2f,y=%5.2f",0.,0.);
  pixval=Create_Entry(LAB_PIXVAL,NULL,"%8d",0);
  gammaval=Create_Spin(LAB_GAMMAVAL,NULL,"%8.1f%f%f",1.,0.2,5.);
  satname=Create_Entry(LAB_SATNM,NULL,"%10s","");
  info=Pack(NULL,'h',position,3,pixval,3,gammaval,3,satname,0,NULL);

/* Create the canvas */
  canvas=Create_Canvas(main_window,
                prefs.canvas_w,prefs.canvas_h,
                configure_event_main,
                key_actions,mouse_actions,mouse_actions);

gtk_object_set_data(GTK_OBJECT(canvas),WDGT_ID,CNVS_ID);

/* Add scrolbars */
  canvas=Add_Scrollbars(canvas,"hbar","vbar",scroll_actions);

/* Collect all parts */
  wz=SPack(NULL,"v",menu,"1",info,"1",canvas,"ef1",NULL);
  gtk_container_add(GTK_CONTAINER (main_window),wz);
  place_window(main_window,
               prefs.main_wnd_x,prefs.main_wnd_y,
               prefs.main_wnd_geom);

/* Load palette:
     If file specified: palette in file
     If no file specified: grey palette 
*/
  if (pci->fp==NULL)
    load_std_palette(main_window,NULL);
  else
    load_std_palette(main_window,pci);

  load_extra_palette(main_window);

/* Set menu items */
  Channel_Select(main_window,pci,vwi);

  init_menubuts(main_window,pci,vwi);
  Set_Entry(main_window,LAB_SATNM,"%s",pci->sat_name);
/*
  Enable window update. 
  It was disabled to prevent re-drawing picture several times because
  of channel select etc.
*/
  Set_Enable_Update(canvas,TRUE);

  gtk_widget_show_all(main_window);

/* Handle auto-record */
  if (*rdi->rec_chan)
  {
    g_idle_add(sched_recordnow,main_window);
  }

  if (vwi->slide_mode)
    g_idle_add(sched_slide,main_window);

  Set_Button(main_window,MENU_SKIP_POST,TRUE);

  gtk_main();

  free(pci);
  free(vwi);
  free(rdi);
  return 0;
}

