/**************************************************
 * RCSId: $Id$
 *
 * Zoom related functions
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log$
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 R. Alblas. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/

#include <stdlib.h>

#include "wsat.h"
#include "menu_wsat.h"


/* Define button names */
#define LAB_ZX "Zx"
#define LAB_ZY "Zy"
#define LAB_OX "Ox"
#define LAB_OY "Oy"
#define LAB_DX "Dx"
#define LAB_DY "Dy"
#define LAB_Close   " Close "
#define LAB_Restore "Restore"
#define LAB_Redraw "Redraw"
#define Lab_Zoomwnd "Zoom"

extern PREFS prefs;

ZOOMVALS zoom_full()
{
  ZOOMVALS zval;
  zval.zx=1.;
  zval.zy=1.;
  zval.ox=0;
  zval.oy=0;
  return zval;
}

/*************************************
 * Update text of zoom buttons 
 *************************************/
void Update_Zoom(GtkWidget *window)
{
  GtkWidget *zoom_window=Find_Window(window,Lab_Zoomwnd);
  GtkWidget *main_window=Find_Window(window,Lab_Main);
  VIEWINFO *vwi=Get_Viewinfo_db(main_window);
  PICINFO *pci=Get_Picinfo_db(main_window);

  if ((zoom_window) && (vwi))
  {
    Set_Adjust(zoom_window,LAB_ZX,"%f",vwi->zval.zx);
    Set_Adjust(zoom_window,LAB_ZY,"%f",vwi->zval.zy);
    Set_Adjust(zoom_window,LAB_OX,"%d",vwi->zval.ox);
    Set_Adjust(zoom_window,LAB_OY,"%d",vwi->zval.oy);
    Set_Adjust(zoom_window,LAB_DX,"%f",vwi->zval.dx);
    Set_Adjust(zoom_window,LAB_DY,"%f",vwi->zval.dy);
    Set_Adjustbnd(zoom_window,LAB_OX,"%?%d",pci->width-1);
    Set_Adjustbnd(zoom_window,LAB_OY,"%?%d",pci->height-1);
  }
}

/*************************************
 * Update entered zoom values  
 * Connected to zoom window.
 *************************************/
void zoom_valcmds(GtkWidget *widget, gpointer data)
{
  GtkWidget *main_window=Find_Window(Find_Parent_Window(widget),Lab_Main);
  GtkWidget *drawing_area=Find_Widget(main_window,"GTK_DRAWING_AREA");
  GtkWidget *zoom_window=Find_Window(main_window,Lab_Zoomwnd);
  VIEWINFO *vwi=Get_Viewinfo_db(widget);
  PICINFO *pci=Get_Picinfo_db(widget);
  GdkEventKey *event=(GdkEventKey *)data;
  char *name=(char *)data;
  static gboolean start=TRUE;
  static gboolean updated;
  int wnd_width=drawing_area->allocation.width;  /* width window */
  int wnd_height=drawing_area->allocation.height; /* height window */

  static ZOOMVALS zvals_pre;

/* Save old zoom values at start */
  if (start)
  {
    zvals_pre=vwi->zval;
    updated=FALSE;
    start=FALSE;
  }

/* Take over entered zoom values */
  if (!strcmp(name,LAB_ZX))
  {
    vwi->zval.zx=GTK_ADJUSTMENT(widget)->value;
    vwi->zval.dx=vwi->zval.zx*(float)wnd_width/(float)pci->width;
    Update_Zoom(zoom_window);
  }
  if (!strcmp(name,LAB_ZY))
  {
    vwi->zval.zy=GTK_ADJUSTMENT(widget)->value;
    vwi->zval.dy=vwi->zval.zy*(float)wnd_height/(float)pci->height;
    Update_Zoom(zoom_window);
  }
  if (!strcmp(name,LAB_OX))
  {
    vwi->zval.ox=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_OY))
  {
    vwi->zval.oy=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_DX))
  {
    vwi->zval.dx=GTK_ADJUSTMENT(widget)->value;
    vwi->zval.zx=vwi->zval.dx/((float)wnd_width/(float)pci->width);
    Update_Zoom(zoom_window);
  }
  if (!strcmp(name,LAB_DY))
  {
    vwi->zval.dy=GTK_ADJUSTMENT(widget)->value;
    vwi->zval.zy=vwi->zval.dy/((float)wnd_height/(float)pci->height);
    Update_Zoom(zoom_window);
  }

/* Update window if 'enter' is hit */
  if ((guchar)event->keyval==(GDK_Return&0xff))
  {
    Update_Main(zoom_window);
    updated=TRUE;
  }

/* Redraw picture */
  if (!strcmp(name,LAB_Redraw))
  {
    Update_Zoom(zoom_window);
    Update_Main(zoom_window);
    updated=TRUE;
  }

/* Exit zoom-window */
  if (!strcmp(name,LAB_Close))
  {
    start=TRUE;

/* Update zoom buttons with current values */
    Update_Zoom(zoom_window);

    Close_Window(zoom_window);
  }

/* Cancel: recover previous zoom values */
  if (!strcmp(name,LAB_Restore))
  {
    vwi->zval=zvals_pre;
    start=TRUE;
    if (updated)
    {
      Update_Main(zoom_window);
    }

/* Update zoom buttons with current values */
    Update_Zoom(zoom_window);
  }
}


/*************************************
 * Create zoom window
 * Activated by menu: View->Zoom
 *************************************/
void Menu_Zoom(GtkWidget *widget)
{
  GtkWidget *w1,*w2,*w3;
  GtkWidget *wa,*wb,*wz;
  GtkWidget *zoom_window;
  GtkWidget *main_window;
  VIEWINFO *vwi=Get_Viewinfo_db(widget);
  PICINFO *pci=Get_Picinfo_db(widget);
  main_window=Find_Parent_Window(widget);

/* Create new window or pop-up it if it already exist. */
  if (!(zoom_window=Create_Window(main_window,0,0,Lab_Zoomwnd,NULL))) return;

  gtk_signal_connect(GTK_OBJECT(zoom_window), "key_press_event",
		      GTK_SIGNAL_FUNC(zoom_valcmds), NULL);

/* Add buttons */
  wa=Create_ButtonArray("Zoom",zoom_valcmds,2,
                    SPIN,LAB_ZX,"%f%f%f",vwi->zval.zx,1.,1000.,
                    SPIN,LAB_ZY,"%f%f%f",vwi->zval.zy,1.,1000.,
                    SPIN,LAB_OX,"%d%d%d",vwi->zval.ox,0,pci->width-1,
                    SPIN,LAB_OY,"%d%d%d",vwi->zval.oy,0,pci->height-1,
                    SPIN,LAB_DX,"%f%f%f",vwi->zval.dx,0.1,100.,
                    SPIN,LAB_DY,"%f%f%f",vwi->zval.dy,0.1,100.,
                    0
                   );

  w1=Create_Button(LAB_Close,zoom_valcmds);
  w2=Create_Button(LAB_Restore,zoom_valcmds);
  w3=Create_Button(LAB_Redraw,zoom_valcmds);
  wb=SPack("","h",w1,"5ef",w2,"5ef",w3,"5ef",NULL);
  wz=Pack(NULL,'v',wa,5,wb,5,NULL);

  gtk_container_add(GTK_CONTAINER(zoom_window),wz);
  gtk_widget_show(zoom_window);
  place_window(zoom_window,prefs.sub_wnd_x,prefs.sub_wnd_y,prefs.sub_wnd_geom);
}



/* Check and correct zoom values */
void correct_zoomval(GtkWidget *drawing_area,VIEWINFO *vwi,PICINFO *pci)
{
  int wnd_width=drawing_area->allocation.width;  /* width window */
  int wnd_height=drawing_area->allocation.height; /* height window */
  vwi->zval.zx=MAX(vwi->zval.zx,1);
  vwi->zval.zy=MAX(vwi->zval.zy,1);
  vwi->zval.ox=MAX(vwi->zval.ox,0);
  vwi->zval.ox=MIN(vwi->zval.ox,(pci->width-pci->width/vwi->zval.zx));
  vwi->zval.oy=MAX(vwi->zval.oy,0);
  vwi->zval.oy=MIN(vwi->zval.oy,(pci->height-pci->height/vwi->zval.zy));
  vwi->zval.dx=vwi->zval.zx*((float)wnd_width/(float)pci->width);
  vwi->zval.dy=vwi->zval.zy*((float)wnd_height/(float)pci->height);
}

