/**************************************************
 * RCSId: $Id$
 *
 * PPM related routines
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log$
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 R. Alblas. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "wsat.h"
#include "ppm.h"
/*************************************
 * Read PPM format header
   into PICINFO struct.
 * Return value:
 *   0 if OK
 *   !0 if error
 *************************************/
int read_ppm_header(PICINFO *pci)
{
  FILE *fp=pci->fp;
  char tmp[100],*p;
  int c,i;
  int wpp;

  fread(tmp,2,1,fp);
/* ---- Check sign ---- */
  if (!memcmp(tmp,PPM_SIGN,strlen(PPM_SIGN)))      wpp=3;
  else if (!memcmp(tmp,PGM_SIGN,strlen(PGM_SIGN))) wpp=1;
  else                                             return 1;

/* ---- Get comment ---- */
  while ((c=fgetc(fp)) && (c!=0x0a))       /* goto new line */
    if (c==EOF) return 2;

  while ((c=fgetc(fp)) && (c==' '));       /* skip spaces */
  fseek(fp,-1,SEEK_CUR);

  while ((c=fgetc(fp)) && (c=='#'))        /* detect comment */
  {
    char n=0;
    while ((c=fgetc(fp)) && (c!=0x0a))
    {
      if (c==EOF) return 2;
      pci->sat_name[n]=c;
      if (n<LENSATNAME-1) n++;
      pci->sat_name[n]=0;
    }
  }

  fseek(fp,-1,SEEK_CUR);

/* ---- get size ---- */

  p=tmp;
  while ((c=fgetc(fp)) && (c!=0x0a))
  {
    if (c==EOF) return 2;
    *p=c; p++;        
    if (p-tmp >= 99) return 3;
  }
  *p=0;
  if (!(p=strtok(tmp," "))) return 4; pci->width=atoi(p);
  if (!(p=strtok(NULL," "))) return 5; pci->height=atoi(p);

/* get max. value; goto end header */
  p=tmp;
  while ((c=fgetc(fp)) && (c!=0x0a))
  {
    if (c==EOF) return 2;
    *p=c; p++;        
    if (p-tmp >= 99) return 3;
  }
  *p=0;
  if (!(p=strtok(tmp," "))) return 4;

/* Copy color map into picinfo structure */
  pci->clrmap.size=atoi(p) + 1;
  pci->clrmap.size=MIN(pci->clrmap.size,MAX_PCICLR);

/* Calculate depth */
  for (i=8; i<=MAX_PCICLR; i++)
  {
    if ((1<<i) >= pci->clrmap.size)
    {
      pci->depth=i;
      break;
    }
  }

/* Set remaining pci stuff */
  pci->fri.words_per_line=pci->width*wpp;
  pci->fri.bytes_per_line=pci->width*wpp;
  if (pci->depth>8) pci->fri.bytes_per_line*=2;

  for (i=0; i<NR_CHANNELS_MAX; i++)
    pci->channels_recorded[i]=FALSE;
  pci->channels_recorded[1]=TRUE;

  pci->fri.inter_chan=1;
  pci->pic_strt=ftell(fp);

  pci->fri.header_len=0;
  pci->scan_hor=W_E;
  pci->scan_ver=N_S;


  for (i=0; i<pci->clrmap.size; i++)
  {
    pci->clrmap.red[i]=i;
    pci->clrmap.green[i]=i;
    pci->clrmap.blue[i]=i;
  }
  return 0;
}

/*************************************
 * Write PPM format header
   from PICINFO struct.
 * Return value:
 *   0 if OK
 *   !0 if error
 *************************************/
int write_ppm_header(PICINFO *pci)
{
  char hdr[30];
  int n=0;

/* sign */
  if (pci->file_type==PPM)
  {
    sprintf(hdr,"%s%c",PPM_SIGN,0xa);
  }
  else /* PGM */
  {
    sprintf(hdr,"%s%c",PGM_SIGN,0xa);
  }
  n+=fwrite(hdr,1,strlen(hdr),pci->fp);

/* comment */
  sprintf(hdr,"#%s%c","Created by wsat",0xa);
  n+=fwrite(hdr,1,strlen(hdr),pci->fp);

/* size */
  sprintf(hdr,"%d %d%c",pci->width,pci->height,0xa);
  n+=fwrite(hdr,1,strlen(hdr),pci->fp);

/* BPP */
  sprintf(hdr,"%d%c",pci->clrmap.size-1,0xa);
  n+=fwrite(hdr,1,strlen(hdr),pci->fp);

  return n;
}

/*************************************
 * Read line 
 * Return value:
 *   Amount of written bytes (=width picture)
 *   0 if error
 *************************************/
int read_ppm_line(PICINFO *pci,int y,
                   guint16 *line_r,guint16 *line_g,guint16 *line_b)
{
  FILE *fp=pci->fp;
  guchar rgb[3];
  guint16 pix;
  int ret=0,i;

  fseek(fp,pci->pic_strt + y*pci->fri.bytes_per_line,SEEK_SET);

  for (i=0; i<pci->fri.words_per_line; i++)
  {
    if (pci->file_type==PPM)
    {
      ret+=fread(&rgb,3,1,fp);
      *(line_r+i)=rgb[0];
      if (line_g) *(line_g+i)=rgb[1];
      if (line_b) *(line_b+i)=rgb[2];
    }
    else if (pci->file_type==PGM)
    {
      if (pci->depth<=8)
      {
        ret+=fread(&rgb,1,1,fp);
        *(line_r+i)=rgb[0];
        if (line_g) *(line_g+i)=rgb[0];
        if (line_b) *(line_b+i)=rgb[0];
      }
      else
      {
        ret+=fread(&pix,2,1,fp);
        pix=GUINT16_FROM_BE(pix);
        pix=MIN(pix,MAX_PCICLR-1);
        *(line_r+i)=pix;
        if (line_g) *(line_g+i)=pix;
        if (line_b) *(line_b+i)=pix;
      }
    }
  }
  return ret;
}

/*************************************
 * Write line 
 * Return value:
 *   Amount of written bytes (=width picture)
 *************************************/
int write_ppm_line(PICINFO *pci,
                   guint16 *line_r,
                   guint16 *line_g,
                   guint16 *line_b)
{
  FILE *fp=pci->fp;
  int width=pci->fri.bytes_per_line;
  char rgb[3];
  guint8 byte;
  guint16 word;
  int ret=0,i;
  for (i=0; i<width; i++)
  {
    if (pci->file_type==PPM)
    {
      rgb[0]=*(line_r+i);
      rgb[1]=*(line_g+i);
      rgb[2]=*(line_b+i);
      ret+=fwrite(&rgb,3,1,fp);
    }
    else
    {
      if (pci->depth<=8)
      {
        byte=*(line_r+i);
        ret+=fwrite(&byte,1,1,fp);
      }
      else
      {
        word=GUINT16_TO_BE(*(line_r+i));
        ret+=fwrite(&word,2,1,fp);
      }
    }
  }
  return ret;
}
